@@ -15,7 +15,7 @@ import (
 	"github.com/rqlite/rqlite/v8/auth"
 	"github.com/rqlite/rqlite/v8/cluster/proto"
 	command "github.com/rqlite/rqlite/v8/command/proto"
-	"github.com/rqlite/rqlite/v8/rtls"
+	"github.com/rqlite/rqlite/v8/internal/rtls"
 	"github.com/rqlite/rqlite/v8/tcp"
 	"github.com/rqlite/rqlite/v8/tcp/pool"
 	pb "google.golang.org/protobuf/proto"
@@ -30,12 +30,21 @@ const (
 )
 
 // CreateRaftDialer creates a dialer for connecting to other nodes' Raft service. If the cert and
-// key arguments are not set, then the returned dialer will not use TLS.
+// key arguments are not set, then the returned dialer will not use TLS. If they are set then
+// the dialer will use TLS. A started CertMonitor will also be returned. The caller is responsible
+// for stopping the CertMonitor when the Dialer is no longer needed. The serverName argument is used
+// to validate the server certificate. If Insecure is true, then the dialer will not validate the
+// server certificate.
 func CreateRaftDialer(cert, key, caCert, serverName string, Insecure bool) (*tcp.Dialer, error) {
 	var dialerTLSConfig *tls.Config
 	var err error
+	var cr *rtls.CertReloader
 	if cert != "" || key != "" {
-		dialerTLSConfig, err = rtls.CreateClientConfig(cert, key, caCert, serverName, Insecure)
+		cr, err = rtls.NewCertReloader(cert, key)
+		if err != nil {
+			return nil, fmt.Errorf("failed to create TLS config for Raft dialer: %s", err.Error())
+		}
+		dialerTLSConfig, err = rtls.CreateClientConfigWithFunc(cr.GetCertificate, caCert, serverName, Insecure)
 		if err != nil {
 			return nil, fmt.Errorf("failed to create TLS config for Raft dialer: %s", err.Error())
 		}
@@ -169,10 +178,10 @@ func (c *Client) GetCommitIndex(nodeAddr string, retries int, timeout time.Durat
 	return a.CommitIndex, nil
 }
 
-// Execute performs an Execute on a remote node. If username is an empty string
+// Execute performs an Execute on a remote node. If creds is nil, then
 // no credential information will be included in the Execute request to the
 // remote node.
-func (c *Client) Execute(er *command.ExecuteRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration, retries int) ([]*command.ExecuteQueryResponse, error) {
+func (c *Client) Execute(er *command.ExecuteRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration, retries int) ([]*command.ExecuteQueryResponse, uint64, error) {
 	command := &proto.Command{
 		Type: proto.Command_COMMAND_TYPE_EXECUTE,
 		Request: &proto.Command_ExecuteRequest{
@@ -183,23 +192,25 @@ func (c *Client) Execute(er *command.ExecuteRequest, nodeAddr string, creds *pro
 	p, nr, err := c.retry(command, nodeAddr, timeout, retries)
 	stats.Add(numClientExecuteRetries, int64(nr))
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	a := &proto.CommandExecuteResponse{}
 	err = pb.Unmarshal(p, a)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	if a.Error != "" {
-		return nil, errors.New(a.Error)
+		return nil, 0, errors.New(a.Error)
 	}
-	return a.Response, nil
+	return a.Response, a.RaftIndex, nil
 }
 
-// Query performs a Query on a remote node.
-func (c *Client) Query(qr *command.QueryRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration) ([]*command.QueryRows, error) {
+// Query performs a Query on a remote node. If creds is nil, then
+// no credential information will be included in the Query request to the
+// remote node.
+func (c *Client) Query(qr *command.QueryRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration) ([]*command.QueryRows, uint64, error) {
 	command := &proto.Command{
 		Type: proto.Command_COMMAND_TYPE_QUERY,
 		Request: &proto.Command_QueryRequest{
@@ -210,23 +221,25 @@ func (c *Client) Query(qr *command.QueryRequest, nodeAddr string, creds *proto.C
 	p, nr, err := c.retry(command, nodeAddr, timeout, defaultMaxRetries)
 	stats.Add(numClientQueryRetries, int64(nr))
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	a := &proto.CommandQueryResponse{}
 	err = pb.Unmarshal(p, a)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	if a.Error != "" {
-		return nil, errors.New(a.Error)
+		return nil, 0, errors.New(a.Error)
 	}
-	return a.Rows, nil
+	return a.Rows, a.RaftIndex, nil
 }
 
-// Request performs an ExecuteQuery on a remote node.
-func (c *Client) Request(r *command.ExecuteQueryRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration, retries int) ([]*command.ExecuteQueryResponse, error) {
+// Request performs an ExecuteQuery on a remote node. If creds is nil, then
+// no credential information will be included in the ExecuteQuery request to the
+// remote node.
+func (c *Client) Request(r *command.ExecuteQueryRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration, retries int) ([]*command.ExecuteQueryResponse, uint64, error) {
 	command := &proto.Command{
 		Type: proto.Command_COMMAND_TYPE_REQUEST,
 		Request: &proto.Command_ExecuteQueryRequest{
@@ -237,22 +250,24 @@ func (c *Client) Request(r *command.ExecuteQueryRequest, nodeAddr string, creds
 	p, nr, err := c.retry(command, nodeAddr, timeout, retries)
 	stats.Add(numClientRequestRetries, int64(nr))
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	a := &proto.CommandRequestResponse{}
 	err = pb.Unmarshal(p, a)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	if a.Error != "" {
-		return nil, errors.New(a.Error)
+		return nil, 0, errors.New(a.Error)
 	}
-	return a.Response, nil
+	return a.Response, a.RaftIndex, nil
 }
 
-// Backup retrieves a backup from a remote node and writes to the io.Writer
+// Backup retrieves a backup from a remote node and writes to the io.Writer.
+// If creds is nil, then no credential information will be included in the
+// Backup request to the remote node.
 func (c *Client) Backup(br *command.BackupRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration, w io.Writer) error {
 	conn, err := c.dial(nodeAddr)
 	if err != nil {
@@ -305,7 +320,8 @@ func (c *Client) Backup(br *command.BackupRequest, nodeAddr string, creds *proto
 	return err
 }
 
-// Load loads a SQLite file into the database.
+// Load loads a SQLite file into the database. If creds is nil, then no
+// credential information will be included in the Load request to the remote node.
 func (c *Client) Load(lr *command.LoadRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration, retries int) error {
 	command := &proto.Command{
 		Type: proto.Command_COMMAND_TYPE_LOAD,
@@ -332,7 +348,9 @@ func (c *Client) Load(lr *command.LoadRequest, nodeAddr string, creds *proto.Cre
 	return nil
 }
 
-// RemoveNode removes a node from the cluster
+// RemoveNode removes a node from the cluster. If creds is nil, then no
+// credential information will be included in the RemoveNode request to the
+// remote node.
 func (c *Client) RemoveNode(rn *command.RemoveNodeRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration) error {
 	conn, err := c.dial(nodeAddr)
 	if err != nil {
@@ -371,7 +389,50 @@ func (c *Client) RemoveNode(rn *command.RemoveNodeRequest, nodeAddr string, cred
 	return nil
 }
 
+// Stepdown triggers leader stepdown on a remote node. If creds is nil, then no
+// credential information will be included in the Stepdown request to the
+// remote node.
+func (c *Client) Stepdown(sr *command.StepdownRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration) error {
+	conn, err := c.dial(nodeAddr)
+	if err != nil {
+		return err
+	}
+	defer conn.Close()
+
+	// Create the request.
+	command := &proto.Command{
+		Type: proto.Command_COMMAND_TYPE_STEPDOWN,
+		Request: &proto.Command_StepdownRequest{
+			StepdownRequest: sr,
+		},
+		Credentials: creds,
+	}
+	if err := writeCommand(conn, command, timeout); err != nil {
+		handleConnError(conn)
+		return err
+	}
+
+	p, err := readResponse(conn, timeout)
+	if err != nil {
+		handleConnError(conn)
+		return err
+	}
+
+	a := &proto.CommandStepdownResponse{}
+	err = pb.Unmarshal(p, a)
+	if err != nil {
+		return err
+	}
+
+	if a.Error != "" {
+		return errors.New(a.Error)
+	}
+	return nil
+}
+
 // Notify notifies a remote node that this node is ready to bootstrap.
+// If creds is nil, then no credential information will be included in
+// // the Notify request to the remote node.
 func (c *Client) Notify(nr *command.NotifyRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration) error {
 	conn, err := c.dial(nodeAddr)
 	if err != nil {
@@ -411,6 +472,8 @@ func (c *Client) Notify(nr *command.NotifyRequest, nodeAddr string, creds *proto
 }
 
 // Join joins this node to a cluster at the remote address nodeAddr.
+// If creds is nil, then no credential information will be included in
+// the Join request to the remote node.
 func (c *Client) Join(jr *command.JoinRequest, nodeAddr string, creds *proto.Credentials, timeout time.Duration) error {
 	for {
 		conn, err := c.dial(nodeAddr)
@@ -459,12 +522,121 @@ func (c *Client) Join(jr *command.JoinRequest, nodeAddr string, creds *proto.Cre
 	}
 }
 
+// BroadcastHWM performs a broadcast to all specified nodes.
+func (c *Client) BroadcastHWM(hwm uint64, retries int, timeout time.Duration, nodeAddr ...string) (map[string]*proto.HighwaterMarkUpdateResponse, error) {
+	if len(nodeAddr) == 0 {
+		return map[string]*proto.HighwaterMarkUpdateResponse{}, nil
+	}
+
+	// Get local node address for the broadcast request
+	c.localMu.RLock()
+	localAddr := c.localNodeAddr
+	c.localMu.RUnlock()
+
+	// Create the broadcast request
+	br := &proto.HighwaterMarkUpdateRequest{
+		NodeId:        localAddr,
+		HighwaterMark: hwm,
+	}
+
+	// Channel to collect results
+	type result struct {
+		resp *proto.HighwaterMarkUpdateResponse
+		addr string
+		err  error
+	}
+
+	resultChan := make(chan result, len(nodeAddr))
+
+	// Launch goroutines for parallel requests
+	for _, addr := range nodeAddr {
+		go func(nodeAddress string) {
+			// Create the command
+			command := &proto.Command{
+				Type: proto.Command_COMMAND_TYPE_HIGHWATER_MARK_UPDATE,
+				Request: &proto.Command_HighwaterMarkUpdateRequest{
+					HighwaterMarkUpdateRequest: br,
+				},
+			}
+
+			// Attempt with retries
+			var lastErr error
+			for attempt := 0; attempt <= retries; attempt++ {
+				conn, err := c.dial(nodeAddress)
+				if err != nil {
+					lastErr = err
+					continue
+				}
+
+				// Write command
+				if err := writeCommand(conn, command, timeout); err != nil {
+					conn.Close()
+					handleConnError(conn)
+					lastErr = err
+					continue
+				}
+
+				// Read response
+				p, err := readResponse(conn, timeout)
+				conn.Close()
+				if err != nil {
+					handleConnError(conn)
+					lastErr = err
+					continue
+				}
+
+				// Parse response
+				resp := &proto.HighwaterMarkUpdateResponse{}
+				if err := pb.Unmarshal(p, resp); err != nil {
+					lastErr = err
+					continue
+				}
+
+				// Success
+				resultChan <- result{resp: resp, addr: nodeAddress, err: nil}
+				return
+			}
+
+			// All retries failed
+			resultChan <- result{resp: nil, addr: nodeAddress, err: lastErr}
+		}(addr)
+	}
+
+	// Collect results with timeout
+	responses := make(map[string]*proto.HighwaterMarkUpdateResponse)
+	collected := 0
+
+	timeoutChan := time.After(timeout)
+
+	for collected < len(nodeAddr) {
+		select {
+		case res := <-resultChan:
+			if res.err != nil {
+				responses[res.addr] = &proto.HighwaterMarkUpdateResponse{Error: res.err.Error()}
+			} else {
+				responses[res.addr] = res.resp
+			}
+			collected++
+		case <-timeoutChan:
+			// Timeout reached, fill remaining responses with timeout errors
+			for _, addr := range nodeAddr {
+				if _, exists := responses[addr]; !exists {
+					responses[addr] = &proto.HighwaterMarkUpdateResponse{Error: "timeout"}
+				}
+			}
+			collected = len(nodeAddr)
+		}
+	}
+
+	return responses, nil
+}
+
 // Stats returns stats on the Client instance
-func (c *Client) Stats() (map[string]interface{}, error) {
+func (c *Client) Stats() (map[string]any, error) {
 	c.poolMu.RLock()
 	defer c.poolMu.RUnlock()
 
-	stats := map[string]interface{}{
+	stats := map[string]any{
 		"timeout":         c.timeout.String(),
 		"local_node_addr": c.localNodeAddr,
 	}
@@ -473,7 +645,7 @@ func (c *Client) Stats() (map[string]interface{}, error) {
 		return stats, nil
 	}
 
-	poolStats := make(map[string]interface{}, len(c.pools))
+	poolStats := make(map[string]any, len(c.pools))
 	for k, v := range c.pools {
 		s, err := v.Stats()
 		if err != nil {
