@@ -25,6 +25,7 @@ var (
 
 func Test_ServiceExecute(t *testing.T) {
 	ln, mux := mustNewMux()
+	defer mux.Close()
 	go mux.Serve()
 	tn := mux.Listen(1) // Could be any byte value.
 	db := mustNewMockDatabase()
@@ -42,13 +43,13 @@ func Test_ServiceExecute(t *testing.T) {
 	}
 
 	// Ready for Execute tests now.
-	db.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error) {
+	db.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
 		if er.Request.Statements[0].Sql != "some SQL" {
 			t.Fatalf("incorrect SQL statement received")
 		}
-		return nil, errors.New("execute failed")
+		return nil, 0, errors.New("execute failed")
 	}
-	_, err := c.Execute(executeRequestFromString("some SQL"), s.Addr(), NO_CREDS, longWait, defaultMaxRetries)
+	_, _, err := c.Execute(executeRequestFromString("some SQL"), s.Addr(), NO_CREDS, longWait, defaultMaxRetries)
 	if err == nil {
 		t.Fatalf("client failed to report error")
 	}
@@ -56,7 +57,7 @@ func Test_ServiceExecute(t *testing.T) {
 		t.Fatalf("incorrect error message received, got: %s", err.Error())
 	}
 
-	db.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error) {
+	db.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
 		if er.Request.Statements[0].Sql != "some SQL" {
 			t.Fatalf("incorrect SQL statement received")
 		}
@@ -68,9 +69,9 @@ func Test_ServiceExecute(t *testing.T) {
 				},
 			},
 		}
-		return []*command.ExecuteQueryResponse{response}, nil
+		return []*command.ExecuteQueryResponse{response}, 0, nil
 	}
-	res, err := c.Execute(executeRequestFromString("some SQL"), s.Addr(), NO_CREDS, longWait, defaultMaxRetries)
+	res, _, err := c.Execute(executeRequestFromString("some SQL"), s.Addr(), NO_CREDS, longWait, defaultMaxRetries)
 	if err != nil {
 		t.Fatalf("failed to execute query: %s", err.Error())
 	}
@@ -78,7 +79,7 @@ func Test_ServiceExecute(t *testing.T) {
 		t.Fatalf("unexpected results for execute, expected %s, got %s", exp, got)
 	}
 
-	db.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error) {
+	db.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
 		if er.Request.Statements[0].Sql != "some SQL" {
 			t.Fatalf("incorrect SQL statement received")
 		}
@@ -87,9 +88,9 @@ func Test_ServiceExecute(t *testing.T) {
 				Error: "no such table",
 			},
 		}
-		return []*command.ExecuteQueryResponse{response}, nil
+		return []*command.ExecuteQueryResponse{response}, 0, nil
 	}
-	res, err = c.Execute(executeRequestFromString("some SQL"), s.Addr(), NO_CREDS, longWait, defaultMaxRetries)
+	res, _, err = c.Execute(executeRequestFromString("some SQL"), s.Addr(), NO_CREDS, longWait, defaultMaxRetries)
 	if err != nil {
 		t.Fatalf("failed to execute: %s", err.Error())
 	}
@@ -97,11 +98,11 @@ func Test_ServiceExecute(t *testing.T) {
 		t.Fatalf("unexpected results for execute, expected %s, got %s", exp, got)
 	}
 
-	db.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error) {
+	db.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
 		time.Sleep(longWait)
-		return nil, nil
+		return nil, 0, nil
 	}
-	_, err = c.Execute(executeRequestFromString("some SQL"), s.Addr(), NO_CREDS, shortWait, defaultMaxRetries)
+	_, _, err = c.Execute(executeRequestFromString("some SQL"), s.Addr(), NO_CREDS, shortWait, defaultMaxRetries)
 	if err == nil {
 		t.Fatalf("failed to receive expected error")
 	}
@@ -120,6 +121,7 @@ func Test_ServiceExecute(t *testing.T) {
 
 func Test_ServiceQuery(t *testing.T) {
 	ln, mux := mustNewMux()
+	defer mux.Close()
 	go mux.Serve()
 	tn := mux.Listen(1) // Could be any byte value.
 	db := mustNewMockDatabase()
@@ -137,13 +139,13 @@ func Test_ServiceQuery(t *testing.T) {
 	}
 
 	// Ready for Query tests now.
-	db.queryFn = func(er *command.QueryRequest) ([]*command.QueryRows, error) {
+	db.queryFn = func(er *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
 		if er.Request.Statements[0].Sql != "SELECT * FROM foo" {
 			t.Fatalf("incorrect SQL query received")
 		}
-		return nil, errors.New("query failed")
+		return nil, 0, errors.New("query failed")
 	}
-	_, err := c.Query(queryRequestFromString("SELECT * FROM foo"), s.Addr(), NO_CREDS, longWait)
+	_, _, err := c.Query(queryRequestFromString("SELECT * FROM foo"), s.Addr(), NO_CREDS, longWait)
 	if err == nil {
 		t.Fatalf("client failed to report error")
 	}
@@ -151,7 +153,7 @@ func Test_ServiceQuery(t *testing.T) {
 		t.Fatalf("incorrect error message received, got: %s", err.Error())
 	}
 
-	db.queryFn = func(qr *command.QueryRequest) ([]*command.QueryRows, error) {
+	db.queryFn = func(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
 		if qr.Request.Statements[0].Sql != "SELECT * FROM foo" {
 			t.Fatalf("incorrect SQL statement received")
 		}
@@ -159,9 +161,9 @@ func Test_ServiceQuery(t *testing.T) {
 			Columns: []string{"c1", "c2"},
 			Types:   []string{"t1", "t2"},
 		}
-		return []*command.QueryRows{rows}, nil
+		return []*command.QueryRows{rows}, 0, nil
 	}
-	res, err := c.Query(queryRequestFromString("SELECT * FROM foo"), s.Addr(), NO_CREDS, longWait)
+	res, _, err := c.Query(queryRequestFromString("SELECT * FROM foo"), s.Addr(), NO_CREDS, longWait)
 	if err != nil {
 		t.Fatalf("failed to query: %s", err.Error())
 	}
@@ -169,16 +171,16 @@ func Test_ServiceQuery(t *testing.T) {
 		t.Fatalf("unexpected results for query, expected %s, got %s", exp, got)
 	}
 
-	db.queryFn = func(qr *command.QueryRequest) ([]*command.QueryRows, error) {
+	db.queryFn = func(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
 		if qr.Request.Statements[0].Sql != "SELECT * FROM foo" {
 			t.Fatalf("incorrect SQL statement received")
 		}
 		rows := &command.QueryRows{
 			Error: "no such table",
 		}
-		return []*command.QueryRows{rows}, nil
+		return []*command.QueryRows{rows}, 0, nil
 	}
-	res, err = c.Query(queryRequestFromString("SELECT * FROM foo"), s.Addr(), NO_CREDS, longWait)
+	res, _, err = c.Query(queryRequestFromString("SELECT * FROM foo"), s.Addr(), NO_CREDS, longWait)
 	if err != nil {
 		t.Fatalf("failed to query: %s", err.Error())
 	}
@@ -186,11 +188,11 @@ func Test_ServiceQuery(t *testing.T) {
 		t.Fatalf("unexpected results for query, expected %s, got %s", exp, got)
 	}
 
-	db.queryFn = func(er *command.QueryRequest) ([]*command.QueryRows, error) {
+	db.queryFn = func(er *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
 		time.Sleep(longWait)
-		return nil, nil
+		return nil, 0, nil
 	}
-	_, err = c.Query(queryRequestFromString("some SQL"), s.Addr(), NO_CREDS, shortWait)
+	_, _, err = c.Query(queryRequestFromString("some SQL"), s.Addr(), NO_CREDS, shortWait)
 	if err == nil {
 		t.Fatalf("failed to receive expected error")
 	}
@@ -211,6 +213,7 @@ func Test_ServiceQuery(t *testing.T) {
 // encoded and decoded correctly.
 func Test_ServiceQueryLarge(t *testing.T) {
 	ln, mux := mustNewMux()
+	defer mux.Close()
 	go mux.Serve()
 	tn := mux.Listen(1) // Could be any byte value.
 	db := mustNewMockDatabase()
@@ -236,7 +239,7 @@ func Test_ServiceQueryLarge(t *testing.T) {
 	}
 
 	// Ready for Query tests now.
-	db.queryFn = func(qr *command.QueryRequest) ([]*command.QueryRows, error) {
+	db.queryFn = func(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
 		parameter := &command.Parameter{
 			Value: &command.Parameter_S{
 				S: b.String(),
@@ -251,9 +254,9 @@ func Test_ServiceQueryLarge(t *testing.T) {
 			Types:   []string{"t1"},
 			Values:  []*command.Values{value},
 		}
-		return []*command.QueryRows{rows}, nil
+		return []*command.QueryRows{rows}, 0, nil
 	}
-	res, err := c.Query(queryRequestFromString("SELECT * FROM foo"), s.Addr(), NO_CREDS, longWait)
+	res, _, err := c.Query(queryRequestFromString("SELECT * FROM foo"), s.Addr(), NO_CREDS, longWait)
 	if err != nil {
 		t.Fatalf("failed to query: %s", err.Error())
 	}
@@ -272,6 +275,7 @@ func Test_ServiceQueryLarge(t *testing.T) {
 
 func Test_ServiceBackup(t *testing.T) {
 	ln, mux := mustNewMux()
+	defer mux.Close()
 	go mux.Serve()
 	tn := mux.Listen(1) // Could be any byte value.
 	db := mustNewMockDatabase()
@@ -319,6 +323,7 @@ func Test_ServiceBackup(t *testing.T) {
 
 func Test_ServiceLoad(t *testing.T) {
 	ln, mux := mustNewMux()
+	defer mux.Close()
 	go mux.Serve()
 	tn := mux.Listen(1) // Could be any byte value.
 	db := mustNewMockDatabase()
@@ -366,6 +371,7 @@ func Test_ServiceLoad(t *testing.T) {
 
 func Test_ServiceRemoveNode(t *testing.T) {
 	ln, mux := mustNewMux()
+	defer mux.Close()
 	go mux.Serve()
 	tn := mux.Listen(1) // Could be any byte value.
 	db := mustNewMockDatabase()
@@ -412,6 +418,7 @@ func Test_ServiceRemoveNode(t *testing.T) {
 
 func Test_ServiceJoinNode(t *testing.T) {
 	ln, mux := mustNewMux()
+	defer mux.Close()
 	go mux.Serve()
 	tn := mux.Listen(1) // Could be any byte value.
 	db := mustNewMockDatabase()
@@ -469,6 +476,7 @@ func Test_ServiceJoinNodeForwarded(t *testing.T) {
 
 	// Create the Leader service.
 	lnL, muxL := mustNewMux()
+	defer muxL.Close()
 	go muxL.Serve()
 	tnL := muxL.Listen(headerByte)
 	dbL := mustNewMockDatabase()
@@ -487,6 +495,7 @@ func Test_ServiceJoinNodeForwarded(t *testing.T) {
 
 	// Create the Follower service.
 	lnF, muxF := mustNewMux()
+	defer muxF.Close()
 	go muxF.Serve()
 	tnF := muxF.Listen(headerByte)
 	dbF := mustNewMockDatabase()
@@ -637,7 +646,7 @@ func mustGZIPCompress(b []byte) []byte {
 	return buf.Bytes()
 }
 
-func asJSON(v interface{}) string {
+func asJSON(v any) string {
 	enc := encoding.Encoder{}
 	b, err := enc.JSONMarshal(v)
 	if err != nil {
