@@ -241,12 +241,12 @@ func Test_NewServiceTestExecuteQueryAuthNoCredentials(t *testing.T) {
 		t.Fatalf("failed to set cluster client local parameters: %s", err)
 	}
 	er := &command.ExecuteRequest{}
-	_, err := cl.Execute(er, s.Addr(), nil, 5*time.Second, defaultMaxRetries)
+	_, _, err := cl.Execute(er, s.Addr(), nil, 5*time.Second, defaultMaxRetries)
 	if err != nil {
 		t.Fatal(err)
 	}
 	qr := &command.QueryRequest{}
-	_, err = cl.Query(qr, s.Addr(), nil, 5*time.Second)
+	_, _, err = cl.Query(qr, s.Addr(), nil, 5*time.Second)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -284,21 +284,21 @@ func Test_NewServiceTestExecuteQueryAuth(t *testing.T) {
 		t.Fatalf("failed to set cluster client local parameters: %s", err)
 	}
 	er := &command.ExecuteRequest{}
-	_, err := cl.Execute(er, s.Addr(), makeCredentials("alice", "secret1"), 5*time.Second, defaultMaxRetries)
+	_, _, err := cl.Execute(er, s.Addr(), makeCredentials("alice", "secret1"), 5*time.Second, defaultMaxRetries)
 	if err != nil {
 		t.Fatal("alice improperly unauthorized to execute")
 	}
-	_, err = cl.Execute(er, s.Addr(), makeCredentials("bob", "secret1"), 5*time.Second, defaultMaxRetries)
+	_, _, err = cl.Execute(er, s.Addr(), makeCredentials("bob", "secret1"), 5*time.Second, defaultMaxRetries)
 	if err == nil {
 		t.Fatal("bob improperly authorized to execute")
 	}
 	qr := &command.QueryRequest{}
-	_, err = cl.Query(qr, s.Addr(), makeCredentials("bob", "secret1"), 5*time.Second)
+	_, _, err = cl.Query(qr, s.Addr(), makeCredentials("bob", "secret1"), 5*time.Second)
 	if err != nil && err.Error() != "unauthorized" {
 		fmt.Println(err)
 		t.Fatal("bob improperly unauthorized to query")
 	}
-	_, err = cl.Query(qr, s.Addr(), makeCredentials("alice", "secret1"), 5*time.Second)
+	_, _, err = cl.Query(qr, s.Addr(), makeCredentials("alice", "secret1"), 5*time.Second)
 	if err != nil && err.Error() != "unauthorized" {
 		t.Fatal("alice improperly authorized to query")
 	}
@@ -424,6 +424,91 @@ func Test_NewServiceJoin(t *testing.T) {
 	}
 }
 
+func Test_ServiceHandleHighwaterMarkUpdate(t *testing.T) {
+	ml := mustNewMockTransport()
+	mgr := mustNewMockManager()
+	s := New(ml, mustNewMockDatabase(), mgr, mustNewMockCredentialStore())
+	if s == nil {
+		t.Fatalf("failed to create cluster service")
+	}
+
+	if err := s.Open(); err != nil {
+		t.Fatalf("failed to open cluster service")
+	}
+	defer s.Close()
+
+	// Create a client and send highwater mark update
+	c := NewClient(ml, 30*time.Second)
+	c.SetLocal("test-node", nil)
+
+	// Use the client to send a highwater mark update
+	responses, err := c.BroadcastHWM(987654, 0, 5*time.Second, s.Addr())
+	if err != nil {
+		t.Fatalf("failed to broadcast highwater mark update: %s", err)
+	}
+
+	// Check that we got a response for the service address
+	resp, ok := responses[s.Addr()]
+	if !ok {
+		t.Fatalf("expected response for address %s", s.Addr())
+	}
+
+	// Check response has no error
+	if resp.Error != "" {
+		t.Fatalf("expected no error, got: %s", resp.Error)
+	}
+}
+
+func Test_ServiceRegisterHWMUpdate(t *testing.T) {
+	ml := mustNewMockTransport()
+	mgr := mustNewMockManager()
+	s := New(ml, mustNewMockDatabase(), mgr, mustNewMockCredentialStore())
+	if s == nil {
+		t.Fatalf("failed to create cluster service")
+	}
+
+	if err := s.Open(); err != nil {
+		t.Fatalf("failed to open cluster service")
+	}
+	defer s.Close()
+
+	// Create a channel to receive highwater mark updates
+	hwmCh := make(chan uint64, 1)
+	s.RegisterHWMUpdate(hwmCh)
+
+	// Create a client and send highwater mark update
+	c := NewClient(ml, 30*time.Second)
+	c.SetLocal("test-node", nil)
+
+	// Use the client to send a highwater mark update
+	testHWM := uint64(123456)
+	responses, err := c.BroadcastHWM(testHWM, 0, 5*time.Second, s.Addr())
+	if err != nil {
+		t.Fatalf("failed to broadcast highwater mark update: %s", err)
+	}
+
+	// Check that we got a response for the service address
+	resp, ok := responses[s.Addr()]
+	if !ok {
+		t.Fatalf("expected response for address %s", s.Addr())
+	}
+
+	// Check response has no error
+	if resp.Error != "" {
+		t.Fatalf("expected no error, got: %s", resp.Error)
+	}
+
+	// Check that we received the update on the channel
+	select {
+	case hwmUpdate := <-hwmCh:
+		if hwmUpdate != testHWM {
+			t.Fatalf("expected highwater_mark to be %d, got: %d", testHWM, hwmUpdate)
+		}
+	case <-time.After(5 * time.Second):
+		t.Fatalf("timeout waiting for highwater mark update on channel")
+	}
+}
+
 type mockTransport struct {
 	tn              net.Listener
 	remoteEncrypted bool
@@ -477,24 +562,24 @@ func mustNewMockTLSTransport() *mockTransport {
 }
 
 type mockDatabase struct {
-	executeFn func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error)
-	queryFn   func(qr *command.QueryRequest) ([]*command.QueryRows, error)
-	requestFn func(rr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, error)
+	executeFn func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error)
+	queryFn   func(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error)
+	requestFn func(rr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, uint64, error)
 	backupFn  func(br *command.BackupRequest, dst io.Writer) error
 	loadFn    func(lr *command.LoadRequest) error
 }
 
-func (m *mockDatabase) Execute(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error) {
+func (m *mockDatabase) Execute(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
 	return m.executeFn(er)
 }
 
-func (m *mockDatabase) Query(qr *command.QueryRequest) ([]*command.QueryRows, error) {
+func (m *mockDatabase) Query(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
 	return m.queryFn(qr)
 }
 
-func (m *mockDatabase) Request(rr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, error) {
+func (m *mockDatabase) Request(rr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
 	if m.requestFn == nil {
-		return []*command.ExecuteQueryResponse{}, nil
+		return []*command.ExecuteQueryResponse{}, 0, nil
 	}
 	return m.requestFn(rr)
 }
@@ -514,11 +599,11 @@ func (m *mockDatabase) Load(lr *command.LoadRequest) error {
 }
 
 func mustNewMockDatabase() *mockDatabase {
-	e := func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error) {
-		return []*command.ExecuteQueryResponse{}, nil
+	e := func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
+		return []*command.ExecuteQueryResponse{}, 0, nil
 	}
-	q := func(er *command.QueryRequest) ([]*command.QueryRows, error) {
-		return []*command.QueryRows{}, nil
+	q := func(er *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
+		return []*command.QueryRows{}, 0, nil
 	}
 	return &mockDatabase{executeFn: e, queryFn: q}
 }
@@ -528,6 +613,7 @@ type MockManager struct {
 	notifyFn     func(n *command.NotifyRequest) error
 	joinFn       func(j *command.JoinRequest) error
 	leaderAddrFn func() (string, error)
+	stepdownFn   func(wait bool, id string) error
 	commitIndex  uint64
 }
 
@@ -552,6 +638,13 @@ func (m *MockManager) Join(j *command.JoinRequest) error {
 	return m.joinFn(j)
 }
 
+func (m *MockManager) Stepdown(wait bool, id string) error {
+	if m.stepdownFn == nil {
+		return nil
+	}
+	return m.stepdownFn(wait, id)
+}
+
 func (m *MockManager) LeaderAddr() (string, error) {
 	if m.leaderAddrFn == nil {
 		return "", nil
