@@ -15,15 +15,17 @@ import (
 	"strings"
 	"syscall"
 
-	"github.com/Bowery/prompt"
 	"github.com/mkideal/cli"
 	clix "github.com/mkideal/cli/ext"
+	"github.com/peterh/liner"
 	"github.com/rqlite/rqlite/v8/cmd"
 	"github.com/rqlite/rqlite/v8/cmd/rqlite/history"
 	httpcl "github.com/rqlite/rqlite/v8/cmd/rqlite/http"
-	"github.com/rqlite/rqlite/v8/rtls"
+	"github.com/rqlite/rqlite/v8/internal/rtls"
 )
 
+const HOST_ENV_VAR = "RQLITE_HOST"
+
 const maxRedirect = 21
 
 type Node struct {
@@ -42,8 +44,8 @@ type argT struct {
 	Prefix       string        `cli:"P,prefix" usage:"rqlited HTTP URL prefix" dft:"/"`
 	Insecure     bool          `cli:"i,insecure" usage:"do not verify rqlited HTTPS certificate" dft:"false"`
 	CACert       string        `cli:"c,ca-cert" usage:"path to trusted X.509 root CA certificate"`
-	ClientCert   string        `cli:"d,client-cert" usage:"path to client X.509 certificate for mTLS"`
-	ClientKey    string        `cli:"k,client-key" usage:"path to client X.509 key for mTLS"`
+	ClientCert   string        `cli:"d,client-cert" usage:"path to client X.509 certificate for mutual TLS"`
+	ClientKey    string        `cli:"k,client-key" usage:"path to client X.509 key for mutual TLS"`
 	Credentials  string        `cli:"u,user" usage:"set basic auth credentials in form username:password"`
 	Version      bool          `cli:"v,version" usage:"display CLI version"`
 	HTTPTimeout  clix.Duration `cli:"t,http-timeout" usage:"set timeout on HTTP requests" dft:"30s"`
@@ -57,7 +59,7 @@ func init() {
 		`.blobarray on|off                             Display BLOB data as byte arrays`,
 		`.boot FILE                                    Boot the node using a SQLite file read from FILE`,
 		`.consistency [none|weak|linearizable|strong]  Show or set read consistency level`,
-		`.dump FILE                                    Dump the database in SQL text format to FILE`,
+		`.dump FILE [TABLE,TABLE...]                   Dump the database in SQL text to FILE, optionally limited to TABLEs`,
 		`.exit                                         Exit this program`,
 		`.expvar                                       Show expvar (Go runtime) information for connected node`,
 		`.extensions                                   Show loaded SQLite extensions`,
@@ -71,6 +73,7 @@ func init() {
 		`.schema                                       Show CREATE statements for all tables`,
 		`.snapshot                                     Request a Raft snapshot and log truncation on connected node`,
 		`.status                                       Show status and diagnostic information for connected node`,
+		`.stepdown [NODEID]                            Instruct the leader to stepdown, optionally specifying new Leader node`,
 		`.sysdump FILE                                 Dump system diagnostics to FILE`,
 		`.tables                                       List names of tables`,
 		`.timer on|off                                 Turn query timings on or off`,
@@ -93,6 +96,23 @@ func main() {
 			return nil
 		}
 
+		_, hostEnvSet := os.LookupEnv(HOST_ENV_VAR)
+		if hostEnvSet && !(ctx.IsSet("--host", "-H") || ctx.IsSet("--port", "-p") || ctx.IsSet("--scheme", "-s")) {
+			protocol, host, port, err := httpcl.ParseHostEnv(HOST_ENV_VAR)
+			if err != nil {
+				return err
+			}
+			if protocol != "" {
+				argv.Protocol = protocol
+			}
+			if host != "" {
+				argv.Host = host
+			}
+			if port != 0 {
+				argv.Port = port
+			}
+		}
+
 		httpClient, err := getHTTPClient(argv)
 		if err != nil {
 			ctx.String("%s %v\n", ctx.Color().Red("ERR!"), err)
@@ -119,19 +139,17 @@ func main() {
 		timer := false
 		consistency := "weak"
 		prefix := fmt.Sprintf("%s>", address6(argv))
-		term, err := prompt.NewTerminal()
-		if err != nil {
-			ctx.String("%s %v\n", ctx.Color().Red("ERR!"), err)
-			return nil
-		}
-		term.Close()
+
+		// Set up line editing with liner
+		line := liner.NewLiner()
+		defer line.Close()
 
 		// Set up command history.
 		hr := history.Reader()
 		if hr != nil {
-			histCmds, err := history.Read(hr)
-			if err == nil {
-				term.History = histCmds
+			_, err := line.ReadHistory(hr)
+			if err != nil {
+				// If reading history fails, it's not critical - just continue
 			}
 			hr.Close()
 		}
@@ -144,35 +162,40 @@ func main() {
 
 	FOR_READ:
 		for {
-			term.Reopen()
-			line, err := term.Basic(prefix, false)
-			term.Close()
+			input, err := line.Prompt(prefix + " ")
 			if err != nil {
-				if errors.Is(err, prompt.ErrEOF) {
+				if err == liner.ErrPromptAborted {
+					break FOR_READ
+				}
+				if err == io.EOF {
 					break FOR_READ
 				}
 				return err
 			}
 
-			line = strings.TrimSpace(line)
-			if line == "" {
+			input = strings.TrimSpace(input)
+			if input == "" {
 				continue
 			}
+
+			// Add command to history for up/down arrow navigation
+			line.AppendHistory(input)
+
 			var (
-				index = strings.Index(line, " ")
-				cmd   = line
+				index = strings.Index(input, " ")
+				cmd   = input
 			)
 			if index >= 0 {
-				cmd = line[:index]
+				cmd = input[:index]
 			}
 			cmd = strings.ToUpper(cmd)
 			switch cmd {
 			case ".CONSISTENCY":
-				if index == -1 || index == len(line)-1 {
+				if index == -1 || index == len(input)-1 {
 					ctx.String("%s\n", consistency)
 					break
 				}
-				err = setConsistency(line[index+1:], &consistency)
+				err = setConsistency(input[index+1:], &consistency)
 			case ".EXTENSIONS":
 				err = extensions(ctx, client, cmd, argv)
 			case ".TABLES":
@@ -182,63 +205,88 @@ func main() {
 			case ".SCHEMA":
 				err = queryWithClient(ctx, client, timer, blobArray, consistency, `SELECT sql FROM sqlite_master`)
 			case ".TIMER":
-				err = toggleFlag(line[index+1:], &timer)
+				err = toggleFlag(input[index+1:], &timer)
 			case ".BLOBARRAY":
-				err = toggleFlag(line[index+1:], &blobArray)
+				err = toggleFlag(input[index+1:], &blobArray)
 			case ".STATUS":
 				err = status(ctx, client, cmd, argv)
 			case ".READY":
 				err = ready(ctx, client, argv)
 			case ".NODES":
-				if index == -1 || index == len(line)-1 {
-					err = nodes(ctx, client, cmd, line, argv, false)
+				if index == -1 || index == len(input)-1 {
+					err = nodes(ctx, client, cmd, input, argv, false)
 					break
 				}
-				err = nodes(ctx, client, cmd, line, argv, true)
+				err = nodes(ctx, client, cmd, input, argv, true)
 			case ".EXPVAR":
-				err = expvar(ctx, client, cmd, line, argv)
+				err = expvar(ctx, client, cmd, input, argv)
 			case ".REMOVE":
-				err = removeNode(client, line[index+1:], argv)
+				err = removeNode(client, input[index+1:], argv)
 			case ".BACKUP":
-				if index == -1 || index == len(line)-1 {
+				if index == -1 || index == len(input)-1 {
 					err = fmt.Errorf("please specify an output file for the backup")
 					break
 				}
-				err = backup(ctx, line[index+1:], argv)
+				err = backup(ctx, input[index+1:], argv)
 			case ".RESTORE":
-				if index == -1 || index == len(line)-1 {
+				if index == -1 || index == len(input)-1 {
 					err = fmt.Errorf("please specify an input file to restore from")
 					break
 				}
-				err = restore(ctx, line[index+1:], argv)
+				err = restore(ctx, input[index+1:], argv)
 			case ".BOOT":
-				if index == -1 || index == len(line)-1 {
+				if index == -1 || index == len(input)-1 {
 					err = fmt.Errorf("please specify an input file to boot with")
 					break
 				}
-				err = boot(ctx, line[index+1:], argv)
+				err = boot(ctx, input[index+1:], argv)
 			case ".SYSDUMP":
-				if index == -1 || index == len(line)-1 {
+				if index == -1 || index == len(input)-1 {
 					err = fmt.Errorf("please specify an output file for the sysdump")
 					break
 				}
-				err = sysdump(ctx, httpClient, line[index+1:], argv)
+				err = sysdump(ctx, httpClient, input[index+1:], argv)
 			case ".DUMP":
-				if index == -1 || index == len(line)-1 {
+				if index == -1 || index == len(input)-1 {
 					err = fmt.Errorf("please specify an output file for the SQL text")
 					break
 				}
-				err = dump(ctx, line[index+1:], argv)
+				args := strings.Fields(input[index+1:])
+				if len(args) == 0 {
+					err = fmt.Errorf("please specify an output file for the SQL text")
+					break
+				}
+				filename := args[0]
+				var tables []string
+				if len(args) > 1 {
+					// Join all remaining arguments as table specification and split by comma
+					tablesStr := strings.Join(args[1:], " ")
+					tables = strings.Split(tablesStr, ",")
+					// Trim whitespace from table names and filter empty ones
+					var cleanTables []string
+					for _, table := range tables {
+						table = strings.TrimSpace(table)
+						if table != "" {
+							cleanTables = append(cleanTables, table)
+						}
+					}
+					tables = cleanTables
+				}
+				err = dump(ctx, filename, tables, argv)
 			case ".HELP":
-				ctx.String("%s", strings.Join(cliHelp, "\n"))
+				ctx.String("%s\n", strings.Join(cliHelp, "\n"))
 			case ".QUIT", "QUIT", "EXIT", ".EXIT":
 				break FOR_READ
 			case ".SNAPSHOT":
 				err = snapshot(client, argv)
-			case "SELECT", "PRAGMA":
-				err = queryWithClient(ctx, client, timer, blobArray, consistency, line)
+			case ".STEPDOWN":
+				nodeID := ""
+				if index != -1 && index < len(input)-1 {
+					nodeID = strings.TrimSpace(input[index+1:])
+				}
+				err = stepdown(client, nodeID, argv)
 			default:
-				err = executeWithClient(ctx, client, timer, line)
+				err = requestWithClient(ctx, client, timer, input)
 			}
 			if hcerr, ok := err.(*httpcl.HostChangedError); ok {
 				// If a previous request was executed on a different host, make that change
@@ -254,9 +302,10 @@ func main() {
 		hw := history.Writer()
 		if hw != nil {
 			sz := history.Size()
-			history.Write(term.History, sz, hw)
+			// Write history using liner's WriteHistory method
+			_, err := line.WriteHistory(hw)
 			hw.Close()
-			if sz <= 0 {
+			if err != nil || sz <= 0 {
 				history.Delete()
 			}
 		}
@@ -356,6 +405,48 @@ func snapshot(client *httpcl.Client, argv *argT) error {
 	return nil
 }
 
+func stepdown(client *httpcl.Client, nodeID string, argv *argT) error {
+	url := fmt.Sprintf("%s://%s/leader?wait=true", argv.Protocol, address6(argv))
+
+	var body io.Reader
+	if nodeID != "" {
+		// Create JSON body with node ID
+		reqBody := map[string]string{"id": nodeID}
+		jsonBody, err := json.Marshal(reqBody)
+		if err != nil {
+			return err
+		}
+		body = bytes.NewReader(jsonBody)
+	}
+
+	req, err := http.NewRequest("POST", url, body)
+	if err != nil {
+		return err
+	}
+
+	if nodeID != "" {
+		req.Header.Set("Content-Type", "application/json")
+	}
+
+	if argv.Credentials != "" {
+		creds := strings.Split(argv.Credentials, ":")
+		if len(creds) != 2 {
+			return fmt.Errorf("invalid Basic Auth credentials format")
+		}
+		req.SetBasicAuth(creds[0], creds[1])
+	}
+
+	resp, err := client.Do(req)
+	if err != nil {
+		return err
+	}
+	defer resp.Body.Close()
+	if resp.StatusCode != http.StatusOK {
+		return fmt.Errorf("server responded with %s", resp.Status)
+	}
+	return nil
+}
+
 func sysdump(ctx *cli.Context, client *http.Client, filename string, argv *argT) error {
 	_ = ctx
 	nodes, err := getNodes(client, argv)
@@ -556,7 +647,7 @@ func sendRequestW(ctx *cli.Context, makeNewRequest func(string) (*http.Request,
 	}
 }
 
-func parseResponse(response *[]byte, ret interface{}) error {
+func parseResponse(response *[]byte, ret any) error {
 	decoder := json.NewDecoder(strings.NewReader(string(*response)))
 	decoder.UseNumber()
 	return decoder.Decode(ret)
@@ -582,7 +673,7 @@ func extensions(ctx *cli.Context, client *httpcl.Client, cmd string, argv *argT)
 		return err
 	}
 
-	ret := make(map[string]interface{})
+	ret := make(map[string]any)
 	decoder := json.NewDecoder(strings.NewReader(string(body)))
 	decoder.UseNumber()
 	if err := decoder.Decode(&ret); err != nil {
@@ -593,7 +684,7 @@ func extensions(ctx *cli.Context, client *httpcl.Client, cmd string, argv *argT)
 	if !ok {
 		return nil
 	}
-	for _, ext := range exts.([]interface{}) {
+	for _, ext := range exts.([]any) {
 		ctx.String("%s\n", ext.(string))
 	}
 
@@ -606,15 +697,15 @@ func extensions(ctx *cli.Context, client *httpcl.Client, cmd string, argv *argT)
 func cliJSON(ctx *cli.Context, client *httpcl.Client, line, url string) error {
 	_ = ctx
 	// Recursive JSON printer.
-	var pprint func(indent int, m map[string]interface{})
-	pprint = func(indent int, m map[string]interface{}) {
+	var pprint func(indent int, m map[string]any)
+	pprint = func(indent int, m map[string]any) {
 		indentation := "  "
 		for k, v := range m {
 			if v == nil {
 				continue
 			}
 			switch w := v.(type) {
-			case map[string]interface{}:
+			case map[string]any:
 				for i := 0; i < indent; i++ {
 					fmt.Print(indentation)
 				}
@@ -644,7 +735,7 @@ func cliJSON(ctx *cli.Context, client *httpcl.Client, line, url string) error {
 		return err
 	}
 
-	ret := make(map[string]interface{})
+	ret := make(map[string]any)
 	decoder := json.NewDecoder(strings.NewReader(string(body)))
 	decoder.UseNumber()
 	if err := decoder.Decode(&ret); err != nil {
@@ -654,7 +745,7 @@ func cliJSON(ctx *cli.Context, client *httpcl.Client, line, url string) error {
 	// Specific key requested?
 	parts := strings.Split(line, " ")
 	if len(parts) >= 2 {
-		ret = map[string]interface{}{parts[1]: ret[parts[1]]}
+		ret = map[string]any{parts[1]: ret[parts[1]]}
 	}
 	pprint(0, ret)
 
