@@ -21,15 +21,14 @@ import (
 	"github.com/rqlite/rqlite/v8/auth"
 	"github.com/rqlite/rqlite/v8/auto/backup"
 	"github.com/rqlite/rqlite/v8/auto/restore"
-	"github.com/rqlite/rqlite/v8/aws"
 	"github.com/rqlite/rqlite/v8/cluster"
+	"github.com/rqlite/rqlite/v8/cluster/disco"
 	"github.com/rqlite/rqlite/v8/cmd"
 	"github.com/rqlite/rqlite/v8/db"
-	"github.com/rqlite/rqlite/v8/disco"
-	"github.com/rqlite/rqlite/v8/extensions"
+	"github.com/rqlite/rqlite/v8/db/extensions"
 	httpd "github.com/rqlite/rqlite/v8/http"
-	"github.com/rqlite/rqlite/v8/rarchive"
-	"github.com/rqlite/rqlite/v8/rtls"
+	"github.com/rqlite/rqlite/v8/internal/rarchive"
+	"github.com/rqlite/rqlite/v8/internal/rtls"
 	"github.com/rqlite/rqlite/v8/store"
 	"github.com/rqlite/rqlite/v8/tcp"
 )
@@ -94,7 +93,10 @@ func main() {
 		log.Fatalf("failed to start node mux: %s", err.Error())
 	}
 
-	// Raft internode layer
+	// Create the Raft internode layer. All Raft-related traffic is sent over this layer.
+	// This layer allows both connections to be served from other nodes in the cluster
+	// for the purposes of Raft communication, and making connections to be made to other
+	// nodes in the cluster, again for Raft traffic.
 	raftLn := mux.Listen(cluster.MuxRaftHeader)
 	raftDialer, err := cluster.CreateRaftDialer(cfg.NodeX509Cert, cfg.NodeX509Key, cfg.NodeX509CACert,
 		cfg.NodeVerifyServerName, cfg.NoNodeVerify)
@@ -181,17 +183,15 @@ func main() {
 	}
 
 	// Register remaining status providers.
-	if err := httpServ.RegisterStatus("cluster", clstrServ); err != nil {
-		log.Fatalf("failed to register cluster status provider: %s", err.Error())
-	}
-	if err := httpServ.RegisterStatus("network", tcp.NetworkReporter{}); err != nil {
-		log.Fatalf("failed to register network status provider: %s", err.Error())
-	}
-	if err := httpServ.RegisterStatus("mux", mux); err != nil {
-		log.Fatalf("failed to register mux status provider: %s", err.Error())
-	}
-	if err := httpServ.RegisterStatus("extensions", extensionsStore); err != nil {
-		log.Fatalf("failed to register extensions status provider: %s", err.Error())
+	for n, r := range map[string]httpd.StatusReporter{
+		"cluster":    clstrServ,
+		"network":    tcp.NetworkReporter{},
+		"mux":        mux,
+		"extensions": extensionsStore,
+	} {
+		if err := httpServ.RegisterStatus(n, r); err != nil {
+			log.Fatalf("failed to register %s status provider: %s", n, err.Error())
+		}
 	}
 
 	// Create the cluster!
@@ -241,9 +241,10 @@ func main() {
 			log.Printf("stepping down as Leader before shutdown")
 		}
 		// Perform a stepdown, ignore any errors.
-		str.Stepdown(true)
+		str.Stepdown(true, "")
 	}
 	muxLn.Close()
+	defer mux.Close()
 
 	if err := str.Close(true); err != nil {
 		log.Printf("failed to close store: %s", err.Error())
@@ -262,20 +263,11 @@ func startAutoBackups(ctx context.Context, cfg *Config, str *store.Store) (*back
 		return nil, fmt.Errorf("failed to read auto-backup file: %s", err.Error())
 	}
 
-	uCfg, s3cfg, err := backup.Unmarshal(b)
+	uCfg, sc, err := backup.NewStorageClient(b)
 	if err != nil {
 		return nil, fmt.Errorf("failed to parse auto-backup file: %s", err.Error())
 	}
 	provider := store.NewProvider(str, uCfg.Vacuum, !uCfg.NoCompress)
-	s3ClientOps := &aws.S3ClientOpts{
-		ForcePathStyle: s3cfg.ForcePathStyle,
-		Timestamp:      uCfg.Timestamp,
-	}
-	sc, err := aws.NewS3Client(s3cfg.Endpoint, s3cfg.Region, s3cfg.AccessKeyID, s3cfg.SecretAccessKey,
-		s3cfg.Bucket, s3cfg.Path, s3ClientOps)
-	if err != nil {
-		return nil, fmt.Errorf("failed to create aws S3 client: %s", err.Error())
-	}
 	u := backup.NewUploader(sc, provider, time.Duration(uCfg.Interval))
 	u.Start(ctx, str.IsLeader)
 	return u, nil
@@ -399,7 +391,7 @@ func startHTTPService(cfg *Config, str *store.Store, cltr *cluster.Client, credS
 	s.DefaultQueueBatchSz = cfg.WriteQueueBatchSz
 	s.DefaultQueueTimeout = cfg.WriteQueueTimeout
 	s.DefaultQueueTx = cfg.WriteQueueTx
-	s.BuildInfo = map[string]interface{}{
+	s.BuildInfo = map[string]any{
 		"commit":             cmd.Commit,
 		"branch":             cmd.Branch,
 		"version":            cmd.Version,
@@ -429,12 +421,12 @@ func startNodeMux(cfg *Config, ln net.Listener) (*tcp.Mux, error) {
 		}
 		if cfg.NodeVerifyClient {
 			b.WriteString(", mutual TLS enabled")
+			mux, err = tcp.NewMutualTLSMux(ln, adv, cfg.NodeX509Cert, cfg.NodeX509Key, cfg.NodeX509CACert)
 		} else {
 			b.WriteString(", mutual TLS disabled")
+			mux, err = tcp.NewTLSMux(ln, adv, cfg.NodeX509Cert, cfg.NodeX509Key)
 		}
 		log.Println(b.String())
-		mux, err = tcp.NewTLSMux(ln, adv, cfg.NodeX509Cert, cfg.NodeX509Key, cfg.NodeX509CACert,
-			cfg.NoNodeVerify, cfg.NodeVerifyClient)
 	} else {
 		mux, err = tcp.NewMux(ln, adv)
 	}
@@ -466,8 +458,13 @@ func clusterService(cfg *Config, ln net.Listener, db cluster.Database, mgr clust
 func createClusterClient(cfg *Config, clstr *cluster.Service) (*cluster.Client, error) {
 	var dialerTLSConfig *tls.Config
 	var err error
+	var cr *rtls.CertReloader
 	if cfg.NodeX509Cert != "" || cfg.NodeX509CACert != "" {
-		dialerTLSConfig, err = rtls.CreateClientConfig(cfg.NodeX509Cert, cfg.NodeX509Key,
+		cr, err = rtls.NewCertReloader(cfg.NodeX509Cert, cfg.NodeX509Key)
+		if err != nil {
+			return nil, fmt.Errorf("failed to create certificate monitor: %s", err.Error())
+		}
+		dialerTLSConfig, err = rtls.CreateClientConfigWithFunc(cr.GetCertificate,
 			cfg.NodeX509CACert, cfg.NodeVerifyServerName, cfg.NoNodeVerify)
 		if err != nil {
 			return nil, fmt.Errorf("failed to create TLS config for cluster dialer: %s", err.Error())
@@ -475,10 +472,10 @@ func createClusterClient(cfg *Config, clstr *cluster.Service) (*cluster.Client,
 	}
 	clstrDialer := tcp.NewDialer(cluster.MuxClusterHeader, dialerTLSConfig)
 	clstrClient := cluster.NewClient(clstrDialer, cfg.ClusterConnectTimeout)
-	if err := clstrClient.SetLocal(cfg.RaftAdv, clstr); err != nil {
+	if err = clstrClient.SetLocal(cfg.RaftAdv, clstr); err != nil {
 		return nil, fmt.Errorf("failed to set cluster client local parameters: %s", err.Error())
 	}
-	if err := clstrClient.SetLocalVersion(cmd.Version); err != nil {
+	if err = clstrClient.SetLocalVersion(cmd.Version); err != nil {
 		return nil, fmt.Errorf("failed to set cluster client local version: %s", err.Error())
 	}
 	return clstrClient, nil
@@ -513,6 +510,18 @@ func createCluster(ctx context.Context, cfg *Config, hasPeers bool, client *clus
 	joiner := cluster.NewJoiner(client, cfg.JoinAttempts, cfg.JoinInterval)
 	joiner.SetCredentials(cluster.CredentialsFor(credStr, cfg.JoinAs))
 	if joins != nil && cfg.BootstrapExpect == 0 {
+		// Check if this node is already a member of a single-node cluster.
+		nodes, err := str.Nodes()
+		if err != nil {
+			return fmt.Errorf("failed to get nodes: %s", err.Error())
+		}
+
+		// If this node is the only node in a single-node cluster,
+		// it doesn't make sense to join it to another cluster.
+		if len(nodes) == 1 && nodes[0].ID == str.ID() {
+			return fmt.Errorf("node is the only node in a single-node cluster, joining other clusters not supported")
+		}
+
 		// Explicit join operation requested, so do it.
 		j, err := joiner.Do(ctx, joins, str.ID(), cfg.RaftAdv, clusterSuf)
 		if err != nil {
