@@ -4,13 +4,13 @@ import (
 	"expvar"
 	"fmt"
 	"math"
-	"math/rand"
+	"math/rand/v2"
 	"strconv"
 	"strings"
 	"time"
 
 	"github.com/rqlite/rqlite/v8/command/proto"
-	"github.com/rqlite/rqlite/v8/random"
+	"github.com/rqlite/rqlite/v8/internal/random"
 	"github.com/rqlite/sql"
 	rsql "github.com/rqlite/sql"
 )
@@ -128,10 +128,8 @@ func NewRewriter() *Rewriter {
 		RewriteRand: true,
 		RewriteTime: true,
 
-		randFn: func() int64 {
-			return rand.Int63()
-		},
-		nowFn: time.Now,
+		randFn: rand.Int64,
+		nowFn:  time.Now,
 	}
 }
 
