@@ -16,10 +16,10 @@ import (
 	"sync"
 	"time"
 
-	"github.com/rqlite/go-sqlite3"
+	"github.com/mattn/go-sqlite3"
 	command "github.com/rqlite/rqlite/v8/command/proto"
 	"github.com/rqlite/rqlite/v8/db/humanize"
-	"github.com/rqlite/rqlite/v8/rsum"
+	"github.com/rqlite/rqlite/v8/internal/rsum"
 )
 
 const (
@@ -52,6 +52,11 @@ const (
 	numPreupdates             = "preupdates"
 	numPreupdatesErrors       = "preupdates_errors"
 	numPreupdatesCBErrors     = "preupdates_callback_errors"
+	numUpdateHooks            = "update_hooks"
+	numUpdateHooksCBErrors    = "update_hooks_callback_errors"
+	numUpdateHooksErrors      = "update_hooks_errors"
+	numCommitHooks            = "commit_hooks"
+	cdcDroppedEvents          = "dropped_cdc_events"
 )
 
 var (
@@ -119,6 +124,11 @@ func ResetStats() {
 	stats.Add(numPreupdates, 0)
 	stats.Add(numPreupdatesErrors, 0)
 	stats.Add(numPreupdatesCBErrors, 0)
+	stats.Add(numUpdateHooks, 0)
+	stats.Add(numUpdateHooksCBErrors, 0)
+	stats.Add(numUpdateHooksErrors, 0)
+	stats.Add(numCommitHooks, 0)
+	stats.Add(cdcDroppedEvents, 0)
 }
 
 // DB is the SQL database.
@@ -316,7 +326,7 @@ func (db *DB) RegisterPreUpdateHook(hook PreUpdateHookCallback, rowIDsOnly bool)
 			}
 		}
 	}
-	f := func(driverConn interface{}) error {
+	f := func(driverConn any) error {
 		conn := driverConn.(*sqlite3.SQLiteConn)
 		conn.RegisterPreUpdateHook(cb)
 		return nil
@@ -333,6 +343,102 @@ func (db *DB) RegisterPreUpdateHook(hook PreUpdateHookCallback, rowIDsOnly bool)
 	return nil
 }
 
+// UpdateHookCallback is a callback function that is called before a row is modified
+// in the database.
+type UpdateHookCallback func(ev *command.UpdateHookEvent) error
+
+// RegisterUpdateHook registers a callback that is called when a row is modified
+// in the database. If a callback is already registered, it is replaced. If hook is nil,
+// the callback is removed.
+func (db *DB) RegisterUpdateHook(hook UpdateHookCallback) error {
+	// Convert from SQLite hook data to rqlite hook data.
+	convertFn := func(op int, _, table string, rowID int64) (*command.UpdateHookEvent, error) {
+		he := &command.UpdateHookEvent{
+			Table: table,
+			RowId: rowID,
+		}
+
+		switch op {
+		case sqlite3.SQLITE_INSERT:
+			he.Op = command.UpdateHookEvent_INSERT
+		case sqlite3.SQLITE_UPDATE:
+			he.Op = command.UpdateHookEvent_UPDATE
+		case sqlite3.SQLITE_DELETE:
+			he.Op = command.UpdateHookEvent_DELETE
+		default:
+			return nil, fmt.Errorf("unknown update hook operation %d", op)
+		}
+		return he, nil
+	}
+
+	// Register the callback with the SQLite connection.
+	var cb func(int, string, string, int64)
+	if hook != nil {
+		cb = func(op int, dbName, tblName string, rowID int64) {
+			stats.Add(numUpdateHooks, 1)
+			ev, err := convertFn(op, dbName, tblName, rowID)
+			if err != nil {
+				stats.Add(numUpdateHooks, 1)
+				ev.Error = err.Error()
+			}
+			if err := hook(ev); err != nil {
+				stats.Add(numUpdateHooksCBErrors, 1)
+			}
+		}
+	}
+	f := func(driverConn any) error {
+		conn := driverConn.(*sqlite3.SQLiteConn)
+		conn.RegisterUpdateHook(cb)
+		return nil
+	}
+
+	conn, err := db.rwDB.Conn(context.Background())
+	if err != nil {
+		return err
+	}
+	defer conn.Close()
+	if err := conn.Raw(f); err != nil {
+		return err
+	}
+	return nil
+}
+
+// CommitHookCallback is a callback function that is called whenever a transaction
+// is committed to the database. If the callback returns true the transaction
+// is committed, otherwise it is rolled back.
+type CommitHookCallback func() bool
+
+// RegisterCommitHook registers a callback that is called whenever a transaction
+// is committed to the database. If a callback is already registered, it is replaced.
+// If hook is nil, the callback is removed.
+func (db *DB) RegisterCommitHook(hook CommitHookCallback) error {
+	var cb func() int
+	if hook != nil {
+		cb = func() int {
+			stats.Add(numCommitHooks, 1)
+			if hook() {
+				return 0
+			}
+			return 1
+		}
+	}
+	f := func(driverConn any) error {
+		conn := driverConn.(*sqlite3.SQLiteConn)
+		conn.RegisterCommitHook(cb)
+		return nil
+	}
+
+	conn, err := db.rwDB.Conn(context.Background())
+	if err != nil {
+		return err
+	}
+	defer conn.Close()
+	if err := conn.Raw(f); err != nil {
+		return err
+	}
+	return nil
+}
+
 // LastModified returns the last modified time of the database file, or the WAL file,
 // whichever is most recent.
 func (db *DB) LastModified() (time.Time, error) {
@@ -379,7 +485,7 @@ func (db *DB) Close() error {
 }
 
 // Stats returns status and diagnostics for the database.
-func (db *DB) Stats() (map[string]interface{}, error) {
+func (db *DB) Stats() (map[string]any, error) {
 	copts, err := db.CompileOptions()
 	if err != nil {
 		return nil, err
@@ -388,7 +494,7 @@ func (db *DB) Stats() (map[string]interface{}, error) {
 	if err != nil {
 		return nil, err
 	}
-	connPoolStats := map[string]interface{}{
+	connPoolStats := map[string]any{
 		"ro": db.ConnectionPoolStats(db.roDB),
 		"rw": db.ConnectionPoolStats(db.rwDB),
 	}
@@ -400,7 +506,11 @@ func (db *DB) Stats() (map[string]interface{}, error) {
 	if err != nil {
 		return nil, err
 	}
-	stats := map[string]interface{}{
+	txStatus, err := db.txStatus()
+	if err != nil {
+		return nil, err
+	}
+	stats := map[string]any{
 		"extensions":       db.ExtensionNames(),
 		"version":          DBVersion,
 		"compile_options":  copts,
@@ -411,6 +521,7 @@ func (db *DB) Stats() (map[string]interface{}, error) {
 		"ro_dsn":           db.roDSN,
 		"conn_pool_stats":  connPoolStats,
 		"pragmas":          pragmas,
+		"transaction":      txStatus,
 	}
 
 	lm, err := db.LastModified()
@@ -441,7 +552,7 @@ func (db *DB) Size() (int64, error) {
 		return 0, fmt.Errorf("no rows returned")
 	}
 	if rows[0].Error != "" {
-		return 0, fmt.Errorf(rows[0].Error)
+		return 0, errors.New(rows[0].Error)
 	}
 	return rows[0].Values[0].Parameters[0].GetI(), nil
 }
@@ -785,6 +896,10 @@ func (db *DB) executeWithConn(ctx context.Context, req *command.Request, xTime b
 			tx = nil
 			return false
 		}
+		if req.RollbackOnError {
+			db.executeStmtWithConn(ctx, &command.Statement{Sql: "ROLLBACK"}, false, eqer, time.Duration(req.DbTimeout))
+			return false
+		}
 		return true
 	}
 
@@ -940,7 +1055,7 @@ func (db *DB) Query(req *command.Request, xTime bool) ([]*command.QueryRows, err
 }
 
 type queryer interface {
-	QueryContext(ctx context.Context, query string, args ...interface{}) (*sql.Rows, error)
+	QueryContext(ctx context.Context, query string, args ...any) (*sql.Rows, error)
 }
 
 func (db *DB) queryWithConn(ctx context.Context, req *command.Request, xTime bool, conn *sql.Conn) ([]*command.QueryRows, error) {
@@ -1044,8 +1159,8 @@ func (db *DB) queryStmtWithConn(ctx context.Context, stmt *command.Statement, xT
 	}
 	needsQueryTypes := containsEmptyType(xTypes)
 
-	dest := make([]interface{}, len(columns))
-	ptrs := make([]interface{}, len(dest))
+	dest := make([]any, len(columns))
+	ptrs := make([]any, len(dest))
 	for rs.Next() {
 		dest := dest[:]
 		ptrs := ptrs[:]
@@ -1262,7 +1377,7 @@ func (db *DB) Serialize() ([]byte, error) {
 
 // Dump writes a consistent snapshot of the database in SQL text format.
 // This function can be called when changes to the database are in flight.
-func (db *DB) Dump(w io.Writer) error {
+func (db *DB) Dump(w io.Writer, tableNames ...string) error {
 	conn, err := db.roDB.Conn(context.Background())
 	if err != nil {
 		return err
@@ -1286,15 +1401,14 @@ func (db *DB) Dump(w io.Writer) error {
 	}
 
 	// Get the schema.
-	query := `SELECT "name", "type", "sql" FROM "sqlite_master"
-              WHERE "sql" NOT NULL AND "type" == 'table' ORDER BY "name"`
-	rows, err := db.queryWithConn(ctx, commReq(query), false, conn)
+	rows, err := db.queryWithConn(ctx, DumpTablesReq(tableNames...), false, conn)
 	if err != nil {
 		return err
 	}
 	row := rows[0]
 	for _, v := range row.Values {
 		table := v.Parameters[0].GetS()
+
 		var stmt string
 
 		if table == "sqlite_sequence" {
@@ -1322,7 +1436,7 @@ func (db *DB) Dump(w io.Writer) error {
 			columnNames = append(columnNames, fmt.Sprintf(`'||quote("%s")||'`, w.Parameters[1].GetS()))
 		}
 
-		query = fmt.Sprintf(`SELECT 'INSERT INTO "%s" VALUES(%s)' FROM "%s";`,
+		query := fmt.Sprintf(`SELECT 'INSERT INTO "%s" VALUES(%s)' FROM "%s";`,
 			tableIndent,
 			strings.Join(columnNames, ","),
 			tableIndent)
@@ -1340,7 +1454,7 @@ func (db *DB) Dump(w io.Writer) error {
 	}
 
 	// Do indexes, triggers, and views.
-	query = `SELECT "name", "type", "sql" FROM "sqlite_master"
+	query := `SELECT "name", "type", "sql" FROM "sqlite_master"
 			  WHERE "sql" NOT NULL AND "type" IN ('index', 'trigger', 'view')`
 	rows, err = db.queryWithConn(ctx, commReq(query), false, conn)
 	if err != nil {
@@ -1348,6 +1462,8 @@ func (db *DB) Dump(w io.Writer) error {
 	}
 	row = rows[0]
 	for _, v := range row.Values {
+		// For indexes, triggers, and views, we could add more sophisticated filtering
+		// based on the table they relate to, but for now include all of them
 		if _, err := w.Write([]byte(fmt.Sprintf("%s;\n", v.Parameters[2].GetS()))); err != nil {
 			return err
 		}
@@ -1373,7 +1489,7 @@ func (db *DB) StmtReadOnly(sql string) (bool, error) {
 // the given connection.
 func (db *DB) StmtReadOnlyWithConn(sql string, conn *sql.Conn) (bool, error) {
 	var readOnly bool
-	f := func(driverConn interface{}) error {
+	f := func(driverConn any) error {
 		c := driverConn.(*sqlite3.SQLiteConn)
 		drvStmt, err := c.Prepare(sql)
 		if err != nil {
@@ -1391,13 +1507,13 @@ func (db *DB) StmtReadOnlyWithConn(sql string, conn *sql.Conn) (bool, error) {
 	return readOnly, nil
 }
 
-func (db *DB) pragmas() (map[string]interface{}, error) {
+func (db *DB) pragmas() (map[string]any, error) {
 	conns := map[string]*sql.DB{
 		"rw": db.rwDB,
 		"ro": db.roDB,
 	}
 
-	connsMap := make(map[string]interface{})
+	connsMap := make(map[string]any)
 	for k, v := range conns {
 		pragmasMap := make(map[string]string)
 		for _, p := range []string{
@@ -1418,6 +1534,36 @@ func (db *DB) pragmas() (map[string]interface{}, error) {
 	return connsMap, nil
 }
 
+// txStatus returns whether there is an active transaction on each database
+// connection.
+func (db *DB) txStatus() (map[string]any, error) {
+	conns := map[string]*sql.DB{
+		"rw": db.rwDB,
+		"ro": db.roDB,
+	}
+
+	t := false
+	f := func(driverConn any) error {
+		conn := driverConn.(*sqlite3.SQLiteConn)
+		t = conn.AutoCommit()
+		return nil
+	}
+
+	txStatusMap := make(map[string]any)
+	for k, v := range conns {
+		conn, err := v.Conn(context.Background())
+		if err != nil {
+			return nil, err
+		}
+		defer conn.Close()
+		if err := conn.Raw(f); err != nil {
+			return nil, err
+		}
+		txStatusMap[k] = !t
+	}
+	return txStatusMap, nil
+}
+
 func (db *DB) memStats() (map[string]int64, error) {
 	ms := make(map[string]int64)
 	for _, p := range []string{
@@ -1434,7 +1580,7 @@ func (db *DB) memStats() (map[string]int64, error) {
 			return nil, err
 		}
 		if res[0].Error != "" {
-			return nil, fmt.Errorf(res[0].Error)
+			return nil, errors.New(res[0].Error)
 		}
 		ms[p] = res[0].Values[0].Parameters[0].GetI()
 	}
@@ -1477,12 +1623,12 @@ func copyDatabase(dst *DB, src *DB) error {
 
 	var dstSQLiteConn *sqlite3.SQLiteConn
 
-	bf := func(driverConn interface{}) error {
+	bf := func(driverConn any) error {
 		srcSQLiteConn := driverConn.(*sqlite3.SQLiteConn)
 		return copyDatabaseConnection(dstSQLiteConn, srcSQLiteConn)
 	}
 	return dstConn.Raw(
-		func(driverConn interface{}) error {
+		func(driverConn any) error {
 			dstSQLiteConn = driverConn.(*sqlite3.SQLiteConn)
 			return srcConn.Raw(bf)
 		})
@@ -1512,12 +1658,12 @@ func copyDatabaseConnection(dst, src *sqlite3.SQLiteConn) error {
 }
 
 // parametersToValues maps values in the proto params to SQL driver values.
-func parametersToValues(parameters []*command.Parameter) ([]interface{}, error) {
+func parametersToValues(parameters []*command.Parameter) ([]any, error) {
 	if parameters == nil {
 		return nil, nil
 	}
 
-	values := make([]interface{}, len(parameters))
+	values := make([]any, len(parameters))
 	for i := range parameters {
 		switch w := parameters[i].GetValue().(type) {
 		case *command.Parameter_I:
@@ -1571,7 +1717,7 @@ func populateEmptyTypes(types []string, params []*command.Parameter) error {
 // Text values come over (from sqlite-go) as []byte instead of strings
 // for some reason, so we have explicitly converted (but only when type
 // is "text" so we don't affect BLOB types)
-func normalizeRowParameters(row []interface{}, types []string) ([]*command.Parameter, error) {
+func normalizeRowParameters(row []any, types []string) ([]*command.Parameter, error) {
 	values := make([]*command.Parameter, len(types))
 	for i, v := range row {
 		switch val := v.(type) {
