@@ -10,8 +10,8 @@ import (
 	"path/filepath"
 	"testing"
 
-	"github.com/rqlite/go-sqlite3"
-	"github.com/rqlite/rqlite/v8/random"
+	"github.com/mattn/go-sqlite3"
+	"github.com/rqlite/rqlite/v8/internal/random"
 )
 
 func Test_IsDisallowedPragmas(t *testing.T) {
@@ -57,6 +57,7 @@ func Test_AllowedPragmas(t *testing.T) {
 		"foreign keys",
 		"optimize",
 		"FOO PRAGMA main.synchronous=OFF",
+		`SELECT * FROM foo WHERE s="PRAGMA wal_autocheckpoint = 1000"`,
 	}
 
 	for _, s := range tests {
