@@ -27,8 +27,8 @@ import (
 	command "github.com/rqlite/rqlite/v8/command/proto"
 	"github.com/rqlite/rqlite/v8/command/sql"
 	"github.com/rqlite/rqlite/v8/db"
+	"github.com/rqlite/rqlite/v8/internal/rtls"
 	"github.com/rqlite/rqlite/v8/queue"
-	"github.com/rqlite/rqlite/v8/rtls"
 	"github.com/rqlite/rqlite/v8/store"
 )
 
@@ -37,6 +37,8 @@ var (
 	ErrLeaderNotFound = errors.New("leader not found")
 )
 
+const kubernetesServiceHostEnv = "KUBERNETES_SERVICE_HOST"
+
 type ResultsError interface {
 	Error() string
 	IsAuthorized() bool
@@ -48,17 +50,17 @@ type Database interface {
 	// to return rows. If timings is true, then timing information will
 	// be return. If tx is true, then either all queries will be executed
 	// successfully or it will as though none executed.
-	Execute(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error)
+	Execute(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error)
 
 	// Query executes a slice of queries, each of which returns rows. If
 	// timings is true, then timing information will be returned. If tx
 	// is true, then all queries will take place while a read transaction
 	// is held on the database.
-	Query(qr *command.QueryRequest) ([]*command.QueryRows, error)
+	Query(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error)
 
 	// Request processes a slice of requests, each of which can be either
 	// an Execute or Query request.
-	Request(eqr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, error)
+	Request(eqr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, uint64, error)
 
 	// Load loads a SQLite file into the system via Raft consensus.
 	Load(lr *command.LoadRequest) error
@@ -68,24 +70,24 @@ type Database interface {
 type Store interface {
 	Database
 
-	// Remove removes the node from the cluster.
-	Remove(rn *command.RemoveNodeRequest) error
+	// Leader returns the Leader of the cluster
+	Leader() (*store.Server, error)
 
-	// LeaderAddr returns the Raft address of the leader of the cluster.
-	LeaderAddr() (string, error)
+	// Nodes returns the slice of store.Servers in the cluster
+	Nodes() ([]*store.Server, error)
 
 	// Ready returns whether the Store is ready to service requests.
 	Ready() bool
 
+	// Remove removes the node from the cluster.
+	Remove(rn *command.RemoveNodeRequest) error
+
 	// Committed blocks until the local commit index is greater than or
 	// equal to the Leader index, as checked when the function is called.
 	Committed(timeout time.Duration) (uint64, error)
 
 	// Stats returns stats on the Store.
-	Stats() (map[string]interface{}, error)
-
-	// Nodes returns the slice of store.Servers in the cluster
-	Nodes() ([]*store.Server, error)
+	Stats() (map[string]any, error)
 
 	// Backup writes backup of the node state to dst
 	Backup(br *command.BackupRequest, dst io.Writer) error
@@ -97,6 +99,11 @@ type Store interface {
 	// the Raft system. It then triggers a Raft snapshot, which will then make
 	// Raft aware of the new data.
 	ReadFrom(r io.Reader) (int64, error)
+
+	// Stepdown forces this node to relinquish leadership to another node in
+	// the cluster. If id is non-empty, leadership will be transferred to the
+	// node with the given ID.
+	Stepdown(wait bool, id string) error
 }
 
 // GetNodeMetaer is the interface that wraps the GetNodeMeta method.
@@ -110,13 +117,13 @@ type Cluster interface {
 	GetNodeMetaer
 
 	// Execute performs an Execute Request on a remote node.
-	Execute(er *command.ExecuteRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration, retries int) ([]*command.ExecuteQueryResponse, error)
+	Execute(er *command.ExecuteRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration, retries int) ([]*command.ExecuteQueryResponse, uint64, error)
 
 	// Query performs an Query Request on a remote node.
-	Query(qr *command.QueryRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration) ([]*command.QueryRows, error)
+	Query(qr *command.QueryRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration) ([]*command.QueryRows, uint64, error)
 
 	// Request performs an ExecuteQuery Request on a remote node.
-	Request(eqr *command.ExecuteQueryRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration, retries int) ([]*command.ExecuteQueryResponse, error)
+	Request(eqr *command.ExecuteQueryRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration, retries int) ([]*command.ExecuteQueryResponse, uint64, error)
 
 	// Backup retrieves a backup from a remote node and writes to the io.Writer.
 	Backup(br *command.BackupRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration, w io.Writer) error
@@ -127,8 +134,11 @@ type Cluster interface {
 	// RemoveNode removes a node from the cluster.
 	RemoveNode(rn *command.RemoveNodeRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration) error
 
+	// Stepdown triggers leader stepdown on a remote node.
+	Stepdown(sr *command.StepdownRequest, nodeAddr string, creds *clstrPB.Credentials, timeout time.Duration) error
+
 	// Stats returns stats on the Cluster.
-	Stats() (map[string]interface{}, error)
+	Stats() (map[string]any, error)
 }
 
 // CredentialStore is the interface credential stores must support.
@@ -139,7 +149,7 @@ type CredentialStore interface {
 
 // StatusReporter is the interface status providers must implement.
 type StatusReporter interface {
-	Stats() (map[string]interface{}, error)
+	Stats() (map[string]any, error)
 }
 
 // DBResults stores either an Execute result, a Query result, or
@@ -169,7 +179,7 @@ func (d *DBResults) MarshalJSON() ([]byte, error) {
 	} else if d.ExecuteQueryResponse != nil {
 		return enc.JSONMarshal(d.ExecuteQueryResponse)
 	}
-	return json.Marshal(make([]interface{}, 0))
+	return json.Marshal(make([]any, 0))
 }
 
 // Response represents a response from the HTTP service.
@@ -178,6 +188,7 @@ type Response struct {
 	Error       string     `json:"error,omitempty"`
 	Time        float64    `json:"time,omitempty"`
 	SequenceNum int64      `json:"sequence_number,omitempty"`
+	RaftIndex   uint64     `json:"raft_index,omitempty"`
 
 	start time.Time
 	end   time.Time
@@ -228,6 +239,7 @@ const (
 	numRemoteBackups                  = "remote_backups"
 	numRemoteLoads                    = "remote_loads"
 	numRemoteRemoveNode               = "remote_remove_node"
+	numRemoteStepdowns                = "remote_stepdowns"
 	numReadyz                         = "num_readyz"
 	numStatus                         = "num_status"
 	numBackups                        = "backups"
@@ -235,8 +247,9 @@ const (
 	numLoadAborted                    = "loads_aborted"
 	numBoot                           = "boot"
 	numSnapshots                      = "user_snapshots"
-	numAuthOK                         = "authOK"
-	numAuthFail                       = "authFail"
+	numAuthOK                         = "auth_ok"
+	numAuthFail                       = "auth_fail"
+	numTLSCertFetched                 = "tls_cert_fetched"
 
 	// Default timeout for cluster communications.
 	defaultTimeout = 30 * time.Second
@@ -300,6 +313,7 @@ func ResetStats() {
 	stats.Add(numRemoteBackups, 0)
 	stats.Add(numRemoteLoads, 0)
 	stats.Add(numRemoteRemoveNode, 0)
+	stats.Add(numRemoteStepdowns, 0)
 	stats.Add(numReadyz, 0)
 	stats.Add(numStatus, 0)
 	stats.Add(numBackups, 0)
@@ -309,6 +323,7 @@ func ResetStats() {
 	stats.Add(numSnapshots, 0)
 	stats.Add(numAuthOK, 0)
 	stats.Add(numAuthFail, 0)
+	stats.Add(numTLSCertFetched, 0)
 }
 
 // Service provides HTTP service.
@@ -335,6 +350,7 @@ type Service struct {
 	CertFile     string // Path to server's own x509 certificate.
 	KeyFile      string // Path to server's own x509 private key.
 	ClientVerify bool   // Whether client certificates should verified.
+	certReloader *rtls.CertReloader
 	tlsConfig    *tls.Config
 
 	aoMu        sync.RWMutex
@@ -349,7 +365,7 @@ type Service struct {
 
 	credentialStore CredentialStore
 
-	BuildInfo map[string]interface{}
+	BuildInfo map[string]any
 
 	logger *log.Logger
 }
@@ -389,7 +405,18 @@ func (s *Service) Start() error {
 		if s.ClientVerify {
 			mTLSState = rtls.MTLSStateEnabled
 		}
-		s.tlsConfig, err = rtls.CreateServerConfig(s.CertFile, s.KeyFile, s.CACertFile, mTLSState)
+		s.certReloader, err = rtls.NewCertReloader(s.CertFile, s.KeyFile)
+		if err != nil {
+			return err
+		}
+
+		// Wrap the GetCertificate function so we update the stats.
+		getCertFunc := func() (*tls.Certificate, error) {
+			stats.Add(numTLSCertFetched, 1)
+			return s.certReloader.GetCertificate()
+		}
+
+		s.tlsConfig, err = rtls.CreateServerConfigWithFunc(getCertFunc, s.CACertFile, mTLSState)
 		if err != nil {
 			return err
 		}
@@ -407,7 +434,6 @@ func (s *Service) Start() error {
 		} else {
 			b.WriteString(", mutual TLS disabled")
 		}
-		// print the message
 		s.logger.Println(b.String())
 	}
 	s.ln = ln
@@ -513,6 +539,8 @@ func (s *Service) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 		s.handleStatus(w, r, params)
 	case strings.HasPrefix(r.URL.Path, "/nodes"):
 		s.handleNodes(w, r, params)
+	case r.URL.Path == "/leader":
+		s.handleLeader(w, r, params)
 	case strings.HasPrefix(r.URL.Path, "/readyz"):
 		stats.Add(numReadyz, 1)
 		s.handleReadyz(w, r, params)
@@ -534,7 +562,6 @@ func (s *Service) RegisterStatus(key string, stat StatusReporter) error {
 		return fmt.Errorf("status already registered with key %s", key)
 	}
 	s.statuses[key] = stat
-
 	return nil
 }
 
@@ -583,25 +610,19 @@ func (s *Service) handleRemove(w http.ResponseWriter, r *http.Request, qp QueryP
 				return
 			}
 
-			addr, err := s.store.LeaderAddr()
+			addr, err := s.LeaderAddr()
 			if err != nil {
-				http.Error(w, fmt.Sprintf("leader address: %s", err.Error()),
-					http.StatusInternalServerError)
+				if errors.Is(err, ErrLeaderNotFound) {
+					stats.Add(numLeaderNotFound, 1)
+					http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+					return
+				}
+				http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
 				return
 			}
-			if addr == "" {
-				stats.Add(numLeaderNotFound, 1)
-				http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
-				return
-			}
-
-			username, password, ok := r.BasicAuth()
-			if !ok {
-				username = ""
-			}
 
 			w.Header().Add(ServedByHTTPHeader, addr)
-			removeErr := s.cluster.RemoveNode(rn, addr, makeCredentials(username, password), qp.Timeout(defaultTimeout))
+			removeErr := s.cluster.RemoveNode(rn, addr, makeCredentials(r), qp.Timeout(defaultTimeout))
 			if removeErr != nil {
 				if removeErr.Error() == "unauthorized" {
 					http.Error(w, "remote remove node not authorized", http.StatusUnauthorized)
@@ -635,6 +656,7 @@ func (s *Service) handleBackup(w http.ResponseWriter, r *http.Request, qp QueryP
 		Leader:   !qp.NoLeader(),
 		Vacuum:   qp.Vacuum(),
 		Compress: qp.Compress(),
+		Tables:   qp.Tables(),
 	}
 	addBackupFormatHeader(w, qp)
 
@@ -645,25 +667,19 @@ func (s *Service) handleBackup(w http.ResponseWriter, r *http.Request, qp QueryP
 				return
 			}
 
-			addr, err := s.store.LeaderAddr()
+			addr, err := s.LeaderAddr()
 			if err != nil {
-				http.Error(w, fmt.Sprintf("leader address: %s", err.Error()),
-					http.StatusInternalServerError)
+				if errors.Is(err, ErrLeaderNotFound) {
+					stats.Add(numLeaderNotFound, 1)
+					http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+					return
+				}
+				http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
 				return
 			}
-			if addr == "" {
-				stats.Add(numLeaderNotFound, 1)
-				http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
-				return
-			}
-
-			username, password, ok := r.BasicAuth()
-			if !ok {
-				username = ""
-			}
 
 			w.Header().Add(ServedByHTTPHeader, addr)
-			backupErr := s.cluster.Backup(br, addr, makeCredentials(username, password), qp.Timeout(defaultTimeout), w)
+			backupErr := s.cluster.Backup(br, addr, makeCredentials(r), qp.Timeout(defaultTimeout), w)
 			if backupErr != nil {
 				if backupErr.Error() == "unauthorized" {
 					http.Error(w, "remote backup not authorized", http.StatusUnauthorized)
@@ -697,6 +713,26 @@ func (s *Service) handleLoad(w http.ResponseWriter, r *http.Request, qp QueryPar
 		return
 	}
 
+	// Determine some perhaps-needed details.
+	ldrAddr, err := s.LeaderAddr()
+	if err != nil {
+		if errors.Is(err, ErrLeaderNotFound) {
+			stats.Add(numLeaderNotFound, 1)
+			http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+			return
+		}
+		http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
+		return
+	}
+
+	handleRemoteErr := func(err error) {
+		if err.Error() == "unauthorized" {
+			http.Error(w, "remote load not authorized", http.StatusUnauthorized)
+		} else {
+			http.Error(w, err.Error(), http.StatusInternalServerError)
+		}
+	}
+
 	resp := NewResponse()
 	b, err := io.ReadAll(r.Body)
 	if err != nil {
@@ -720,32 +756,10 @@ func (s *Service) handleLoad(w http.ResponseWriter, r *http.Request, qp QueryPar
 				return
 			}
 
-			addr, err := s.store.LeaderAddr()
-			if err != nil {
-				http.Error(w, fmt.Sprintf("leader address: %s", err.Error()),
-					http.StatusInternalServerError)
-				return
-			}
-			if addr == "" {
-				stats.Add(numLeaderNotFound, 1)
-				http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
-				return
-			}
-
-			username, password, ok := r.BasicAuth()
-			if !ok {
-				username = ""
-			}
-
-			w.Header().Add(ServedByHTTPHeader, addr)
-			loadErr := s.cluster.Load(lr, addr, makeCredentials(username, password),
-				qp.Timeout(defaultTimeout), qp.Retries(0))
+			w.Header().Add(ServedByHTTPHeader, ldrAddr)
+			loadErr := s.cluster.Load(lr, ldrAddr, makeCredentials(r), qp.Timeout(defaultTimeout), qp.Retries(0))
 			if loadErr != nil {
-				if loadErr.Error() == "unauthorized" {
-					http.Error(w, "remote load not authorized", http.StatusUnauthorized)
-				} else {
-					http.Error(w, loadErr.Error(), http.StatusInternalServerError)
-				}
+				handleRemoteErr(loadErr)
 				return
 			}
 			stats.Add(numRemoteLoads, 1)
@@ -753,19 +767,35 @@ func (s *Service) handleLoad(w http.ResponseWriter, r *http.Request, qp QueryPar
 			// forwarding was put in place.
 		}
 	} else {
-		// No JSON structure expected for this API.
+		// No JSON structure expected for this API, just a bunch of SQL statements.
 		queries := []string{string(b)}
 		er := executeRequestFromStrings(queries, qp.Timings(), false)
+		er.Request.RollbackOnError = true
 
-		response, err := s.store.Execute(er)
+		response, _, err := s.store.Execute(er)
 		if err != nil {
 			if err == store.ErrNotLeader {
 				if s.DoRedirect(w, r, qp) {
 					return
 				}
+
+				w.Header().Add(ServedByHTTPHeader, ldrAddr)
+				var exErr error
+				response, _, exErr = s.cluster.Execute(er, ldrAddr, makeCredentials(r),
+					qp.Timeout(defaultTimeout), qp.Retries(0))
+				if exErr != nil {
+					handleRemoteErr(exErr)
+					return
+				}
+				resp.Results.ExecuteQueryResponse = response
+				stats.Add(numRemoteLoads, 1)
+			} else {
+				// Local execute failed for some reason other than not
+				// being the leader. Nothing we can do here.
+				resp.Error = err.Error()
 			}
-			resp.Error = err.Error()
 		} else {
+			// Successful local execute.
 			resp.Results.ExecuteQueryResponse = response
 		}
 		resp.end = time.Now()
@@ -855,16 +885,17 @@ func (s *Service) handleStatus(w http.ResponseWriter, r *http.Request, qp QueryP
 		return
 	}
 
-	rt := map[string]interface{}{
-		"GOARCH":        runtime.GOARCH,
-		"GOOS":          runtime.GOOS,
-		"GOMAXPROCS":    runtime.GOMAXPROCS(0),
-		"num_cpu":       runtime.NumCPU(),
-		"num_goroutine": runtime.NumGoroutine(),
-		"version":       runtime.Version(),
+	rt := map[string]any{
+		"GOARCH":          runtime.GOARCH,
+		"GOOS":            runtime.GOOS,
+		"GOMAXPROCS":      runtime.GOMAXPROCS(0),
+		"num_cpu":         runtime.NumCPU(),
+		"num_goroutine":   runtime.NumGoroutine(),
+		"version":         runtime.Version(),
+		"kubernetes_hint": kubernetesHint(),
 	}
 
-	oss := map[string]interface{}{
+	oss := map[string]any{
 		"pid":       os.Getpid(),
 		"ppid":      os.Getppid(),
 		"page_size": os.Getpagesize(),
@@ -885,10 +916,10 @@ func (s *Service) handleStatus(w http.ResponseWriter, r *http.Request, qp QueryP
 		return
 	}
 	qs["sequence_number"] = atomic.LoadInt64(&s.seqNum)
-	queueStats := map[string]interface{}{
+	queueStats := map[string]any{
 		"_default": qs,
 	}
-	httpStatus := map[string]interface{}{
+	httpStatus := map[string]any{
 		"bind_addr": s.Addr().String(),
 		"auth":      prettyEnabled(s.credentialStore != nil),
 		"cluster":   clusterStatus,
@@ -900,14 +931,14 @@ func (s *Service) handleStatus(w http.ResponseWriter, r *http.Request, qp QueryP
 		httpStatus["allow_origin"] = ao
 	}
 
-	nodeStatus := map[string]interface{}{
+	nodeStatus := map[string]any{
 		"start_time":   s.start,
 		"current_time": time.Now(),
 		"uptime":       time.Since(s.start).String(),
 	}
 
 	// Build the status response.
-	status := map[string]interface{}{
+	status := map[string]any{
 		"os":      oss,
 		"runtime": rt,
 		"store":   storeStatus,
@@ -929,7 +960,7 @@ func (s *Service) handleStatus(w http.ResponseWriter, r *http.Request, qp QueryP
 			stat, err := v.Stats()
 			if err != nil {
 				s.logger.Printf("failed to retrieve stats for registered reporter %s: %s", k, err.Error())
-				stat = map[string]interface{}{"error": err.Error()}
+				stat = map[string]any{"error": err.Error()}
 			}
 			status[k] = stat
 		}
@@ -993,13 +1024,15 @@ func (s *Service) handleNodes(w http.ResponseWriter, r *http.Request, qp QueryPa
 	}
 
 	// Now test the nodes
-	lAddr, err := s.store.LeaderAddr()
-	if err != nil {
-		http.Error(w, fmt.Sprintf("leader address: %s", err.Error()),
-			http.StatusInternalServerError)
+	laddr := ""
+	lNode, err := s.store.Leader()
+	if err != nil && err != store.ErrLeaderNotFound {
+		http.Error(w, fmt.Sprintf("leader: %s", err.Error()), http.StatusInternalServerError)
 		return
+	} else if lNode != nil {
+		laddr = lNode.Addr
 	}
-	nodes.Test(s.cluster, lAddr, qp.Retries(0), qp.Timeout(defaultTimeout))
+	nodes.Test(s.cluster, laddr, qp.Retries(0), qp.Timeout(defaultTimeout))
 
 	enc := NewNodesRespEncoder(w, qp.Version() != "2")
 	if qp.Pretty() {
@@ -1012,6 +1045,122 @@ func (s *Service) handleNodes(w http.ResponseWriter, r *http.Request, qp QueryPa
 	}
 }
 
+// handleLeader returns leader information, or triggers leader stepdown.
+func (s *Service) handleLeader(w http.ResponseWriter, r *http.Request, qp QueryParams) {
+	w.Header().Set("Content-Type", "application/json; charset=utf-8")
+
+	if !s.CheckRequestPerm(r, auth.PermLeaderOps) {
+		w.WriteHeader(http.StatusUnauthorized)
+		return
+	}
+
+	switch r.Method {
+	case "GET":
+		// Return leader information
+		laddr := ""
+		ldr, err := s.store.Leader()
+		if err != nil {
+			if err == store.ErrLeaderNotFound || err == store.ErrNotOpen {
+				http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+			} else {
+				http.Error(w, fmt.Sprintf("leader: %s", err.Error()), http.StatusInternalServerError)
+			}
+			return
+		}
+		if ldr != nil {
+			laddr = ldr.Addr
+		}
+
+		node := NewNodeFromServer(ldr)
+		if node == nil {
+			http.Error(w, "leader node", http.StatusInternalServerError)
+			return
+		}
+		node.Test(s.cluster, laddr, qp.Retries(0), qp.Timeout(defaultTimeout))
+
+		b, err := func() ([]byte, error) {
+			if qp.Pretty() {
+				return json.MarshalIndent(node, "", "    ")
+			}
+			return json.Marshal(node)
+		}()
+		if err != nil {
+			http.Error(w, fmt.Sprintf("JSON marshal: %s", err.Error()), http.StatusInternalServerError)
+			return
+		}
+		_, err = w.Write(b)
+		if err != nil {
+			s.logger.Println("failed to write leader node to client", err.Error())
+			return
+		}
+
+	case "POST":
+		// Trigger leader stepdown
+		wait := qp.Wait()
+
+		// Parse optional JSON body for target node ID
+		var nodeID string
+		if r.Header.Get("Content-Type") == "application/json" && r.ContentLength > 0 {
+			var reqBody struct {
+				ID string `json:"id"`
+			}
+			if err := json.NewDecoder(r.Body).Decode(&reqBody); err != nil {
+				http.Error(w, fmt.Sprintf("invalid JSON: %s", err.Error()), http.StatusBadRequest)
+				return
+			}
+			nodeID = reqBody.ID
+		}
+
+		if err := s.store.Stepdown(wait, nodeID); err != nil {
+			if err == store.ErrNotLeader {
+				if s.DoRedirect(w, r, qp) {
+					return
+				}
+
+				addr, err := s.LeaderAddr()
+				if err != nil {
+					if errors.Is(err, ErrLeaderNotFound) {
+						stats.Add(numLeaderNotFound, 1)
+						http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+						return
+					}
+					http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
+					return
+				}
+
+				w.Header().Add(ServedByHTTPHeader, addr)
+				sr := &command.StepdownRequest{
+					Id:   nodeID,
+					Wait: wait,
+				}
+				stepdownErr := s.cluster.Stepdown(sr, addr, makeCredentials(r), qp.Timeout(defaultTimeout))
+				if stepdownErr != nil {
+					if stepdownErr.Error() == "unauthorized" {
+						http.Error(w, "remote stepdown not authorized", http.StatusUnauthorized)
+					} else {
+						http.Error(w, stepdownErr.Error(), http.StatusInternalServerError)
+					}
+					return
+				}
+				stats.Add(numRemoteStepdowns, 1)
+				w.WriteHeader(http.StatusOK)
+				return
+			}
+
+			statusCode := http.StatusInternalServerError
+			if err == store.ErrNotOpen {
+				statusCode = http.StatusServiceUnavailable
+			}
+			http.Error(w, fmt.Sprintf("stepdown: %s", err.Error()), statusCode)
+			return
+		}
+		w.WriteHeader(http.StatusOK)
+
+	default:
+		w.WriteHeader(http.StatusMethodNotAllowed)
+	}
+}
+
 // handleReadyz returns whether the node is ready.
 func (s *Service) handleReadyz(w http.ResponseWriter, r *http.Request, qp QueryParams) {
 	if !s.CheckRequestPerm(r, auth.PermReady) {
@@ -1031,16 +1180,14 @@ func (s *Service) handleReadyz(w http.ResponseWriter, r *http.Request, qp QueryP
 		return
 	}
 
-	lAddr, err := s.store.LeaderAddr()
+	lAddr, err := s.LeaderAddr()
 	if err != nil {
-		http.Error(w, fmt.Sprintf("leader address: %s", err.Error()),
-			http.StatusInternalServerError)
-		return
-	}
-
-	if lAddr == "" {
-		w.WriteHeader(http.StatusServiceUnavailable)
-		w.Write([]byte("[+]node ok\n[+]leader does not exist"))
+		if errors.Is(err, ErrLeaderNotFound) {
+			w.WriteHeader(http.StatusServiceUnavailable)
+			w.Write([]byte("[+]node ok\n[+]leader does not exist"))
+			return
+		}
+		http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
 		return
 	}
 
@@ -1098,10 +1245,14 @@ func (s *Service) queuedExecute(w http.ResponseWriter, r *http.Request, qp Query
 	// Perform a leader check, unless disabled. This prevents generating queued writes on
 	// a node that does not appear to be connected to a cluster (even a single-node cluster).
 	if !qp.NoLeader() {
-		addr, err := s.store.LeaderAddr()
-		if err != nil || addr == "" {
-			stats.Add(numLeaderNotFound, 1)
-			http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+		_, err := s.LeaderAddr()
+		if err != nil {
+			if errors.Is(err, ErrLeaderNotFound) {
+				stats.Add(numLeaderNotFound, 1)
+				http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+				return
+			}
+			http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
 			return
 		}
 	}
@@ -1188,31 +1339,25 @@ func (s *Service) execute(w http.ResponseWriter, r *http.Request, qp QueryParams
 		Timings: qp.Timings(),
 	}
 
-	results, resultsErr := s.store.Execute(er)
+	results, raftIndex, resultsErr := s.store.Execute(er)
 	if resultsErr != nil && resultsErr == store.ErrNotLeader {
 		if s.DoRedirect(w, r, qp) {
 			return
 		}
 
-		addr, err := s.store.LeaderAddr()
+		addr, err := s.LeaderAddr()
 		if err != nil {
-			http.Error(w, fmt.Sprintf("leader address: %s", err.Error()),
-				http.StatusInternalServerError)
+			if errors.Is(err, ErrLeaderNotFound) {
+				stats.Add(numLeaderNotFound, 1)
+				http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+				return
+			}
+			http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
 			return
 		}
-		if addr == "" {
-			stats.Add(numLeaderNotFound, 1)
-			http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
-			return
-		}
-
-		username, password, ok := r.BasicAuth()
-		if !ok {
-			username = ""
-		}
 
 		w.Header().Add(ServedByHTTPHeader, addr)
-		results, resultsErr = s.cluster.Execute(er, addr, makeCredentials(username, password),
+		results, raftIndex, resultsErr = s.cluster.Execute(er, addr, makeCredentials(r),
 			qp.Timeout(defaultTimeout), qp.Retries(0))
 		if resultsErr != nil {
 			stats.Add(numRemoteExecutionsFailed, 1)
@@ -1230,6 +1375,9 @@ func (s *Service) execute(w http.ResponseWriter, r *http.Request, qp QueryParams
 		resp.Error = resultsErr.Error()
 	} else {
 		resp.Results.ExecuteQueryResponse = results
+		if qp.RaftIndex() {
+			resp.RaftIndex = raftIndex
+		}
 	}
 	resp.end = time.Now()
 	s.writeResponse(w, qp, resp)
@@ -1300,29 +1448,25 @@ func (s *Service) handleQuery(w http.ResponseWriter, r *http.Request, qp QueryPa
 		LinearizableTimeout: qp.LinearizableTimeout(defaultLinearTimeout).Nanoseconds(),
 	}
 
-	results, resultsErr := s.store.Query(qr)
+	results, raftIndex, resultsErr := s.store.Query(qr)
 	if resultsErr != nil && resultsErr == store.ErrNotLeader {
 		if s.DoRedirect(w, r, qp) {
 			return
 		}
 
-		addr, err := s.store.LeaderAddr()
+		addr, err := s.LeaderAddr()
 		if err != nil {
-			http.Error(w, err.Error(), http.StatusInternalServerError)
+			if errors.Is(err, ErrLeaderNotFound) {
+				stats.Add(numLeaderNotFound, 1)
+				http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+				return
+			}
+			http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
 			return
 		}
-		if addr == "" {
-			stats.Add(numLeaderNotFound, 1)
-			http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
-			return
-		}
-		username, password, ok := r.BasicAuth()
-		if !ok {
-			username = ""
-		}
 
 		w.Header().Add(ServedByHTTPHeader, addr)
-		results, resultsErr = s.cluster.Query(qr, addr, makeCredentials(username, password), qp.Timeout(defaultTimeout))
+		results, raftIndex, resultsErr = s.cluster.Query(qr, addr, makeCredentials(r), qp.Timeout(defaultTimeout))
 		if resultsErr != nil {
 			stats.Add(numRemoteQueriesFailed, 1)
 			if resultsErr.Error() == "unauthorized" {
@@ -1339,6 +1483,9 @@ func (s *Service) handleQuery(w http.ResponseWriter, r *http.Request, qp QueryPa
 		resp.Error = resultsErr.Error()
 	} else {
 		resp.Results.QueryRows = results
+		if qp.RaftIndex() {
+			resp.RaftIndex = raftIndex
+		}
 	}
 	resp.end = time.Now()
 	s.writeResponse(w, qp, resp)
@@ -1387,29 +1534,25 @@ func (s *Service) handleRequest(w http.ResponseWriter, r *http.Request, qp Query
 		FreshnessStrict: qp.FreshnessStrict(),
 	}
 
-	results, resultsErr := s.store.Request(eqr)
+	results, raftIndex, resultsErr := s.store.Request(eqr)
 	if resultsErr != nil && resultsErr == store.ErrNotLeader {
 		if s.DoRedirect(w, r, qp) {
 			return
 		}
 
-		addr, err := s.store.LeaderAddr()
+		addr, err := s.LeaderAddr()
 		if err != nil {
-			http.Error(w, err.Error(), http.StatusInternalServerError)
+			if errors.Is(err, ErrLeaderNotFound) {
+				stats.Add(numLeaderNotFound, 1)
+				http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
+				return
+			}
+			http.Error(w, fmt.Sprintf("leader address: %s", err.Error()), http.StatusInternalServerError)
 			return
 		}
-		if addr == "" {
-			stats.Add(numLeaderNotFound, 1)
-			http.Error(w, ErrLeaderNotFound.Error(), http.StatusServiceUnavailable)
-			return
-		}
-		username, password, ok := r.BasicAuth()
-		if !ok {
-			username = ""
-		}
 
 		w.Header().Add(ServedByHTTPHeader, addr)
-		results, resultsErr = s.cluster.Request(eqr, addr, makeCredentials(username, password),
+		results, raftIndex, resultsErr = s.cluster.Request(eqr, addr, makeCredentials(r),
 			qp.Timeout(defaultTimeout), qp.Retries(0))
 		if resultsErr != nil {
 			stats.Add(numRemoteRequestsFailed, 1)
@@ -1427,6 +1570,9 @@ func (s *Service) handleRequest(w http.ResponseWriter, r *http.Request, qp Query
 		resp.Error = resultsErr.Error()
 	} else {
 		resp.Results.ExecuteQueryResponse = results
+		if qp.RaftIndex() {
+			resp.RaftIndex = raftIndex
+		}
 	}
 	resp.end = time.Now()
 	s.writeResponse(w, qp, resp)
@@ -1564,14 +1710,26 @@ func (s *Service) CheckRequestPermAll(r *http.Request, perms ...string) (b bool)
 	return true
 }
 
-// LeaderAPIAddr returns the API address of the leader, as known by this node.
+// LeaderAddr returns the Raft address of the leader, as known by this node.
+func (s *Service) LeaderAddr() (string, error) {
+	ldr, err := s.store.Leader()
+	if err != nil {
+		return "", err
+	}
+	if ldr.Addr == "" {
+		return "", ErrLeaderNotFound
+	}
+	return ldr.Addr, nil
+}
+
+// LeaderAPIAddr returns the HTTP API address of the leader, as known by this node.
 func (s *Service) LeaderAPIAddr() string {
-	nodeAddr, err := s.store.LeaderAddr()
+	ldr, err := s.store.Leader()
 	if err != nil {
 		return ""
 	}
 
-	meta, err := s.cluster.GetNodeMeta(nodeAddr, 0, defaultTimeout)
+	meta, err := s.cluster.GetNodeMeta(ldr.Addr, 0, defaultTimeout)
 	if err != nil {
 		return ""
 	}
@@ -1600,20 +1758,20 @@ func (s *Service) runQueue() {
 			// a "checkpoint" through the queue.
 			if er.Request.Statements != nil {
 				for {
-					_, err = s.store.Execute(er)
+					_, _, err = s.store.Execute(er)
 					if err == nil {
 						// Success!
 						break
 					}
 
 					if err == store.ErrNotLeader {
-						addr, err := s.store.LeaderAddr()
-						if err != nil || addr == "" {
+						ldr, err := s.store.Leader()
+						if err != nil || ldr.Addr == "" {
 							s.logger.Printf("execute queue can't find leader for sequence number %d on node %s",
 								req.SequenceNumber, s.Addr().String())
 							stats.Add(numQueuedExecutionsNoLeader, 1)
 						} else {
-							_, err = s.cluster.Execute(er, addr, nil, defaultTimeout, 0)
+							_, _, err = s.cluster.Execute(er, ldr.Addr, nil, defaultTimeout, 0)
 							if err != nil {
 								s.logger.Printf("execute queue write failed for sequence number %d on node %s: %s",
 									req.SequenceNumber, s.Addr().String(), err.Error())
@@ -1648,7 +1806,6 @@ func (s *Service) runQueue() {
 
 // addBuildVersion adds the build version to the HTTP response.
 func (s *Service) addBuildVersion(w http.ResponseWriter) {
-	// Add version header to every response, if available.
 	version := "unknown"
 	if v, ok := s.BuildInfo["version"].(string); ok {
 		version = v
@@ -1681,8 +1838,8 @@ func addBackupFormatHeader(w http.ResponseWriter, qp QueryParams) {
 }
 
 // tlsStats returns the TLS stats for the service.
-func (s *Service) tlsStats() map[string]interface{} {
-	m := map[string]interface{}{
+func (s *Service) tlsStats() map[string]any {
+	m := map[string]any{
 		"enabled": fmt.Sprintf("%t", s.tlsConfig != nil),
 	}
 	if s.tlsConfig != nil {
@@ -1736,14 +1893,14 @@ func getSubJSON(jsonBlob []byte, keyString string) (json.RawMessage, error) {
 	}
 
 	keys := strings.Split(keyString, ".")
-	var obj interface{}
+	var obj any
 	if err := json.Unmarshal(jsonBlob, &obj); err != nil {
 		return nil, fmt.Errorf("failed to unmarshal json: %w", err)
 	}
 
 	for _, key := range keys {
 		switch val := obj.(type) {
-		case map[string]interface{}:
+		case map[string]any:
 			if value, ok := val[key]; ok {
 				obj = value
 			} else {
@@ -1775,6 +1932,11 @@ func prettyEnabled(e bool) string {
 	return "disabled"
 }
 
+func kubernetesHint() bool {
+	_, ok := os.LookupEnv(kubernetesServiceHostEnv)
+	return ok
+}
+
 // queryRequestFromStrings converts a slice of strings into a command.QueryRequest
 func executeRequestFromStrings(s []string, timings, tx bool) *command.ExecuteRequest {
 	stmts := make([]*command.Statement, len(s))
@@ -1793,7 +1955,11 @@ func executeRequestFromStrings(s []string, timings, tx bool) *command.ExecuteReq
 	}
 }
 
-func makeCredentials(username, password string) *clstrPB.Credentials {
+func makeCredentials(r *http.Request) *clstrPB.Credentials {
+	username, password, ok := r.BasicAuth()
+	if !ok {
+		return nil
+	}
 	return &clstrPB.Credentials{
 		Username: username,
 		Password: password,
