@@ -3,6 +3,7 @@ package http
 import (
 	"bytes"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
@@ -115,7 +116,7 @@ func Test_HasVersionHeader(t *testing.T) {
 		t.Fatalf("failed to start service")
 	}
 	defer s.Close()
-	s.BuildInfo = map[string]interface{}{
+	s.BuildInfo = map[string]any{
 		"version": "the version",
 	}
 	url := fmt.Sprintf("http://%s", s.Addr().String())
@@ -248,6 +249,48 @@ func Test_HasVersionHeaderUnknown(t *testing.T) {
 	}
 }
 
+func Test_LeaderAddrsOK(t *testing.T) {
+	m := &MockStore{
+		leaderAddr: "foo:1234",
+	}
+	c := &mockClusterService{
+		apiAddr: "http://bar:5678",
+	}
+	s := New("127.0.0.1:0", m, c, nil)
+	if err := s.Start(); err != nil {
+		t.Fatalf("failed to start service")
+	}
+	defer s.Close()
+
+	addr, err := s.LeaderAddr()
+	if err != nil {
+		t.Fatalf("failed to get leader address")
+	}
+	if addr != "foo:1234" {
+		t.Fatalf("incorrect leader address, got: %s", addr)
+	}
+
+	addr = s.LeaderAPIAddr()
+	if addr != "http://bar:5678" {
+		t.Fatalf("incorrect leader API address, got: %s", addr)
+	}
+}
+
+func Test_LeaderAddrsFail(t *testing.T) {
+	m := &MockStore{}
+	c := &mockClusterService{}
+	s := New("127.0.0.1:0", m, c, nil)
+	if err := s.Start(); err != nil {
+		t.Fatalf("failed to start service")
+	}
+	defer s.Close()
+
+	_, err := s.LeaderAddr()
+	if err != ErrLeaderNotFound {
+		t.Fatalf("failed to get expected ErrLeaderNotFound")
+	}
+}
+
 func Test_404Routes(t *testing.T) {
 	m := &MockStore{}
 	c := &mockClusterService{}
@@ -372,6 +415,7 @@ func Test_401Routes_NoBasicAuth(t *testing.T) {
 		"/remove",
 		"/status",
 		"/nodes",
+		"/leader",
 		"/readyz",
 		"/debug/vars",
 		"/debug/pprof/cmdline",
@@ -410,6 +454,7 @@ func Test_401Routes_BasicAuthBadPassword(t *testing.T) {
 		"/boot",
 		"/status",
 		"/nodes",
+		"/leader",
 		"/readyz",
 		"/debug/vars",
 		"/debug/pprof/cmdline",
@@ -456,6 +501,7 @@ func Test_401Routes_BasicAuthBadPerm(t *testing.T) {
 		"/snapshot",
 		"/status",
 		"/nodes",
+		"/leader",
 		"/readyz",
 		"/debug/vars",
 		"/debug/pprof/cmdline",
@@ -708,7 +754,7 @@ func Test_BackupFlagsInvalid(t *testing.T) {
 	}
 }
 
-func Test_LoadOK(t *testing.T) {
+func Test_BackupDeleteOK(t *testing.T) {
 	m := &MockStore{}
 	c := &mockClusterService{}
 	s := New("127.0.0.1:0", m, c, nil)
@@ -717,6 +763,87 @@ func Test_LoadOK(t *testing.T) {
 	}
 	defer s.Close()
 
+	// Track that the backup function is called with DELETE format
+	var capturedRequest *command.BackupRequest
+	m.backupFn = func(br *command.BackupRequest, dst io.Writer) error {
+		capturedRequest = br
+		return nil
+	}
+
+	client := &http.Client{}
+	host := fmt.Sprintf("http://%s", s.Addr().String())
+	resp, err := client.Get(host + "/db/backup?fmt=delete")
+	if err != nil {
+		t.Fatalf("failed to make backup request")
+	}
+	if resp.StatusCode != http.StatusOK {
+		t.Fatalf("failed to get expected StatusOK for backup, got %d", resp.StatusCode)
+	}
+
+	// Verify that the backup request has the DELETE format
+	if capturedRequest == nil {
+		t.Fatalf("backup function was not called")
+	}
+	if capturedRequest.Format != command.BackupRequest_BACKUP_REQUEST_FORMAT_DELETE {
+		t.Fatalf("expected DELETE format, got %v", capturedRequest.Format)
+	}
+}
+
+func Test_BackupTablesOK(t *testing.T) {
+	m := &MockStore{}
+	c := &mockClusterService{}
+	s := New("127.0.0.1:0", m, c, nil)
+	if err := s.Start(); err != nil {
+		t.Fatalf("failed to start service")
+	}
+	defer s.Close()
+
+	// Track that the backup function is called with specified tables
+	var capturedRequest *command.BackupRequest
+	m.backupFn = func(br *command.BackupRequest, dst io.Writer) error {
+		capturedRequest = br
+		return nil
+	}
+
+	client := &http.Client{}
+	host := fmt.Sprintf("http://%s", s.Addr().String())
+	resp, err := client.Get(host + "/db/backup?fmt=sql&tables=users,products")
+	if err != nil {
+		t.Fatalf("failed to make backup request")
+	}
+	if resp.StatusCode != http.StatusOK {
+		t.Fatalf("failed to get expected StatusOK for backup, got %d", resp.StatusCode)
+	}
+
+	// Verify that the backup request has the specified tables
+	if capturedRequest == nil {
+		t.Fatalf("backup function was not called")
+	}
+	if capturedRequest.Format != command.BackupRequest_BACKUP_REQUEST_FORMAT_SQL {
+		t.Fatalf("expected SQL format, got %v", capturedRequest.Format)
+	}
+	expectedTables := []string{"users", "products"}
+	if len(capturedRequest.Tables) != len(expectedTables) {
+		t.Fatalf("expected %d tables, got %d", len(expectedTables), len(capturedRequest.Tables))
+	}
+	for i, table := range expectedTables {
+		if capturedRequest.Tables[i] != table {
+			t.Fatalf("expected table %s at index %d, got %s", table, i, capturedRequest.Tables[i])
+		}
+	}
+}
+
+func Test_LoadOK(t *testing.T) {
+	m := &MockStore{
+		leaderAddr: "foo:1234",
+	}
+	c := &mockClusterService{}
+	s := New("127.0.0.1:0", m, c, nil)
+	if err := s.Start(); err != nil {
+		t.Fatalf("failed to start service")
+	}
+	defer s.Close()
+
 	client := &http.Client{}
 	host := fmt.Sprintf("http://%s", s.Addr().String())
 	resp, err := client.Post(host+"/db/load", "application/octet-stream", strings.NewReader("SELECT"))
@@ -725,7 +852,7 @@ func Test_LoadOK(t *testing.T) {
 	}
 	defer resp.Body.Close()
 	if resp.StatusCode != http.StatusOK {
-		t.Fatalf("failed to get expected StatusOK for load, got %d", resp.StatusCode)
+		t.Fatalf("failed to get expected StatusOK for load, got %d, %s", resp.StatusCode, mustReadBody(t, resp))
 	}
 	if exp, got := `{"results":[]}`, mustReadBody(t, resp); exp != got {
 		t.Fatalf("incorrect response body, exp: %s, got %s", exp, got)
@@ -1151,19 +1278,19 @@ func Test_ForwardingRedirectQuery(t *testing.T) {
 	m := &MockStore{
 		leaderAddr: "foo:1234",
 	}
-	m.queryFn = func(qr *command.QueryRequest) ([]*command.QueryRows, error) {
-		return nil, store.ErrNotLeader
+	m.queryFn = func(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
+		return nil, 0, store.ErrNotLeader
 	}
 
 	c := &mockClusterService{
 		apiAddr: "https://bar:5678",
 	}
-	c.queryFn = func(qr *command.QueryRequest, addr string, timeout time.Duration) ([]*command.QueryRows, error) {
+	c.queryFn = func(qr *command.QueryRequest, addr string, timeout time.Duration) ([]*command.QueryRows, uint64, error) {
 		rows := &command.QueryRows{
 			Columns: []string{},
 			Types:   []string{},
 		}
-		return []*command.QueryRows{rows}, nil
+		return []*command.QueryRows{rows}, 0, nil
 	}
 
 	s := New("127.0.0.1:0", m, c, nil)
@@ -1211,14 +1338,14 @@ func Test_ForwardingRedirectExecute(t *testing.T) {
 	m := &MockStore{
 		leaderAddr: "foo:1234",
 	}
-	m.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error) {
-		return nil, store.ErrNotLeader
+	m.executeFn = func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
+		return nil, 0, store.ErrNotLeader
 	}
 
 	c := &mockClusterService{
 		apiAddr: "https://bar:5678",
 	}
-	c.executeFn = func(er *command.ExecuteRequest, addr string, timeout time.Duration) ([]*command.ExecuteQueryResponse, error) {
+	c.executeFn = func(er *command.ExecuteRequest, addr string, timeout time.Duration) ([]*command.ExecuteQueryResponse, uint64, error) {
 		result := &command.ExecuteQueryResponse{
 			Result: &command.ExecuteQueryResponse_E{
 				E: &command.ExecuteResult{
@@ -1227,7 +1354,7 @@ func Test_ForwardingRedirectExecute(t *testing.T) {
 				},
 			},
 		}
-		return []*command.ExecuteQueryResponse{result}, nil
+		return []*command.ExecuteQueryResponse{result}, 0, nil
 	}
 
 	s := New("127.0.0.1:0", m, c, nil)
@@ -1275,14 +1402,14 @@ func Test_ForwardingRedirectExecuteQuery(t *testing.T) {
 	m := &MockStore{
 		leaderAddr: "foo:1234",
 	}
-	m.requestFn = func(er *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, error) {
-		return nil, store.ErrNotLeader
+	m.requestFn = func(er *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
+		return nil, 0, store.ErrNotLeader
 	}
 
 	c := &mockClusterService{
 		apiAddr: "https://bar:5678",
 	}
-	c.requestFn = func(er *command.ExecuteQueryRequest, addr string, timeout time.Duration) ([]*command.ExecuteQueryResponse, error) {
+	c.requestFn = func(er *command.ExecuteQueryRequest, addr string, timeout time.Duration) ([]*command.ExecuteQueryResponse, uint64, error) {
 		resp := &command.ExecuteQueryResponse{
 			Result: &command.ExecuteQueryResponse_E{
 				E: &command.ExecuteResult{
@@ -1291,7 +1418,7 @@ func Test_ForwardingRedirectExecuteQuery(t *testing.T) {
 				},
 			},
 		}
-		return []*command.ExecuteQueryResponse{resp}, nil
+		return []*command.ExecuteQueryResponse{resp}, 0, nil
 	}
 
 	s := New("127.0.0.1:0", m, c, nil)
@@ -1463,37 +1590,38 @@ func Test_DBTimeoutQueryParam(t *testing.T) {
 }
 
 type MockStore struct {
-	executeFn   func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error)
-	queryFn     func(qr *command.QueryRequest) ([]*command.QueryRows, error)
-	requestFn   func(eqr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, error)
+	executeFn   func(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error)
+	queryFn     func(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error)
+	requestFn   func(eqr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, uint64, error)
 	backupFn    func(br *command.BackupRequest, dst io.Writer) error
 	loadFn      func(lr *command.LoadRequest) error
 	snapshotFn  func(n uint64) error
 	readFromFn  func(r io.Reader) (int64, error)
 	committedFn func(timeout time.Duration) (uint64, error)
+	stepdownFn  func(wait bool, id string) error
 	leaderAddr  string
 	notReady    bool // Default value is true, easier to test.
 }
 
-func (m *MockStore) Execute(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, error) {
+func (m *MockStore) Execute(er *command.ExecuteRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
 	if m.executeFn != nil {
 		return m.executeFn(er)
 	}
-	return nil, nil
+	return nil, 0, nil
 }
 
-func (m *MockStore) Query(qr *command.QueryRequest) ([]*command.QueryRows, error) {
+func (m *MockStore) Query(qr *command.QueryRequest) ([]*command.QueryRows, uint64, error) {
 	if m.queryFn != nil {
 		return m.queryFn(qr)
 	}
-	return nil, nil
+	return nil, 0, nil
 }
 
-func (m *MockStore) Request(eqr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, error) {
+func (m *MockStore) Request(eqr *command.ExecuteQueryRequest) ([]*command.ExecuteQueryResponse, uint64, error) {
 	if m.requestFn != nil {
 		return m.requestFn(eqr)
 	}
-	return nil, nil
+	return nil, 0, nil
 }
 
 func (m *MockStore) Join(jr *command.JoinRequest) error {
@@ -1508,8 +1636,10 @@ func (m *MockStore) Remove(rn *command.RemoveNodeRequest) error {
 	return nil
 }
 
-func (m *MockStore) LeaderAddr() (string, error) {
-	return m.leaderAddr, nil
+func (m *MockStore) Leader() (*store.Server, error) {
+	return &store.Server{
+		Addr: m.leaderAddr,
+	}, nil
 }
 
 func (m *MockStore) Ready() bool {
@@ -1523,7 +1653,7 @@ func (m *MockStore) Committed(timeout time.Duration) (uint64, error) {
 	return 0, nil
 }
 
-func (m *MockStore) Stats() (map[string]interface{}, error) {
+func (m *MockStore) Stats() (map[string]any, error) {
 	return nil, nil
 }
 
@@ -1559,14 +1689,22 @@ func (m *MockStore) ReadFrom(r io.Reader) (int64, error) {
 	return 0, nil
 }
 
+func (m *MockStore) Stepdown(wait bool, id string) error {
+	if m.stepdownFn != nil {
+		return m.stepdownFn(wait, id)
+	}
+	return nil
+}
+
 type mockClusterService struct {
 	apiAddr      string
-	executeFn    func(er *command.ExecuteRequest, addr string, t time.Duration) ([]*command.ExecuteQueryResponse, error)
-	queryFn      func(qr *command.QueryRequest, addr string, t time.Duration) ([]*command.QueryRows, error)
-	requestFn    func(eqr *command.ExecuteQueryRequest, nodeAddr string, timeout time.Duration) ([]*command.ExecuteQueryResponse, error)
+	executeFn    func(er *command.ExecuteRequest, addr string, t time.Duration) ([]*command.ExecuteQueryResponse, uint64, error)
+	queryFn      func(qr *command.QueryRequest, addr string, t time.Duration) ([]*command.QueryRows, uint64, error)
+	requestFn    func(eqr *command.ExecuteQueryRequest, nodeAddr string, timeout time.Duration) ([]*command.ExecuteQueryResponse, uint64, error)
 	backupFn     func(br *command.BackupRequest, addr string, t time.Duration, w io.Writer) error
 	loadFn       func(lr *command.LoadRequest, addr string, t time.Duration) error
 	removeNodeFn func(rn *command.RemoveNodeRequest, nodeAddr string, t time.Duration) error
+	stepdownFn   func(sr *command.StepdownRequest, nodeAddr string, t time.Duration) error
 }
 
 func (m *mockClusterService) GetNodeMeta(a string, r int, t time.Duration) (*cluster.NodeMeta, error) {
@@ -1575,25 +1713,25 @@ func (m *mockClusterService) GetNodeMeta(a string, r int, t time.Duration) (*clu
 	}, nil
 }
 
-func (m *mockClusterService) Execute(er *command.ExecuteRequest, addr string, creds *cluster.Credentials, t time.Duration, r int) ([]*command.ExecuteQueryResponse, error) {
+func (m *mockClusterService) Execute(er *command.ExecuteRequest, addr string, creds *cluster.Credentials, t time.Duration, r int) ([]*command.ExecuteQueryResponse, uint64, error) {
 	if m.executeFn != nil {
 		return m.executeFn(er, addr, t)
 	}
-	return nil, nil
+	return nil, 0, nil
 }
 
-func (m *mockClusterService) Query(qr *command.QueryRequest, addr string, creds *cluster.Credentials, t time.Duration) ([]*command.QueryRows, error) {
+func (m *mockClusterService) Query(qr *command.QueryRequest, addr string, creds *cluster.Credentials, t time.Duration) ([]*command.QueryRows, uint64, error) {
 	if m.queryFn != nil {
 		return m.queryFn(qr, addr, t)
 	}
-	return nil, nil
+	return nil, 0, nil
 }
 
-func (m *mockClusterService) Request(eqr *command.ExecuteQueryRequest, nodeAddr string, creds *cluster.Credentials, timeout time.Duration, r int) ([]*command.ExecuteQueryResponse, error) {
+func (m *mockClusterService) Request(eqr *command.ExecuteQueryRequest, nodeAddr string, creds *cluster.Credentials, timeout time.Duration, r int) ([]*command.ExecuteQueryResponse, uint64, error) {
 	if m.requestFn != nil {
 		return m.requestFn(eqr, nodeAddr, timeout)
 	}
-	return nil, nil
+	return nil, 0, nil
 }
 
 func (m *mockClusterService) Backup(br *command.BackupRequest, addr string, creds *cluster.Credentials, t time.Duration, w io.Writer) error {
@@ -1617,6 +1755,348 @@ func (m *mockClusterService) RemoveNode(rn *command.RemoveNodeRequest, addr stri
 	return nil
 }
 
+func (m *mockClusterService) Stepdown(sr *command.StepdownRequest, addr string, creds *cluster.Credentials, t time.Duration) error {
+	if m.stepdownFn != nil {
+		return m.stepdownFn(sr, addr, t)
+	}
+	return nil
+}
+
+func Test_Leader_GET(t *testing.T) {
+	store := &MockStore{leaderAddr: "127.0.0.1:8001"}
+	cluster := &mockClusterService{apiAddr: "http://127.0.0.1:4001"}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	// Test GET request
+	req, err := http.NewRequest("GET", "/leader", nil)
+	if err != nil {
+		t.Fatalf("failed to create GET request: %s", err.Error())
+	}
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusOK {
+		t.Fatalf("expected 200, got %d", rr.Code)
+	}
+
+	// Ensure the response is well formed.
+	var node Node
+	if err := json.Unmarshal(rr.Body.Bytes(), &node); err != nil {
+		t.Fatalf("failed to unmarshal response body: %s", err.Error())
+	}
+}
+
+func Test_Leader_POST(t *testing.T) {
+	stepdownCalled := false
+	stepdownWait := false
+	store := &MockStore{
+		leaderAddr: "127.0.0.1:8001",
+		stepdownFn: func(wait bool, id string) error {
+			stepdownCalled = true
+			stepdownWait = wait
+			return nil
+		},
+	}
+	cluster := &mockClusterService{apiAddr: "http://127.0.0.1:4001"}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	// Test POST request without wait
+	req, err := http.NewRequest("POST", "/leader", nil)
+	if err != nil {
+		t.Fatalf("failed to create POST request: %s", err.Error())
+	}
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusOK {
+		t.Fatalf("expected 200, got %d", rr.Code)
+	}
+
+	if !stepdownCalled {
+		t.Fatalf("expected stepdown to be called")
+	}
+
+	if stepdownWait {
+		t.Fatalf("expected stepdown to be called with wait=false")
+	}
+
+	// Test POST request with wait
+	stepdownCalled = false
+	stepdownWait = false
+
+	req, err = http.NewRequest("POST", "/leader?wait", nil)
+	if err != nil {
+		t.Fatalf("failed to create POST request with wait: %s", err.Error())
+	}
+	rr = httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusOK {
+		t.Fatalf("expected 200, got %d", rr.Code)
+	}
+
+	if !stepdownCalled {
+		t.Fatalf("expected stepdown to be called")
+	}
+
+	if !stepdownWait {
+		t.Fatalf("expected stepdown to be called with wait=true")
+	}
+}
+
+func Test_LeaderPOST_ForwardToLeader(t *testing.T) {
+	stepdownCalled := false
+	store := &MockStore{
+		leaderAddr: "127.0.0.1:8001",
+		stepdownFn: func(wait bool, id string) error {
+			return store.ErrNotLeader
+		},
+	}
+	cluster := &mockClusterService{
+		apiAddr: "http://127.0.0.1:4001",
+		stepdownFn: func(sr *command.StepdownRequest, nodeAddr string, t time.Duration) error {
+			stepdownCalled = true
+			return nil
+		},
+	}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	req, err := http.NewRequest("POST", "/leader", nil)
+	if err != nil {
+		t.Fatalf("failed to create POST request: %s", err.Error())
+	}
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusOK {
+		t.Fatalf("expected 200, got %d", rr.Code)
+	}
+
+	if !stepdownCalled {
+		t.Fatalf("expected stepdown to be forwarded to leader")
+	}
+}
+
+func Test_LeaderPOST_ForwardError(t *testing.T) {
+	store := &MockStore{
+		leaderAddr: "127.0.0.1:8001",
+		stepdownFn: func(wait bool, id string) error {
+			return store.ErrNotLeader
+		},
+	}
+	cluster := &mockClusterService{
+		apiAddr: "http://127.0.0.1:4001",
+		stepdownFn: func(sr *command.StepdownRequest, nodeAddr string, t time.Duration) error {
+			return errors.New("stepdown failed")
+		},
+	}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	req, err := http.NewRequest("POST", "/leader", nil)
+	if err != nil {
+		t.Fatalf("failed to create POST request: %s", err.Error())
+	}
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusInternalServerError {
+		t.Fatalf("expected 500, got %d", rr.Code)
+	}
+
+	if !strings.Contains(rr.Body.String(), "stepdown failed") {
+		t.Fatalf("expected error message to contain 'stepdown failed', got %s", rr.Body.String())
+	}
+}
+
+func Test_LeaderMethodNotAllowed(t *testing.T) {
+	store := &MockStore{leaderAddr: "127.0.0.1:8001"}
+	cluster := &mockClusterService{apiAddr: "http://127.0.0.1:4001"}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	req, err := http.NewRequest("DELETE", "/leader", nil)
+	if err != nil {
+		t.Fatalf("failed to create DELETE request: %s", err.Error())
+	}
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusMethodNotAllowed {
+		t.Fatalf("expected 405, got %d", rr.Code)
+	}
+}
+
+func Test_Leader_POST_JSON_TargetNode(t *testing.T) {
+	stepdownCalled := false
+	stepdownWait := false
+	stepdownID := ""
+	store := &MockStore{
+		leaderAddr: "127.0.0.1:8001",
+		stepdownFn: func(wait bool, id string) error {
+			stepdownCalled = true
+			stepdownWait = wait
+			stepdownID = id
+			return nil
+		},
+	}
+	cluster := &mockClusterService{apiAddr: "http://127.0.0.1:4001"}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	// Test with JSON body specifying target node
+	reqBody := `{"id": "node1"}`
+	req, err := http.NewRequest("POST", "/leader?wait", strings.NewReader(reqBody))
+	if err != nil {
+		t.Fatalf("failed to create POST request: %s", err.Error())
+	}
+	req.Header.Set("Content-Type", "application/json")
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusOK {
+		t.Fatalf("expected 200, got %d", rr.Code)
+	}
+
+	if !stepdownCalled {
+		t.Fatalf("expected stepdown to be called")
+	}
+
+	if !stepdownWait {
+		t.Fatalf("expected stepdown to be called with wait=true")
+	}
+
+	if stepdownID != "node1" {
+		t.Fatalf("expected stepdown to be called with id='node1', got '%s'", stepdownID)
+	}
+}
+
+func Test_Leader_POST_JSON_EmptyID(t *testing.T) {
+	stepdownCalled := false
+	stepdownID := ""
+	store := &MockStore{
+		leaderAddr: "127.0.0.1:8001",
+		stepdownFn: func(wait bool, id string) error {
+			stepdownCalled = true
+			stepdownID = id
+			return nil
+		},
+	}
+	cluster := &mockClusterService{apiAddr: "http://127.0.0.1:4001"}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	// Test with JSON body but empty ID (should behave like no JSON)
+	reqBody := `{"id": ""}`
+	req, err := http.NewRequest("POST", "/leader", strings.NewReader(reqBody))
+	if err != nil {
+		t.Fatalf("failed to create POST request: %s", err.Error())
+	}
+	req.Header.Set("Content-Type", "application/json")
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusOK {
+		t.Fatalf("expected 200, got %d", rr.Code)
+	}
+
+	if !stepdownCalled {
+		t.Fatalf("expected stepdown to be called")
+	}
+
+	if stepdownID != "" {
+		t.Fatalf("expected stepdown to be called with empty id, got '%s'", stepdownID)
+	}
+}
+
+func Test_Leader_POST_JSON_InvalidJSON(t *testing.T) {
+	store := &MockStore{leaderAddr: "127.0.0.1:8001"}
+	cluster := &mockClusterService{apiAddr: "http://127.0.0.1:4001"}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	// Test with invalid JSON
+	reqBody := `{"id": "node1"`
+	req, err := http.NewRequest("POST", "/leader", strings.NewReader(reqBody))
+	if err != nil {
+		t.Fatalf("failed to create POST request: %s", err.Error())
+	}
+	req.Header.Set("Content-Type", "application/json")
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusBadRequest {
+		t.Fatalf("expected 400, got %d", rr.Code)
+	}
+
+	if !strings.Contains(rr.Body.String(), "invalid JSON") {
+		t.Fatalf("expected error message to contain 'invalid JSON', got %s", rr.Body.String())
+	}
+}
+
+func Test_Leader_POST_JSON_ForwardToLeader(t *testing.T) {
+	stepdownCalled := false
+	var forwardedRequest *command.StepdownRequest
+	store := &MockStore{
+		leaderAddr: "127.0.0.1:8001",
+		stepdownFn: func(wait bool, id string) error {
+			return store.ErrNotLeader
+		},
+	}
+	cluster := &mockClusterService{
+		apiAddr: "http://127.0.0.1:4001",
+		stepdownFn: func(sr *command.StepdownRequest, nodeAddr string, t time.Duration) error {
+			stepdownCalled = true
+			forwardedRequest = sr
+			return nil
+		},
+	}
+	cred := &mockCredentialStore{HasPermOK: true}
+
+	s := New("127.0.0.1:4001", store, cluster, cred)
+
+	// Test forwarding with target node ID
+	reqBody := `{"id": "node1"}`
+	req, err := http.NewRequest("POST", "/leader?wait", strings.NewReader(reqBody))
+	if err != nil {
+		t.Fatalf("failed to create POST request: %s", err.Error())
+	}
+	req.Header.Set("Content-Type", "application/json")
+	rr := httptest.NewRecorder()
+	s.ServeHTTP(rr, req)
+
+	if rr.Code != http.StatusOK {
+		t.Fatalf("expected 200, got %d", rr.Code)
+	}
+
+	if !stepdownCalled {
+		t.Fatalf("expected stepdown to be forwarded to leader")
+	}
+
+	if forwardedRequest == nil {
+		t.Fatalf("expected stepdown request to be forwarded")
+	}
+
+	if forwardedRequest.Id != "node1" {
+		t.Fatalf("expected forwarded request to have id='node1', got '%s'", forwardedRequest.Id)
+	}
+
+	if !forwardedRequest.Wait {
+		t.Fatalf("expected forwarded request to have wait=true")
+	}
+}
+
 type mockCredentialStore struct {
 	HasPermOK bool
 	aaFunc    func(username, password, perm string) bool
@@ -1633,14 +2113,14 @@ func (m *mockCredentialStore) AA(username, password, perm string) bool {
 	return m.HasPermOK
 }
 
-func (m *mockClusterService) Stats() (map[string]interface{}, error) {
+func (m *mockClusterService) Stats() (map[string]any, error) {
 	return nil, nil
 }
 
 type mockStatusReporter struct {
 }
 
-func (m *mockStatusReporter) Stats() (map[string]interface{}, error) {
+func (m *mockStatusReporter) Stats() (map[string]any, error) {
 	return nil, nil
 }
 
