@@ -30,11 +30,11 @@ import (
 	sql "github.com/rqlite/rqlite/v8/db"
 	"github.com/rqlite/rqlite/v8/db/humanize"
 	wal "github.com/rqlite/rqlite/v8/db/wal"
-	rlog "github.com/rqlite/rqlite/v8/log"
-	"github.com/rqlite/rqlite/v8/progress"
-	"github.com/rqlite/rqlite/v8/random"
-	"github.com/rqlite/rqlite/v8/rsync"
+	"github.com/rqlite/rqlite/v8/internal/progress"
+	"github.com/rqlite/rqlite/v8/internal/random"
+	"github.com/rqlite/rqlite/v8/internal/rsync"
 	"github.com/rqlite/rqlite/v8/snapshot"
+	rlog "github.com/rqlite/rqlite/v8/store/log"
 )
 
 var (
@@ -51,6 +51,9 @@ var (
 	// operation.
 	ErrNotLeader = errors.New("not leader")
 
+	// ErrLeaderNotFound is returned when a node cannot locate a leader.
+	ErrLeaderNotFound = errors.New("leader not found")
+
 	// ErrNotSingleNode is returned when a node attempts to execute a single-node
 	// only operation.
 	ErrNotSingleNode = errors.New("not single-node")
@@ -79,6 +82,9 @@ var (
 	// is not valid.
 	ErrInvalidBackupFormat = errors.New("invalid backup format")
 
+	// ErrCDCEnabled is returned when CDC is already enabled.
+	ErrCDCEnabled = errors.New("CDC already enabled")
+
 	// ErrInvalidVacuumFormat is returned when the requested backup format is not
 	// compatible with vacuum.
 	ErrInvalidVacuum = errors.New("invalid vacuum")
@@ -90,30 +96,31 @@ var (
 	// ErrNothingNewToSnapshot is returned when a snapshot is requested but there
 	// are no new log entries to snapshot.
 	ErrNothingNewToSnapshot = errors.New("nothing new to snapshot")
+
+	// ErrNodeNotFound is returned when a node with a given ID is not found in the cluster.
+	ErrNodeNotFound = errors.New("node not found in cluster")
 )
 
 const (
-	snapshotsDirName           = "rsnapshots"
-	restoreScratchPattern      = "rqlite-restore-*"
-	bootScatchPattern          = "rqlite-boot-*"
-	backupScratchPattern       = "rqlite-backup-*"
-	walSnapshotScratchPattern  = "rqlite-wal-snapshot-*"
-	raftDBPath                 = "raft.db" // Changing this will break backwards compatibility.
-	peersPath                  = "raft/peers.json"
-	peersInfoPath              = "raft/peers.info"
-	retainSnapshotCount        = 1
-	applyTimeout               = 10 * time.Second
-	openTimeout                = 120 * time.Second
-	sqliteFile                 = "db.sqlite"
-	leaderWaitDelay            = 100 * time.Millisecond
-	appliedWaitDelay           = 100 * time.Millisecond
-	commitEquivalenceDelay     = 50 * time.Millisecond
-	appliedIndexUpdateInterval = 5 * time.Second
-	connectionPoolCount        = 5
-	connectionTimeout          = 10 * time.Second
-	raftLogCacheSize           = 512
-	trailingScale              = 1.25
-	observerChanLen            = 50
+	snapshotsDirName          = "rsnapshots"
+	restoreScratchPattern     = "rqlite-restore-*"
+	bootScatchPattern         = "rqlite-boot-*"
+	backupScratchPattern      = "rqlite-backup-*"
+	walSnapshotScratchPattern = "rqlite-wal-snapshot-*"
+	raftDBPath                = "raft.db" // Changing this will break backwards compatibility.
+	peersPath                 = "raft/peers.json"
+	peersInfoPath             = "raft/peers.info"
+	applyTimeout              = 10 * time.Second
+	sqliteFile                = "db.sqlite"
+	linearizableTimeout       = 1 * time.Second
+	leaderWaitDelay           = 100 * time.Millisecond
+	appliedWaitDelay          = 100 * time.Millisecond
+	commitEquivalenceDelay    = 50 * time.Millisecond
+	connectionPoolCount       = 5
+	connectionTimeout         = 10 * time.Second
+	raftLogCacheSize          = 512
+	trailingScale             = 1.25
+	observerChanLen           = 50
 
 	baseVacuumTimeKey   = "rqlite_base_vacuum"
 	lastVacuumTimeKey   = "rqlite_last_vacuum"
@@ -248,7 +255,7 @@ type SnapshotStore interface {
 	SetFullNeeded() error
 
 	// Stats returns stats about the Snapshot Store.
-	Stats() (map[string]interface{}, error)
+	Stats() (map[string]any, error)
 }
 
 // ClusterState defines the possible Raft states the current node can be in
@@ -287,6 +294,9 @@ type Store struct {
 	dbDrv *sql.Driver      // The SQLite database driver.
 	db    *sql.SwappableDB // The underlying SQLite store.
 
+	cdcMu       sync.RWMutex
+	cdcStreamer *sql.CDCStreamer // The CDC streamer for change data capture.
+
 	dechunkManager *chunking.DechunkerManager
 	cmdProc        *CommandProcessor
 
@@ -327,7 +337,7 @@ type Store struct {
 
 	// Raft changes observer
 	leaderObserversMu sync.RWMutex
-	leaderObservers   []chan<- struct{}
+	leaderObservers   []chan<- bool
 	observerClose     chan struct{}
 	observerDone      chan struct{}
 	observerChan      chan raft.Observation
@@ -409,7 +419,7 @@ func New(ly Layer, c *Config) *Store {
 		dbDir:           filepath.Dir(dbPath),
 		dbDrv:           sql.DefaultDriver(),
 		readyChans:      rsync.NewReadyChannels(),
-		leaderObservers: make([]chan<- struct{}, 0),
+		leaderObservers: make([]chan<- bool, 0),
 		reqMarshaller:   command.NewRequestMarshaler(),
 		logger:          logger,
 		notifyingNodes:  make(map[string]*Server),
@@ -659,17 +669,60 @@ func (s *Store) Bootstrap(servers ...*Server) error {
 	return fut.Error()
 }
 
+// getServerAddressByID returns the server address for the given server ID.
+// It returns an error if the server ID is not found in the cluster configuration.
+func (s *Store) getServerAddressByID(id string) (raft.ServerAddress, error) {
+	configFuture := s.raft.GetConfiguration()
+	if configFuture.Error() != nil {
+		return "", configFuture.Error()
+	}
+
+	config := configFuture.Configuration()
+	for _, server := range config.Servers {
+		if string(server.ID) == id {
+			return server.Address, nil
+		}
+	}
+
+	return "", ErrNodeNotFound
+}
+
 // Stepdown forces this node to relinquish leadership to another node in
 // the cluster. If this node is not the leader, and 'wait' is true, an error
-// will be returned.
-func (s *Store) Stepdown(wait bool) error {
+// will be returned. If id is non-empty, leadership will be transferred to the
+// node with the given ID.
+func (s *Store) Stepdown(wait bool, id string) error {
 	if !s.open.Is() {
 		return ErrNotOpen
 	}
-	f := s.raft.LeadershipTransfer()
+
+	if lid, err := s.LeaderID(); err == nil && lid == id {
+		return fmt.Errorf("cannot step down to the current Leader")
+	}
+
+	var f raft.Future
+	if id == "" {
+		// Transfer leadership to any available node
+		s.logger.Printf("stepping down from leadership, transferring to any available node")
+		f = s.raft.LeadershipTransfer()
+	} else {
+		// Transfer leadership to specific node
+		targetAddr, err := s.getServerAddressByID(id)
+		if err != nil {
+			return err
+		}
+
+		// Transfer leadership to the specific node
+		s.logger.Printf("stepping down from leadership, transferring to node %s at %s", id, targetAddr)
+		f = s.raft.LeadershipTransferToServer(raft.ServerID(id), targetAddr)
+	}
+
 	if !wait {
 		return nil
 	}
+	if f.Error() == raft.ErrNotLeader {
+		return ErrNotLeader
+	}
 	return f.Error()
 }
 
@@ -885,6 +938,22 @@ func (s *Store) ID() string {
 	return s.raftID
 }
 
+// Leader returns the current Raft leader, if one exists.
+func (s *Store) Leader() (*Server, error) {
+	if !s.open.Is() {
+		return nil, ErrNotOpen
+	}
+	addr, id := s.raft.LeaderWithID()
+	if addr == "" || id == "" {
+		return nil, ErrLeaderNotFound
+	}
+	return &Server{
+		ID:       string(id),
+		Addr:     string(addr),
+		Suffrage: raft.Voter.String(),
+	}, nil
+}
+
 // LeaderAddr returns the address of the current leader. Returns a
 // blank string if there is no leader or if the Store is not open.
 func (s *Store) LeaderAddr() (string, error) {
@@ -924,6 +993,42 @@ func (s *Store) HasLeaderID() bool {
 	return id != ""
 }
 
+// Followers returns the list of followers in the cluster, sorted by ID ascending.
+// This is a point-in-time snapshot of the followers, and may not reflect the
+// current state of the cluster if the cluster leadership subsequently changes
+// or changes while this function is running. It is useful for testing purposes
+// where leadership is not expected to change while the function is running.
+func (s *Store) Followers() ([]*Server, error) {
+	if !s.open.Is() {
+		return nil, ErrNotOpen
+	}
+
+	f := s.raft.GetConfiguration()
+	if f.Error() != nil {
+		return nil, f.Error()
+	}
+
+	_, id := s.raft.LeaderWithID()
+	if id == "" {
+		return nil, ErrLeaderNotFound
+	}
+
+	rs := f.Configuration().Servers
+	followers := make([]*Server, 0, len(rs)-1)
+	for i := range rs {
+		if rs[i].ID != raft.ServerID(id) && rs[i].Suffrage == raft.Voter {
+			followers = append(followers, &Server{
+				ID:       string(rs[i].ID),
+				Addr:     string(rs[i].Address),
+				Suffrage: rs[i].Suffrage.String(),
+			})
+		}
+	}
+
+	sort.Sort(Servers(followers))
+	return followers, nil
+}
+
 // CommitIndex returns the Raft commit index.
 func (s *Store) CommitIndex() (uint64, error) {
 	if !s.open.Is() {
@@ -1007,9 +1112,9 @@ func (s *Store) SetRequestCompression(batch, size int) {
 }
 
 // Stats returns stats for the store.
-func (s *Store) Stats() (map[string]interface{}, error) {
+func (s *Store) Stats() (map[string]any, error) {
 	if !s.open.Is() {
-		return map[string]interface{}{
+		return map[string]any{
 			"open": false,
 		}, nil
 	}
@@ -1027,7 +1132,7 @@ func (s *Store) Stats() (map[string]interface{}, error) {
 	leaderAddr, leaderID := s.LeaderWithID()
 
 	// Perform type-conversion to actual numbers where possible.
-	raftStats := make(map[string]interface{})
+	raftStats := make(map[string]any)
 	for k, v := range s.raft.Stats() {
 		if s, err := strconv.ParseInt(v, 10, 64); err != nil {
 			raftStats[k] = v
@@ -1051,7 +1156,7 @@ func (s *Store) Stats() (map[string]interface{}, error) {
 		return nil, err
 	}
 
-	status := map[string]interface{}{
+	status := map[string]any{
 		"open":             s.open,
 		"node_id":          s.raftID,
 		"raft":             raftStats,
@@ -1096,7 +1201,7 @@ func (s *Store) Stats() (map[string]interface{}, error) {
 			return nil, err
 		}
 
-		avm := map[string]interface{}{}
+		avm := map[string]any{}
 		if lvt, err := s.LastVacuumTime(); err == nil {
 			avm["last_vacuum"] = lvt
 			bt = lvt
@@ -1115,29 +1220,29 @@ func (s *Store) Stats() (map[string]interface{}, error) {
 }
 
 // Execute executes queries that return no rows, but do modify the database.
-func (s *Store) Execute(ex *proto.ExecuteRequest) ([]*proto.ExecuteQueryResponse, error) {
+func (s *Store) Execute(ex *proto.ExecuteRequest) ([]*proto.ExecuteQueryResponse, uint64, error) {
 	p := (*PragmaCheckRequest)(ex.Request)
 	if err := p.Check(); err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	if !s.open.Is() {
-		return nil, ErrNotOpen
+		return nil, 0, ErrNotOpen
 	}
 
 	if s.raft.State() != raft.Leader {
-		return nil, ErrNotLeader
+		return nil, 0, ErrNotLeader
 	}
 	if !s.Ready() {
-		return nil, ErrNotReady
+		return nil, 0, ErrNotReady
 	}
 	return s.execute(ex)
 }
 
-func (s *Store) execute(ex *proto.ExecuteRequest) ([]*proto.ExecuteQueryResponse, error) {
+func (s *Store) execute(ex *proto.ExecuteRequest) ([]*proto.ExecuteQueryResponse, uint64, error) {
 	b, compressed, err := s.tryCompress(ex)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	c := &proto.Command{
@@ -1148,39 +1253,39 @@ func (s *Store) execute(ex *proto.ExecuteRequest) ([]*proto.ExecuteQueryResponse
 
 	b, err = command.Marshal(c)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	af := s.raft.Apply(b, s.ApplyTimeout)
 	if af.Error() != nil {
 		if af.Error() == raft.ErrNotLeader {
-			return nil, ErrNotLeader
+			return nil, 0, ErrNotLeader
 		}
-		return nil, af.Error()
+		return nil, 0, af.Error()
 	}
 	r := af.Response().(*fsmExecuteQueryResponse)
-	return r.results, r.error
+	return r.results, af.Index(), r.error
 }
 
 // Query executes queries that return rows, and do not modify the database.
 // If the request read consistency level is LINEARIZABLE, that level may be
 // upgraded to STRONG if the Store determines that is necessary to guarantee
 // a linearizable read.
-func (s *Store) Query(qr *proto.QueryRequest) (rows []*proto.QueryRows, retErr error) {
+func (s *Store) Query(qr *proto.QueryRequest) (rows []*proto.QueryRows, raftIndex uint64, retErr error) {
 	p := (*PragmaCheckRequest)(qr.Request)
 	if err := p.Check(); err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	if !s.open.Is() {
-		return nil, ErrNotOpen
+		return nil, 0, ErrNotOpen
 	}
 
 	if qr.Level == proto.QueryRequest_QUERY_REQUEST_LEVEL_AUTO {
 		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_WEAK
 		isVoter, err := s.IsVoter()
 		if err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 		if !isVoter {
 			qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
@@ -1208,38 +1313,42 @@ func (s *Store) Query(qr *proto.QueryRequest) (rows []*proto.QueryRows, retErr e
 
 	if qr.Level == proto.QueryRequest_QUERY_REQUEST_LEVEL_LINEARIZABLE {
 		if s.raft.State() != raft.Leader {
-			return nil, ErrNotLeader
+			return nil, 0, ErrNotLeader
 		}
 		if !s.Ready() {
-			return nil, ErrNotReady
+			return nil, 0, ErrNotReady
 		}
 
 		// Implement the technique from the Raft dissertation, section
 		// 6.4 "Processing read-only queries more efficiently".
 		readIndex := s.raft.CommitIndex()
 		if err := s.VerifyLeader(); err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 		if s.raft.CurrentTerm() != readTerm {
-			return nil, ErrStaleRead
+			return nil, 0, ErrStaleRead
 		}
-		if _, err := s.WaitForFSMIndex(readIndex, time.Duration(qr.LinearizableTimeout)); err != nil {
-			return nil, err
+		lt := time.Duration(qr.LinearizableTimeout)
+		if lt == 0 {
+			lt = linearizableTimeout
+		}
+		if _, err := s.WaitForFSMIndex(readIndex, time.Duration(lt)); err != nil {
+			return nil, 0, err
 		}
 	}
 
 	if qr.Level == proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG {
 		if s.raft.State() != raft.Leader {
-			return nil, ErrNotLeader
+			return nil, 0, ErrNotLeader
 		}
 
 		if !s.Ready() {
-			return nil, ErrNotReady
+			return nil, 0, ErrNotReady
 		}
 
 		b, compressed, err := s.tryCompress(qr)
 		if err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 		c := &proto.Command{
 			Type:       proto.Command_COMMAND_TYPE_QUERY,
@@ -1249,29 +1358,30 @@ func (s *Store) Query(qr *proto.QueryRequest) (rows []*proto.QueryRows, retErr e
 
 		b, err = command.Marshal(c)
 		if err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 
 		af := s.raft.Apply(b, s.ApplyTimeout)
 		if af.Error() != nil {
 			if af.Error() == raft.ErrNotLeader || af.Error() == raft.ErrLeadershipLost {
-				return nil, ErrNotLeader
+				return nil, 0, ErrNotLeader
 			}
-			return nil, af.Error()
+			return nil, 0, af.Error()
 		}
-		s.strongReadTerm.Store(af.Index())
+		s.strongReadTerm.Store(readTerm)
 		r := af.Response().(*fsmQueryResponse)
-		return r.rows, r.error
+		return r.rows, af.Index(), r.error
 	}
 
 	if qr.Level == proto.QueryRequest_QUERY_REQUEST_LEVEL_WEAK && s.raft.State() != raft.Leader {
-		return nil, ErrNotLeader
+		return nil, 0, ErrNotLeader
 	}
 	if qr.Level == proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE && s.isStaleRead(qr.Freshness, qr.FreshnessStrict) {
-		return nil, ErrStaleRead
+		return nil, 0, ErrStaleRead
 	}
 
-	return s.db.Query(qr.Request, qr.Timings)
+	rows, err := s.db.Query(qr.Request, qr.Timings)
+	return rows, 0, err
 }
 
 // VerifyLeader checks that the current node is the Raft leader.
@@ -1299,14 +1409,14 @@ func (s *Store) VerifyLeader() (retErr error) {
 }
 
 // Request processes a request that may contain both Executes and Queries.
-func (s *Store) Request(eqr *proto.ExecuteQueryRequest) ([]*proto.ExecuteQueryResponse, error) {
+func (s *Store) Request(eqr *proto.ExecuteQueryRequest) ([]*proto.ExecuteQueryResponse, uint64, error) {
 	p := (*PragmaCheckRequest)(eqr.Request)
 	if err := p.Check(); err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	if !s.open.Is() {
-		return nil, ErrNotOpen
+		return nil, 0, ErrNotOpen
 	}
 	nRW, _ := s.RORWCount(eqr)
 	isLeader := s.raft.State() == raft.Leader
@@ -1324,29 +1434,29 @@ func (s *Store) Request(eqr *proto.ExecuteQueryRequest) ([]*proto.ExecuteQueryRe
 			return resp
 		}
 		if eqr.Level == proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE && s.isStaleRead(eqr.Freshness, eqr.FreshnessStrict) {
-			return nil, ErrStaleRead
+			return nil, 0, ErrStaleRead
 		} else if eqr.Level == proto.QueryRequest_QUERY_REQUEST_LEVEL_WEAK {
 			if !isLeader {
-				return nil, ErrNotLeader
+				return nil, 0, ErrNotLeader
 			}
 		}
 		qr, err := s.db.Query(eqr.Request, eqr.Timings)
-		return convertFn(qr), err
+		return convertFn(qr), 0, err
 	}
 
 	// At least one write in the request, or STRONG consistency requested, so
 	// we need to go through consensus. Check that we can do that.
 	if !isLeader {
-		return nil, ErrNotLeader
+		return nil, 0, ErrNotLeader
 	}
 	if !s.Ready() {
-		return nil, ErrNotReady
+		return nil, 0, ErrNotReady
 	}
 
 	// Send the request through consensus.
 	b, compressed, err := s.tryCompress(eqr)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	c := &proto.Command{
 		Type:       proto.Command_COMMAND_TYPE_EXECUTE_QUERY,
@@ -1355,18 +1465,18 @@ func (s *Store) Request(eqr *proto.ExecuteQueryRequest) ([]*proto.ExecuteQueryRe
 	}
 	b, err = command.Marshal(c)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	af := s.raft.Apply(b, s.ApplyTimeout)
 	if af.Error() != nil {
 		if af.Error() == raft.ErrNotLeader {
-			return nil, ErrNotLeader
+			return nil, 0, ErrNotLeader
 		}
-		return nil, af.Error()
+		return nil, 0, af.Error()
 	}
 	r := af.Response().(*fsmExecuteQueryResponse)
-	return r.results, r.error
+	return r.results, af.Index(), r.error
 }
 
 // Backup writes a consistent snapshot of the underlying database to dst. This
@@ -1471,7 +1581,49 @@ func (s *Store) Backup(br *proto.BackupRequest, dst io.Writer) (retErr error) {
 		}
 		return err
 	} else if br.Format == proto.BackupRequest_BACKUP_REQUEST_FORMAT_SQL {
-		return s.db.Dump(dst)
+		ww := dst
+		if br.Compress {
+			dstGz, err := gzip.NewWriterLevel(dst, gzip.BestSpeed)
+			if err != nil {
+				return err
+			}
+			defer dstGz.Close()
+			ww = dstGz
+		}
+		return s.db.Dump(ww, br.Tables...)
+	} else if br.Format == proto.BackupRequest_BACKUP_REQUEST_FORMAT_DELETE {
+		// Create a temporary database file in DELETE mode
+		tmpFD, err := createTemp(s.dbDir, backupScratchPattern)
+		if err != nil {
+			return err
+		}
+		defer os.Remove(tmpFD.Name())
+		defer tmpFD.Close()
+
+		// Copy the current database to the temporary file and convert to DELETE mode
+		if err := s.db.Backup(tmpFD.Name(), br.Vacuum); err != nil {
+			return err
+		}
+
+		// Re-open the temporary file for reading (to ensure all data is written)
+		tmpReadFD, err := os.Open(tmpFD.Name())
+		if err != nil {
+			return err
+		}
+		defer tmpReadFD.Close()
+
+		// Stream the DELETE mode database to the destination
+		if br.Compress {
+			dstGz, err := gzip.NewWriterLevel(dst, gzip.BestSpeed)
+			if err != nil {
+				return err
+			}
+			defer dstGz.Close()
+			_, err = io.Copy(dstGz, tmpReadFD)
+		} else {
+			_, err = io.Copy(dst, tmpReadFD)
+		}
+		return err
 	}
 	return ErrInvalidBackupFormat
 }
@@ -1613,6 +1765,53 @@ func (s *Store) Database(leader bool) ([]byte, error) {
 	return s.db.Serialize()
 }
 
+// EnableCDC enables Change Data Capture on this Store. Events will be streamed
+// to the provided channel. It is the caller's responsibility to ensure that the
+// channel is read from, as the CDCStreamer will drop events if the channel is full.
+// The Store must be open for this call to succeed.
+func (s *Store) EnableCDC(out chan<- *proto.CDCIndexedEventGroup, rowIDsOnly bool) error {
+	if !s.open.Is() {
+		return ErrNotOpen
+	}
+
+	s.cdcMu.Lock()
+	defer s.cdcMu.Unlock()
+	if s.cdcStreamer != nil {
+		return ErrCDCEnabled
+	}
+
+	s.cdcStreamer = sql.NewCDCStreamer(out)
+	if err := s.db.RegisterPreUpdateHook(s.cdcStreamer.PreupdateHook, rowIDsOnly); err != nil {
+		return err
+	}
+	if err := s.db.RegisterCommitHook(s.cdcStreamer.CommitHook); err != nil {
+		// Unregister preupdate hook if commit hook registration fails
+		s.db.RegisterPreUpdateHook(nil, false)
+		return err
+	}
+	return nil
+}
+
+// DisableCDC disables Change Data Capture on this Store.
+func (s *Store) DisableCDC() error {
+	s.cdcMu.Lock()
+	defer s.cdcMu.Unlock()
+	if s.cdcStreamer == nil {
+		return nil
+	}
+
+	if s.db != nil {
+		if err := s.db.RegisterPreUpdateHook(nil, false); err != nil {
+			return fmt.Errorf("failed to unregister preupdate hook: %w", err)
+		}
+		if err := s.db.RegisterCommitHook(nil); err != nil {
+			return fmt.Errorf("failed to unregister commit hook: %w", err)
+		}
+	}
+	s.cdcStreamer = nil
+	return nil
+}
+
 // Notify notifies this Store that a node is ready for bootstrapping at the
 // given address. Once the number of known nodes reaches the expected level
 // bootstrapping will be attempted using this Store. "Expected level" includes
@@ -1695,7 +1894,7 @@ func (s *Store) Join(jr *proto.JoinRequest) error {
 	// to incomplete DNS records across the underlying infrastructure. If it can't
 	// then don't consider this join attempt successful -- so the joining node
 	// will presumably try again.
-	if addr, err := resolvableAddress(addr); err != nil {
+	if _, err := resolvableAddress(addr); err != nil {
 		return fmt.Errorf("failed to resolve %s: %w", addr, err)
 	}
 
@@ -1928,7 +2127,7 @@ type fsmGenericResponse struct {
 }
 
 // fsmApply applies a Raft log entry to the database.
-func (s *Store) fsmApply(l *raft.Log) (e interface{}) {
+func (s *Store) fsmApply(l *raft.Log) (e any) {
 	defer func() {
 		s.fsmIdx.Store(l.Index)
 		s.fsmTarget.Signal(l.Index)
@@ -1942,7 +2141,16 @@ func (s *Store) fsmApply(l *raft.Log) (e interface{}) {
 		s.logger.Printf("first log applied since node start, log at index %d", l.Index)
 	}
 
-	cmd, mutated, r := s.cmdProc.Process(l.Data, s.db)
+	cmd, mutated, r := func() (*proto.Command, bool, any) {
+		// Reset CDC streamer with the current log index before processing if CDC is enabled
+		s.cdcMu.RLock()
+		defer s.cdcMu.RUnlock()
+		if s.cdcStreamer != nil {
+			s.cdcStreamer.Reset(l.Index)
+		}
+		return s.cmdProc.Process(l.Data, s.db)
+	}()
+
 	if mutated {
 		s.dbAppliedIdx.Store(l.Index)
 		s.appliedTarget.Signal(l.Index)
@@ -2206,8 +2414,9 @@ func (s *Store) DeregisterObserver(o *raft.Observer) {
 }
 
 // RegisterLeaderChange registers the given channel which will
-// receive a signal when this node detects that the Leader changes.
-func (s *Store) RegisterLeaderChange(c chan<- struct{}) {
+// receive a boolean when the leader changes. The boolean
+// indicates whether this node is the new leader or not.
+func (s *Store) RegisterLeaderChange(c chan<- bool) {
 	s.leaderObserversMu.Lock()
 	defer s.leaderObserversMu.Unlock()
 	s.leaderObservers = append(s.leaderObservers, c)
@@ -2256,23 +2465,25 @@ func (s *Store) observe() (closeCh, doneCh chan struct{}) {
 					}
 				case raft.LeaderObservation:
 					s.leaderObserversMu.RLock()
+					// check if this node is the leader
+					isLeader := signal.LeaderID == raft.ServerID(s.raftID)
 					for i := range s.leaderObservers {
 						select {
-						case s.leaderObservers[i] <- struct{}{}:
+						case s.leaderObservers[i] <- isLeader:
 							stats.Add(leaderChangesObserved, 1)
 						default:
 							stats.Add(leaderChangesDropped, 1)
 						}
 					}
 					s.leaderObserversMu.RUnlock()
-					s.selfLeaderChange(signal.LeaderID == raft.ServerID(s.raftID))
-					if signal.LeaderID == raft.ServerID(s.raftID) {
-						s.logger.Printf("this node (ID=%s) is now Leader", s.raftID)
+					s.selfLeaderChange(isLeader)
+					if isLeader {
+						s.logger.Printf("this node (ID=%s, addr=%s) is now Leader", s.raftID, s.raftTn.LocalAddr())
 					} else {
 						if signal.LeaderID == "" {
 							s.logger.Printf("Leader is now unknown")
 						} else {
-							s.logger.Printf("node %s is now Leader", signal.LeaderID)
+							s.logger.Printf("node %s at %s is now Leader", signal.LeaderID, signal.LeaderAddr)
 						}
 					}
 				}
