@@ -14,7 +14,64 @@ import (
 	"github.com/rqlite/rqlite/v8/db"
 )
 
-func Test_MultiNode_VerifyLeader(t *testing.T) {
+func Test_MultiNode_Leader_Followers(t *testing.T) {
+	s0, ln0 := mustNewStore(t)
+	defer ln0.Close()
+	if err := s0.Open(); err != nil {
+		t.Fatalf("failed to open single-node store: %s", err.Error())
+	}
+	defer s0.Close(true)
+	if err := s0.Bootstrap(NewServer(s0.ID(), s0.Addr(), true)); err != nil {
+		t.Fatalf("failed to bootstrap single-node store: %s", err.Error())
+	}
+	if _, err := s0.WaitForLeader(10 * time.Second); err != nil {
+		t.Fatalf("Error waiting for leader: %s", err)
+	}
+
+	s1, ln1 := mustNewStore(t)
+	defer ln1.Close()
+	if err := s1.Open(); err != nil {
+		t.Fatalf("failed to open single-node store: %s", err.Error())
+	}
+	defer s1.Close(true)
+	if err := s0.Join(joinRequest(s1.ID(), s1.Addr(), true)); err != nil {
+		t.Fatalf("failed to join single-node store: %s", err.Error())
+	}
+	if _, err := s1.WaitForLeader(10 * time.Second); err != nil {
+		t.Fatalf("Error waiting for leader: %s", err)
+	}
+
+	// Ensure the leader is the same on both nodes.
+	node0, err := s0.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader on single node: %s", err.Error())
+	}
+	node1, err := s1.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader on follower: %s", err.Error())
+	}
+	if !node0.Equal(node1) {
+		t.Fatalf("leader mismatch, got: %s, exp: %s", node1.ID, node0.ID)
+	}
+
+	// Check the followers returned by each node, and compare.
+	followers0, err := s0.Followers()
+	if err != nil {
+		t.Fatalf("failed to get followers on single node: %s", err.Error())
+	}
+	followers1, err := s1.Followers()
+	if err != nil {
+		t.Fatalf("failed to get followers on follower: %s", err.Error())
+	}
+	if len(followers0) != 1 || len(followers1) != 1 {
+		t.Fatalf("expected one follower on each node, got: %d, %d", len(followers0), len(followers1))
+	}
+	if !followers0[0].Equal(followers1[0]) {
+		t.Fatalf("followers mismatch, got: %s, exp: %s", followers1[0].ID, followers0[0].ID)
+	}
+}
+
+func Test_MultiNode_Leader_VerifyLeader(t *testing.T) {
 	s0, ln0 := mustNewStore(t)
 	defer ln0.Close()
 	if err := s0.Open(); err != nil {
@@ -44,6 +101,20 @@ func Test_MultiNode_VerifyLeader(t *testing.T) {
 		t.Fatalf("Error waiting for leader: %s", err)
 	}
 
+	// Ensure the leader is the same on both nodes.
+	node0, err := s0.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader on single node: %s", err.Error())
+	}
+	node1, err := s1.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader on follower: %s", err.Error())
+	}
+	if !node0.Equal(node1) {
+		t.Fatalf("leader mismatch, got: %s, exp: %s", node1.ID, node0.ID)
+	}
+
+	// Verify the leader on both nodes.
 	if err := s0.VerifyLeader(); err != nil {
 		t.Fatalf("failed to verify leader on leader: %s", err.Error())
 	}
@@ -62,6 +133,63 @@ func Test_MultiNode_VerifyLeader(t *testing.T) {
 	}
 }
 
+// Test_MultiNode_LeaderObservations tests that the leader change
+// notifications work as expected in a multi-node cluster.
+func Test_MultiNode_LeaderObservations(t *testing.T) {
+	var wg sync.WaitGroup
+
+	ch0 := make(chan bool)
+	wg.Add(1)
+	go func() {
+		defer wg.Done()
+		b := <-ch0
+		if !b {
+			t.Errorf("expected true on ch0, got false")
+		}
+	}()
+
+	ch1 := make(chan bool)
+	wg.Add(1)
+	go func() {
+		defer wg.Done()
+		b := <-ch1
+		if b {
+			t.Errorf("expected false on ch1, got false")
+		}
+	}()
+
+	s0, ln0 := mustNewStore(t)
+	defer ln0.Close()
+	s0.RegisterLeaderChange(ch0)
+	if err := s0.Open(); err != nil {
+		t.Fatalf("failed to open single-node store: %s", err.Error())
+	}
+	defer s0.Close(true)
+
+	s1, ln1 := mustNewStore(t)
+	defer ln1.Close()
+	s1.RegisterLeaderChange(ch1)
+	if err := s1.Open(); err != nil {
+		t.Fatalf("failed to open single-node store: %s", err.Error())
+	}
+	defer s1.Close(true)
+
+	if err := s0.Bootstrap(NewServer(s0.ID(), s0.Addr(), true)); err != nil {
+		t.Fatalf("failed to bootstrap single-node store: %s", err.Error())
+	}
+	if _, err := s0.WaitForLeader(10 * time.Second); err != nil {
+		t.Fatalf("Error waiting for leader: %s", err)
+	}
+	if err := s0.Join(joinRequest(s1.ID(), s1.Addr(), true)); err != nil {
+		t.Fatalf("failed to join single-node store: %s", err.Error())
+	}
+	if _, err := s1.WaitForLeader(10 * time.Second); err != nil {
+		t.Fatalf("Error waiting for leader: %s", err)
+	}
+
+	wg.Wait()
+}
+
 // Test_MultiNodeSimple tests that a the core operation of a multi-node
 // cluster works as expected. That is, with a two node cluster, writes
 // actually replicate, and reads are consistent.
@@ -100,7 +228,7 @@ func Test_MultiNodeSimple(t *testing.T) {
 		`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		`INSERT INTO foo(id, name) VALUES(1, "fiona")`,
 	}, false, false)
-	_, err := s0.Execute(er)
+	_, _, err := s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -139,7 +267,7 @@ func Test_MultiNodeSimple(t *testing.T) {
 		t.Helper()
 		qr := queryRequestFromString("SELECT * FROM foo", false, false)
 		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
-		r, err := s.Query(qr)
+		r, _, err := s.Query(qr)
 		if err != nil {
 			t.Fatalf("failed to query single node: %s", err.Error())
 		}
@@ -174,7 +302,7 @@ func Test_MultiNodeSimple(t *testing.T) {
 
 	// Write another row using Request
 	rr := executeQueryRequestFromString("INSERT INTO foo(id, name) VALUES(2, 'fiona')", proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG, false, false)
-	_, err = s0.Request(rr)
+	_, _, err = s0.Request(rr)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -188,7 +316,7 @@ func Test_MultiNodeSimple(t *testing.T) {
 		t.Helper()
 		qr := queryRequestFromString("SELECT COUNT(*) FROM foo", false, false)
 		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
-		r, err := s.Query(qr)
+		r, _, err := s.Query(qr)
 		if err != nil {
 			t.Fatalf("failed to query single node: %s", err.Error())
 		}
@@ -346,7 +474,7 @@ func Test_MultiNodeSnapshot_BlockedSnapshot(t *testing.T) {
 		t.Fatalf("Error waiting for leader: %s", err)
 	}
 	er := executeRequestFromString(`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`, false, false)
-	_, err := s0.Execute(er)
+	_, _, err := s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -359,7 +487,7 @@ func Test_MultiNodeSnapshot_BlockedSnapshot(t *testing.T) {
 			stmts = append(stmts, `INSERT INTO foo(name) VALUES("fiona")`)
 		}
 		er = executeRequestFromStrings(stmts, false, false)
-		_, err = s.Execute(er)
+		_, _, err = s.Execute(er)
 		if err != nil {
 			t.Fatalf("failed to execute on single node: %s", err.Error())
 		}
@@ -406,7 +534,7 @@ func Test_MultiNodeSnapshot_BlockedSnapshot(t *testing.T) {
 	// Should be two records in database.
 	qr := queryRequestFromString("SELECT COUNT(*) FROM foo", false, false)
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG
-	r, err := s0.Query(qr)
+	r, _, err := s0.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query single node: %s", err.Error())
 	}
@@ -483,7 +611,7 @@ func Test_MultiNodeDBAppliedIndex(t *testing.T) {
 		`INSERT INTO foo(id, name) VALUES(2, "fiona")`,
 		`INSERT INTO foo(id, name) VALUES(3, "fiona")`,
 	}, false, false)
-	_, err = s0.Execute(er)
+	_, _, err = s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -542,7 +670,7 @@ func Test_MultiNodeDBAppliedIndex(t *testing.T) {
 	er = executeRequestFromStrings([]string{
 		`INSERT INTO foo(id, name) VALUES(4, "fiona")`,
 	}, false, false)
-	_, err = s0.Execute(er)
+	_, _, err = s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -683,8 +811,13 @@ func Test_MultiNodeStepdown(t *testing.T) {
 		t.Fatalf("Error waiting for leader: %s", err)
 	}
 
+	// Telling a follower to stepdown should fail.
+	if err := s1.Stepdown(true, ""); err != ErrNotLeader {
+		t.Fatalf("expected ErrNotLeader when follower is told to step down")
+	}
+
 	// Tell leader to step down. After this finishes there should be a new Leader.
-	if err := s0.Stepdown(true); err != nil {
+	if err := s0.Stepdown(true, ""); err != nil {
 		t.Fatalf("leader failed to step down: %s", err.Error())
 	}
 
@@ -698,6 +831,110 @@ func Test_MultiNodeStepdown(t *testing.T) {
 	testPoll(t, check, 250*time.Millisecond, 10*time.Second)
 }
 
+func Test_MultiNodeStepdownTargetNode(t *testing.T) {
+	s0, ln0 := mustNewStore(t)
+	defer ln0.Close()
+	if err := s0.Open(); err != nil {
+		t.Fatalf("failed to open single-node store: %s", err.Error())
+	}
+	defer s0.Close(true)
+
+	s1, ln1 := mustNewStore(t)
+	defer ln1.Close()
+	if err := s1.Open(); err != nil {
+		t.Fatalf("failed to open single-node store: %s", err.Error())
+	}
+	defer s1.Close(true)
+
+	s2, ln2 := mustNewStore(t)
+	defer ln2.Close()
+	if err := s2.Open(); err != nil {
+		t.Fatalf("failed to open single-node store: %s", err.Error())
+	}
+	defer s2.Close(true)
+
+	// Join stores
+	if err := s0.Bootstrap(NewServer(s0.ID(), s0.Addr(), true)); err != nil {
+		t.Fatalf("failed to bootstrap single-node store: %s", err.Error())
+	}
+	if _, err := s0.WaitForLeader(10 * time.Second); err != nil {
+		t.Fatalf("Error waiting for leader: %s", err)
+	}
+
+	// Join the second node to the cluster
+	if err := s0.Join(joinRequest(s1.ID(), s1.Addr(), true)); err != nil {
+		t.Fatalf("failed to join to node at %s: %s", s0.Addr(), err.Error())
+	}
+	if _, err := s1.WaitForLeader(10 * time.Second); err != nil {
+		t.Fatalf("Error waiting for leader: %s", err)
+	}
+
+	// Join the third node to the cluster
+	if err := s0.Join(joinRequest(s2.ID(), s2.Addr(), true)); err != nil {
+		t.Fatalf("failed to join to node at %s: %s", s0.Addr(), err.Error())
+	}
+	if _, err := s2.WaitForLeader(10 * time.Second); err != nil {
+		t.Fatalf("Error waiting for leader: %s", err)
+	}
+	if f, err := s2.Noop("don't care"); err != nil && f.Error() != nil {
+		t.Fatalf("failed to noop on single node")
+	}
+
+	// Get the leader's commit index, and ensure all nodes have the same one.
+	// This should ensure every node has every log, and that each node can
+	// become leader.
+	leader, err := s0.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader address: %s", err.Error())
+	}
+	if leader.ID != s0.ID() {
+		t.Fatalf("expected leader to be s0, got %s", leader.ID)
+	}
+	idx, err := s0.LeaderCommitIndex()
+	if err != nil {
+		t.Fatalf("failed to get leader commit index: %s", err.Error())
+	}
+	if err := s1.WaitForCommitIndex(idx, 5*time.Second); err != nil {
+		t.Fatalf("failed to wait for commit index on follower: %s", err.Error())
+	}
+	if err := s2.WaitForCommitIndex(idx, 5*time.Second); err != nil {
+		t.Fatalf("failed to wait for commit index on follower: %s", err.Error())
+	}
+
+	// Test stepdown with invalid node ID
+	if err := s0.Stepdown(true, "nonexistent-node"); err != ErrNodeNotFound {
+		t.Fatalf("expected error when stepping down to nonexistent node")
+	}
+
+	// Test stepdown with leader ID, which should fail.
+	if err := s0.Stepdown(true, leader.ID); err == nil {
+		t.Fatalf("expected error when stepping down to Leader's own node ID")
+	}
+
+	// Test stepdown with specific target node, and verify it becomes leader.
+	followers, err := s0.Followers()
+	if err != nil {
+		t.Fatalf("failed to get followers: %s", err.Error())
+	}
+	if len(followers) != 2 {
+		t.Fatalf("expected two followers")
+	}
+	if err := s0.Stepdown(true, followers[0].ID); err != nil {
+		t.Fatalf("leader failed to step down to specific node: %s", err.Error())
+	}
+	check := func(failed bool) bool {
+		leader, err := s1.WaitForLeader(10 * time.Second)
+		if err != nil || leader != followers[0].Addr {
+			if failed {
+				t.Fatalf("stepdown didn't complete successfully, error: %v, leader: %s", err, leader)
+			}
+			return false
+		}
+		return true
+	}
+	testPollLog(t, check, 250*time.Millisecond, 10*time.Second)
+}
+
 func Test_MultiNodeStoreNotifyBootstrap(t *testing.T) {
 	s0, ln0 := mustNewStore(t)
 	defer ln0.Close()
@@ -811,7 +1048,7 @@ func Test_MultiNodeStoreAutoRestoreBootstrap(t *testing.T) {
 	}
 
 	qr := queryRequestFromString("SELECT * FROM foo WHERE id=2", false, true)
-	r, err := s0.Query(qr)
+	r, _, err := s0.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query single node: %s", err.Error())
 	}
@@ -957,7 +1194,7 @@ func Test_MultiNodeExecuteQuery(t *testing.T) {
 		`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		`INSERT INTO foo(id, name) VALUES(1, "fiona")`,
 	}, false, false)
-	_, err := s0.Execute(er)
+	_, _, err := s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -968,7 +1205,7 @@ func Test_MultiNodeExecuteQuery(t *testing.T) {
 
 	qr := queryRequestFromString("SELECT * FROM foo", false, false)
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
-	r, err := s0.Query(qr)
+	r, _, err := s0.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query leader node: %s", err.Error())
 	}
@@ -986,17 +1223,17 @@ func Test_MultiNodeExecuteQuery(t *testing.T) {
 	}
 
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_WEAK
-	_, err = s1.Query(qr)
+	_, _, err = s1.Query(qr)
 	if err == nil {
 		t.Fatalf("successfully queried non-leader node")
 	}
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG
-	_, err = s1.Query(qr)
+	_, _, err = s1.Query(qr)
 	if err == nil {
 		t.Fatalf("successfully queried non-leader node")
 	}
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
-	r, err = s1.Query(qr)
+	r, _, err = s1.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query follower node: %s", err.Error())
 	}
@@ -1014,17 +1251,17 @@ func Test_MultiNodeExecuteQuery(t *testing.T) {
 	}
 
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_WEAK
-	_, err = s1.Query(qr)
+	_, _, err = s1.Query(qr)
 	if err == nil {
 		t.Fatalf("successfully queried non-voting node with Weak")
 	}
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG
-	_, err = s1.Query(qr)
+	_, _, err = s1.Query(qr)
 	if err == nil {
 		t.Fatalf("successfully queried non-voting node with Strong")
 	}
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
-	r, err = s1.Query(qr)
+	r, _, err = s1.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query non-voting node: %s", err.Error())
 	}
@@ -1066,14 +1303,14 @@ func Test_MultiNodeExecuteQueryFreshness(t *testing.T) {
 		`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		`INSERT INTO foo(id, name) VALUES(1, "fiona")`,
 	}, false, false)
-	_, err := s0.Execute(er)
+	_, _, err := s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
 
 	qr := queryRequestFromString("SELECT * FROM foo", false, false)
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
-	r, err := s0.Query(qr)
+	r, _, err := s0.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query leader node: %s", err.Error())
 	}
@@ -1094,14 +1331,14 @@ func Test_MultiNodeExecuteQueryFreshness(t *testing.T) {
 	// is ignored in this case.
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_WEAK
 	qr.Freshness = mustParseDuration("1ns").Nanoseconds()
-	_, err = s0.Query(qr)
+	_, _, err = s0.Query(qr)
 	if err != nil {
 		t.Fatalf("Failed to ignore freshness if level is Weak: %s", err.Error())
 	}
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG
 	// "Strong" consistency queries with 1 nanosecond freshness should pass, because freshness
 	// is ignored in this case.
-	_, err = s0.Query(qr)
+	_, _, err = s0.Query(qr)
 	if err != nil {
 		t.Fatalf("Failed to ignore freshness if level is Strong: %s", err.Error())
 	}
@@ -1112,7 +1349,7 @@ func Test_MultiNodeExecuteQueryFreshness(t *testing.T) {
 	// "None" consistency queries should still work.
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
 	qr.Freshness = 0
-	r, err = s1.Query(qr)
+	r, _, err = s1.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query follower node: %s", err.Error())
 	}
@@ -1130,7 +1367,7 @@ func Test_MultiNodeExecuteQueryFreshness(t *testing.T) {
 	// one nanosecond *should* have passed since leader died (surely!).
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
 	qr.Freshness = mustParseDuration("1ns").Nanoseconds()
-	_, err = s1.Query(qr)
+	_, _, err = s1.Query(qr)
 	if err == nil {
 		t.Fatalf("freshness violating query didn't return an error")
 	}
@@ -1140,7 +1377,7 @@ func Test_MultiNodeExecuteQueryFreshness(t *testing.T) {
 
 	// Freshness of 0 is ignored.
 	qr.Freshness = 0
-	r, err = s1.Query(qr)
+	r, _, err = s1.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query follower node: %s", err.Error())
 	}
@@ -1154,7 +1391,7 @@ func Test_MultiNodeExecuteQueryFreshness(t *testing.T) {
 	// "None" consistency queries with 1 hour freshness should pass, because it should
 	// not be that long since the leader died.
 	qr.Freshness = mustParseDuration("1h").Nanoseconds()
-	r, err = s1.Query(qr)
+	r, _, err = s1.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query follower node: %s", err.Error())
 	}
@@ -1169,7 +1406,7 @@ func Test_MultiNodeExecuteQueryFreshness(t *testing.T) {
 	eqr := executeQueryRequestFromString("SELECT * FROM foo", proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE,
 		false, false)
 	eqr.Freshness = mustParseDuration("1ns").Nanoseconds()
-	_, err = s1.Request(eqr)
+	_, _, err = s1.Request(eqr)
 	if err == nil {
 		t.Fatalf("freshness violating request didn't return an error")
 	}
@@ -1177,7 +1414,7 @@ func Test_MultiNodeExecuteQueryFreshness(t *testing.T) {
 		t.Fatalf("freshness violating request returned wrong error: %s", err.Error())
 	}
 	eqr.Freshness = 0
-	eqresp, err := s1.Request(eqr)
+	eqresp, _, err := s1.Request(eqr)
 	if err != nil {
 		t.Fatalf("inactive freshness violating request returned an error")
 	}
@@ -1262,7 +1499,7 @@ func Test_MultiNodeStoreLogTruncation(t *testing.T) {
 		`INSERT INTO foo(id, name) VALUES(5, "fiona")`,
 	}
 	for i := range queries {
-		_, err := s0.Execute(executeRequestFromString(queries[i], false, false))
+		_, _, err := s0.Execute(executeRequestFromString(queries[i], false, false))
 		if err != nil {
 			t.Fatalf("failed to execute on single node: %s", err.Error())
 		}
@@ -1276,7 +1513,7 @@ func Test_MultiNodeStoreLogTruncation(t *testing.T) {
 
 	// Do one more execute, to ensure there is at least one log not snapshot.
 	// Without this, there is no guarantee fsmIndex will be set on s1.
-	_, err := s0.Execute(executeRequestFromString(`INSERT INTO foo(id, name) VALUES(6, "fiona")`, false, false))
+	_, _, err := s0.Execute(executeRequestFromString(`INSERT INTO foo(id, name) VALUES(6, "fiona")`, false, false))
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -1297,23 +1534,22 @@ func Test_MultiNodeStoreLogTruncation(t *testing.T) {
 	if _, err := s1.WaitForLeader(10 * time.Second); err != nil {
 		t.Fatalf("Error waiting for leader: %s", err)
 	}
-	// Wait until the log entries have been applied to the follower,
-	// and then query.
-	if err := s1.WaitForAppliedIndex(8, 5*time.Second); err != nil {
-		t.Fatalf("error waiting for follower to apply index: %s:", err.Error())
-	}
-	qr := queryRequestFromString("SELECT count(*) FROM foo", false, true)
-	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
-	r, err := s1.Query(qr)
-	if err != nil {
-		t.Fatalf("failed to query single node: %s", err.Error())
-	}
-	if exp, got := `["count(*)"]`, asJSON(r[0].Columns); exp != got {
-		t.Fatalf("unexpected results for query\nexp: %s\ngot: %s", exp, got)
-	}
-	if exp, got := `[[6]]`, asJSON(r[0].Values); exp != got {
-		t.Fatalf("unexpected results for query\nexp: %s\ngot: %s", exp, got)
-	}
+
+	testPoll(t, func() bool {
+		qr := queryRequestFromString("SELECT count(*) FROM foo", false, true)
+		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
+		r, _, err := s1.Query(qr)
+		if err != nil {
+			t.Fatalf("failed to query single node: %s", err.Error())
+		}
+		if exp, got := `["count(*)"]`, asJSON(r[0].Columns); exp != got {
+			return false
+		}
+		if exp, got := `[[6]]`, asJSON(r[0].Values); exp != got {
+			return false
+		}
+		return true
+	}, 100*time.Millisecond, 5*time.Second)
 }
 
 func Test_MultiNodeExecuteQuery_Linearizable_AllUp(t *testing.T) {
@@ -1359,7 +1595,7 @@ func Test_MultiNodeExecuteQuery_Linearizable_AllUp(t *testing.T) {
 		`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		`INSERT INTO foo(id, name) VALUES(1, "fiona")`,
 	}, false, false)
-	_, err := s0.Execute(er)
+	_, _, err := s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on leader: %s", err.Error())
 	}
@@ -1372,7 +1608,7 @@ func Test_MultiNodeExecuteQuery_Linearizable_AllUp(t *testing.T) {
 		}
 		qr := queryRequestFromString("SELECT * FROM foo", false, false)
 		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_LINEARIZABLE
-		r, err := s.Query(qr)
+		r, _, err := s.Query(qr)
 		if err != nil {
 			// if this node is not the leader, it will return an error
 			if !s.IsLeader() {
@@ -1442,7 +1678,7 @@ func Test_MultiNodeExecuteQuery_Linearizable_Quorum(t *testing.T) {
 		`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		`INSERT INTO foo(id, name) VALUES(1, "fiona")`,
 	}, false, false)
-	_, err := s0.Execute(er)
+	_, _, err := s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on leader: %s", err.Error())
 	}
@@ -1460,7 +1696,7 @@ func Test_MultiNodeExecuteQuery_Linearizable_Quorum(t *testing.T) {
 		}
 		qr := queryRequestFromString("SELECT * FROM foo", false, false)
 		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_LINEARIZABLE
-		r, err := s.Query(qr)
+		r, _, err := s.Query(qr)
 		if err != nil {
 			// if this node is not the leader, it will return an error
 			if !s.IsLeader() {
@@ -1530,7 +1766,7 @@ func Test_MultiNodeExecuteQuery_Linearizable_NoQuorum(t *testing.T) {
 		`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		`INSERT INTO foo(id, name) VALUES(1, "fiona")`,
 	}, false, false)
-	_, err := s0.Execute(er)
+	_, _, err := s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on leader: %s", err.Error())
 	}
@@ -1548,7 +1784,7 @@ func Test_MultiNodeExecuteQuery_Linearizable_NoQuorum(t *testing.T) {
 	qr := queryRequestFromString("SELECT * FROM foo", false, false)
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_LINEARIZABLE
 
-	_, err = s0.Query(qr)
+	_, _, err = s0.Query(qr)
 	if err == nil {
 		t.Fatalf("expected query to fail, but it did not")
 	}
@@ -1606,7 +1842,7 @@ func Test_MultiNodeExecuteQuery_Linearizable_Concurrent(t *testing.T) {
 		`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		`INSERT INTO foo(id, name) VALUES(1, "fiona")`,
 	}, false, false)
-	_, err := s0.Execute(er)
+	_, _, err := s0.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on leader: %s", err.Error())
 	}
@@ -1622,7 +1858,7 @@ func Test_MultiNodeExecuteQuery_Linearizable_Concurrent(t *testing.T) {
 			defer wg.Done()
 			qr := queryRequestFromString("SELECT * FROM foo", false, false)
 			qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_LINEARIZABLE
-			r, err := s0.Query(qr)
+			r, _, err := s0.Query(qr)
 			if err != nil {
 				t.Errorf("expected query to succeed, but it did not: %s", err.Error())
 			}
