@@ -8,7 +8,7 @@ import (
 
 	"github.com/rqlite/rqlite/v8/command/proto"
 	"github.com/rqlite/rqlite/v8/db"
-	"github.com/rqlite/rqlite/v8/rarchive"
+	"github.com/rqlite/rqlite/v8/internal/rarchive"
 )
 
 // Test_SingleNodeBackupBinary tests that requesting a binary-formatted
@@ -34,7 +34,7 @@ CREATE TABLE foo (id integer not null primary key, name text);
 INSERT INTO "foo" VALUES(1,'fiona');
 COMMIT;
 `
-	_, err := s.Execute(executeRequestFromString(dump, false, false))
+	_, _, err := s.Execute(executeRequestFromString(dump, false, false))
 	if err != nil {
 		t.Fatalf("failed to load simple dump: %s", err.Error())
 	}
@@ -49,7 +49,7 @@ COMMIT;
 		defer dstDB.Close()
 		qr := queryRequestFromString("SELECT * FROM foo", false, false)
 		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_NONE
-		r, err := s.Query(qr)
+		r, _, err := s.Query(qr)
 		if err != nil {
 			t.Fatalf("failed to query single node: %s", err.Error())
 		}
@@ -114,7 +114,7 @@ func Test_OpenStoreSingleNode_VacuumFullNeeded(t *testing.T) {
 	er := executeRequestFromStrings([]string{
 		`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 	}, false, false)
-	_, err := s.Execute(er)
+	_, _, err := s.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -188,7 +188,7 @@ func Test_SingleNodeExplicitVacuumOK(t *testing.T) {
 
 	doVacuum := func() {
 		er := executeRequestFromString(`VACUUM`, false, false)
-		_, err := s.Execute(er)
+		_, _, err := s.Execute(er)
 		if err != nil {
 			t.Fatalf("failed to execute on single node: %s", err.Error())
 		}
@@ -196,7 +196,7 @@ func Test_SingleNodeExplicitVacuumOK(t *testing.T) {
 	doQuery := func() {
 		qr := queryRequestFromString("SELECT COUNT(*) FROM foo", false, true)
 		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG
-		r, err := s.Query(qr)
+		r, _, err := s.Query(qr)
 		if err != nil {
 			t.Fatalf("failed to query single node: %s", err.Error())
 		}
@@ -218,12 +218,12 @@ func Test_SingleNodeExplicitVacuumOK(t *testing.T) {
 
 	er := executeRequestFromString(`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		false, false)
-	_, err := s.Execute(er)
+	_, _, err := s.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
 	for i := 0; i < 100; i++ {
-		_, err := s.Execute(executeRequestFromString(`INSERT INTO foo(name) VALUES("fiona")`, false, false))
+		_, _, err := s.Execute(executeRequestFromString(`INSERT INTO foo(name) VALUES("fiona")`, false, false))
 		if err != nil {
 			t.Fatalf("failed to execute INSERT on single node: %s", err.Error())
 		}
@@ -273,7 +273,7 @@ func Test_SingleNodeExplicitVacuumOK_Stress(t *testing.T) {
 
 	doVacuum := func() {
 		er := executeRequestFromString(`VACUUM`, false, false)
-		_, err := s.Execute(er)
+		_, _, err := s.Execute(er)
 		if err != nil {
 			t.Fatalf("failed to execute on single node: %s", err.Error())
 		}
@@ -281,7 +281,7 @@ func Test_SingleNodeExplicitVacuumOK_Stress(t *testing.T) {
 	doQuery := func() {
 		qr := queryRequestFromString("SELECT COUNT(*) FROM foo", false, true)
 		qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG
-		r, err := s.Query(qr)
+		r, _, err := s.Query(qr)
 		if err != nil {
 			t.Fatalf("failed to query single node: %s", err.Error())
 		}
@@ -304,7 +304,7 @@ func Test_SingleNodeExplicitVacuumOK_Stress(t *testing.T) {
 	// Create a table
 	er := executeRequestFromString(`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		false, false)
-	_, err := s.Execute(er)
+	_, _, err := s.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -315,7 +315,7 @@ func Test_SingleNodeExplicitVacuumOK_Stress(t *testing.T) {
 	insertFn := func() {
 		defer wg.Done()
 		for i := 0; i < 500; i++ {
-			_, err := s.Execute(executeRequestFromString(`INSERT INTO foo(name) VALUES("fiona")`, false, false))
+			_, _, err := s.Execute(executeRequestFromString(`INSERT INTO foo(name) VALUES("fiona")`, false, false))
 			if err != nil {
 				t.Errorf("failed to execute INSERT on single node: %s", err.Error())
 			}
@@ -374,12 +374,12 @@ func Test_SingleNode_SnapshotWithAutoVac(t *testing.T) {
 	// Create a table, and insert some data.
 	er := executeRequestFromString(`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		false, false)
-	_, err := s.Execute(er)
+	_, _, err := s.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
 	for i := 0; i < 100; i++ {
-		_, err := s.Execute(executeRequestFromString(`INSERT INTO foo(name) VALUES("fiona")`, false, false))
+		_, _, err := s.Execute(executeRequestFromString(`INSERT INTO foo(name) VALUES("fiona")`, false, false))
 		if err != nil {
 			t.Fatalf("failed to execute INSERT on single node: %s", err.Error())
 		}
@@ -424,7 +424,7 @@ func Test_SingleNode_SnapshotWithAutoVac(t *testing.T) {
 
 	// Query the data, make sure it looks good after all this.
 	qr := queryRequestFromString("SELECT COUNT(*) FROM foo", false, true)
-	r, err := s.Query(qr)
+	r, _, err := s.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query single node: %s", err.Error())
 	}
@@ -463,7 +463,7 @@ func Test_SingleNode_SnapshotWithAutoVac(t *testing.T) {
 	}
 
 	// Query the data again, make sure it still looks good after all this.
-	r, err = s.Query(qr)
+	r, _, err = s.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query single node: %s", err.Error())
 	}
@@ -493,7 +493,7 @@ func Test_SingleNode_SnapshotWithAutoVac_Stress(t *testing.T) {
 	// Create a table
 	er := executeRequestFromString(`CREATE TABLE foo (id INTEGER NOT NULL PRIMARY KEY, name TEXT)`,
 		false, false)
-	_, err := s.Execute(er)
+	_, _, err := s.Execute(er)
 	if err != nil {
 		t.Fatalf("failed to execute on single node: %s", err.Error())
 	}
@@ -504,7 +504,7 @@ func Test_SingleNode_SnapshotWithAutoVac_Stress(t *testing.T) {
 	insertFn := func() {
 		defer wg.Done()
 		for i := 0; i < 500; i++ {
-			_, err := s.Execute(executeRequestFromString(`INSERT INTO foo(name) VALUES("fiona")`, false, false))
+			_, _, err := s.Execute(executeRequestFromString(`INSERT INTO foo(name) VALUES("fiona")`, false, false))
 			if err != nil {
 				t.Errorf("failed to execute INSERT on single node: %s", err.Error())
 			}
@@ -521,7 +521,7 @@ func Test_SingleNode_SnapshotWithAutoVac_Stress(t *testing.T) {
 	// Query the data, make sure it looks good after all this.
 	qr := queryRequestFromString("SELECT COUNT(*) FROM foo", false, true)
 	qr.Level = proto.QueryRequest_QUERY_REQUEST_LEVEL_STRONG
-	r, err := s.Query(qr)
+	r, _, err := s.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query single node: %s", err.Error())
 	}
@@ -540,7 +540,7 @@ func Test_SingleNode_SnapshotWithAutoVac_Stress(t *testing.T) {
 	if _, err := s.WaitForLeader(10 * time.Second); err != nil {
 		t.Fatalf("Error waiting for leader: %s", err)
 	}
-	r, err = s.Query(qr)
+	r, _, err = s.Query(qr)
 	if err != nil {
 		t.Fatalf("failed to query single node: %s", err.Error())
 	}
