@@ -2,9 +2,11 @@ package system
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 	"net"
 	"os"
+	"path/filepath"
 	"sync"
 	"testing"
 	"time"
@@ -1386,7 +1388,7 @@ func Test_MultiNodeCluster_Backup(t *testing.T) {
 
 	backupFile := mustTempFile()
 	defer os.Remove(backupFile)
-	if err := followers[0].Backup(backupFile, false); err != nil {
+	if err := followers[0].Backup(backupFile, false, ""); err != nil {
 		t.Fatalf("failed to get backup from follower: %s", err.Error())
 	}
 
@@ -1409,6 +1411,86 @@ func Test_MultiNodeCluster_Backup(t *testing.T) {
 	}
 }
 
+func Test_MultiNodeCluster_Backup_SQL(t *testing.T) {
+	node1 := mustNewLeaderNode("leader1")
+	defer node1.Deprovision()
+
+	node2 := mustNewNode("node2", false)
+	defer node2.Deprovision()
+	if err := node2.Join(node1); err != nil {
+		t.Fatalf("node failed to join leader: %s", err.Error())
+	}
+	_, err := node2.WaitForLeader()
+	if err != nil {
+		t.Fatalf("failed waiting for leader: %s", err.Error())
+	}
+
+	// Create a table and write a record
+	if _, err := node1.Execute(`CREATE TABLE foo (name text)`); err != nil {
+		t.Fatalf("failed to create table: %s", err.Error())
+	}
+
+	// Get a backup from the leader via the follower.
+	c := Cluster{node1, node2}
+	followers, err := c.Followers()
+	if err != nil {
+		t.Fatalf("failed to get followers: %s", err.Error())
+	}
+
+	backupFile := mustTempFile()
+	defer os.Remove(backupFile)
+	if err := followers[0].Backup(backupFile, false, "sql"); err != nil {
+		t.Fatalf("failed to get backup from follower: %s", err.Error())
+	}
+
+	// Check the backup contents
+	sql, err := os.ReadFile(backupFile)
+	if err != nil {
+		t.Fatalf(`reading backup file failed: %s`, err.Error())
+	}
+	schema := `PRAGMA foreign_keys=OFF;
+BEGIN TRANSACTION;
+CREATE TABLE foo (name text);
+COMMIT;
+`
+	if exp, got := schema, string(sql); exp != got {
+		t.Fatalf(`contents of backup file are incorrect exp: %s, got %s`, exp, got)
+	}
+}
+
+func Test_MultiNodeCluster_FollowerLoad_SQL(t *testing.T) {
+	node1 := mustNewLeaderNode("leader1")
+	defer node1.Deprovision()
+
+	node2 := mustNewNode("node2", false)
+	defer node2.Deprovision()
+	if err := node2.Join(node1); err != nil {
+		t.Fatalf("node failed to join leader: %s", err.Error())
+	}
+	_, err := node2.WaitForLeader()
+	if err != nil {
+		t.Fatalf("failed waiting for leader: %s", err.Error())
+	}
+
+	// Get a follower, make sure Load works via it.
+	c := Cluster{node1, node2}
+	followers, err := c.Followers()
+	if err != nil {
+		t.Fatalf("failed to get followers: %s", err.Error())
+	}
+	if _, err := followers[0].Load(filepath.Join("testdata", "auto-restore.sql")); err != nil {
+		t.Fatalf("failed to load via follower: %s", err.Error())
+	}
+
+	r, err := node1.QueryStrongConsistency("SELECT * FROM foo WHERE id=2")
+	if err != nil {
+		t.Fatalf("failed to execute query: %s", err.Error())
+	}
+	if r != `{"results":[{"columns":["id","name"],"types":["integer","text"],"values":[[2,"fiona"]]}]}` {
+		t.Fatalf("test received wrong result got %s", r)
+	}
+}
+
 // Test_MultiNodeClusterWithNonVoter tests formation of a 4-node cluster, one of which is
 // a non-voter. This test also checks that if the Leader changes the non-voter is still in
 // the cluster and gets updates from the new leader.
@@ -2181,6 +2263,105 @@ func Test_MultiNodeCluster_Boot(t *testing.T) {
 	}
 }
 
-func sleepForSecond() {
-	time.Sleep(mustParseDuration("1s"))
+func Test_ClusterLeader_GET(t *testing.T) {
+	leader := mustNewLeaderNode("leader1")
+	defer leader.Deprovision()
+
+	follower := mustNewNode("follower1", false)
+	defer follower.Deprovision()
+
+	if err := follower.Join(leader); err != nil {
+		t.Fatalf("failed to join node: %s", err.Error())
+	}
+	if _, err := follower.WaitForLeader(); err != nil {
+		t.Fatalf("failed waiting for follower: %s", err.Error())
+	}
+
+	// Test leader endpoint on leader
+	leaderInfo, err := leader.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader info from leader: %s", err.Error())
+	}
+	var nodeL http.Node
+	if err := json.Unmarshal([]byte(leaderInfo), &nodeL); err != nil {
+		t.Fatalf("failed to parse leader response: %s", err.Error())
+	}
+	if nodeL.Addr == "" {
+		t.Fatalf("leader Addr is empty")
+	}
+	if nodeL.APIAddr == "" {
+		t.Fatalf("leader API Addr is empty")
+	}
+
+	// Test leader endpoint on follower - should return same leader info
+	followerInfo, err := follower.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader info from follower: %s", err.Error())
+	}
+	var nodeF http.Node
+	if err := json.Unmarshal([]byte(followerInfo), &nodeF); err != nil {
+		t.Fatalf("failed to parse follower response: %s", err.Error())
+	}
+	if nodeF.Addr == "" {
+		t.Fatalf("leader Addr is empty")
+	}
+	if nodeF.APIAddr == "" {
+		t.Fatalf("leader API Addr is empty")
+	}
+
+	if nodeL.Addr != nodeF.Addr {
+		t.Fatalf("leader Addr mismatch: expected %s, got %s", nodeL.Addr, nodeF.Addr)
+	}
+	if nodeL.APIAddr != nodeF.APIAddr {
+		t.Fatalf("leader API Addr mismatch: expected %s, got %s", nodeL.APIAddr, nodeF.APIAddr)
+	}
+}
+
+func Test_ClusterLeader_Stepdown(t *testing.T) {
+	leader := mustNewLeaderNode("leader1")
+	defer leader.Deprovision()
+
+	follower := mustNewNode("follower1", false)
+	defer follower.Deprovision()
+
+	if err := follower.Join(leader); err != nil {
+		t.Fatalf("failed to join node: %s", err.Error())
+	}
+	if _, err := follower.WaitForLeader(); err != nil {
+		t.Fatalf("failed waiting for follower: %s", err.Error())
+	}
+
+	// Wait for cluster to stabilize
+	if _, err := leader.WaitForLeader(); err != nil {
+		t.Fatalf("failed waiting for leader: %s", err.Error())
+	}
+
+	// Get leader info before stepdown
+	leaderInfoBefore, err := leader.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader info before stepdown: %s", err.Error())
+	}
+
+	var ldrNode http.Node
+	if err := json.Unmarshal([]byte(leaderInfoBefore), &ldrNode); err != nil {
+		t.Fatalf("failed to parse leader response: %s", err.Error())
+	}
+
+	// Test stepdown on leader with wait=true for maximum testing
+	err = leader.Stepdown(true)
+	if err != nil {
+		t.Fatalf("failed to trigger stepdown on leader: %s", err.Error())
+	}
+
+	testPoll(t, func() (bool, error) {
+		leaderInfoAfter, err := follower.Leader()
+		if err != nil {
+			return false, nil
+		}
+		var node http.Node
+		if err := json.Unmarshal([]byte(leaderInfoAfter), &node); err != nil {
+			t.Fatalf("failed to parse leader response: %s", err.Error())
+		}
+		return ldrNode.Addr != node.Addr, nil
+	}, time.Second, 10*time.Second)
 }
