@@ -262,6 +262,12 @@ class Node(object):
       return None
     return self.process.pid
 
+  def running(self):
+    """Return true if the process is running, false otherwise."""
+    if self.process is None:
+      return False
+    return self.process.poll() is None
+
   def db_path(self):
     return self.status()['store']['sqlite3']['path']
 
