@@ -19,6 +19,7 @@ import (
 	"testing"
 	"time"
 
+	"github.com/rqlite/rqlite/v8/cdc"
 	"github.com/rqlite/rqlite/v8/cluster"
 	"github.com/rqlite/rqlite/v8/command/encoding"
 	"github.com/rqlite/rqlite/v8/command/proto"
@@ -63,6 +64,8 @@ type Node struct {
 	Service      *httpd.Service
 	Cluster      *cluster.Service
 	Client       *cluster.Client
+	Mux          *tcp.Mux
+	CDC          *cdc.Service
 }
 
 // SameAs returns true if this node is the same as node o.
@@ -77,14 +80,24 @@ func (n *Node) Close(graceful bool) error {
 	}
 	n.Service.Close()
 	n.Cluster.Close()
+	if n.CDC != nil {
+		n.CDC.Stop()
+	}
 	return nil
 }
 
 // Deprovision shuts down and removes all resources associated with the node.
 func (n *Node) Deprovision() {
+	if n == nil {
+		return
+	}
+	if n.CDC != nil {
+		n.CDC.Stop()
+	}
 	n.Service.Close()
 	n.Store.Close(true)
 	n.Cluster.Close()
+	n.Mux.Close()
 	os.RemoveAll(n.Dir)
 }
 
@@ -114,8 +127,8 @@ func (n *Node) ExecuteMulti(stmts []string) (string, error) {
 }
 
 // ExecuteParameterized executes a single parameterized query against the node
-func (n *Node) ExecuteParameterized(stmt []interface{}) (string, error) {
-	m := make([][]interface{}, 1)
+func (n *Node) ExecuteParameterized(stmt []any) (string, error) {
+	m := make([][]any, 1)
 	m[0] = stmt
 
 	j, err := json.Marshal(m)
@@ -191,8 +204,8 @@ func (n *Node) QueryMulti(stmts []string) (string, error) {
 }
 
 // QueryParameterized run a single parameterized query against the node
-func (n *Node) QueryParameterized(stmt []interface{}) (string, error) {
-	m := make([][]interface{}, 1)
+func (n *Node) QueryParameterized(stmt []any) (string, error) {
+	m := make([][]any, 1)
 	m[0] = stmt
 
 	j, err := json.Marshal(m)
@@ -217,8 +230,8 @@ func (n *Node) RequestMulti(stmts []string) (string, error) {
 }
 
 // RequestMultiParameterized runs a single parameterized request against the node
-func (n *Node) RequestMultiParameterized(stmt []interface{}) (string, error) {
-	m := make([][]interface{}, 1)
+func (n *Node) RequestMultiParameterized(stmt []any) (string, error) {
+	m := make([][]any, 1)
 	m[0] = stmt
 
 	j, err := json.Marshal(m)
@@ -234,21 +247,30 @@ func (n *Node) Load(filename string) (string, error) {
 }
 
 // Backup backs up the node's database to the given file.
-func (n *Node) Backup(filename string, compress bool) error {
+func (n *Node) Backup(filename string, compress bool, format string) error {
 	v, _ := url.Parse("http://" + n.APIAddr + "/db/backup")
 	if compress {
 		q := v.Query()
 		q.Set("compress", "true")
 		v.RawQuery = q.Encode()
 	}
+	if format != "" {
+		q := v.Query()
+		q.Set("fmt", format)
+		v.RawQuery = q.Encode()
+	}
 	resp, err := http.Get(v.String())
 	if err != nil {
 		return err
 	}
-	if resp.StatusCode != 200 {
-		return fmt.Errorf("backup returned: %s", resp.Status)
-	}
 	defer resp.Body.Close()
+	if resp.StatusCode != 200 {
+		b, err := io.ReadAll(resp.Body)
+		if err != nil {
+			return fmt.Errorf("failed to read error response body: %s", err)
+		}
+		return fmt.Errorf("backup returned: %s, %s", resp.Status, string(b))
+	}
 	f, err := os.Create(filename)
 	if err != nil {
 		return err
@@ -258,7 +280,7 @@ func (n *Node) Backup(filename string, compress bool) error {
 	return err
 }
 
-// Load loads a SQLite database file into the node.
+// Boot boots a node using a SQLite database file.
 func (n *Node) Boot(filename string) (string, error) {
 	return n.postFile("/boot", filename)
 }
@@ -274,7 +296,7 @@ func (n *Node) Noop(id string) error {
 
 // EnableTLSClient enables TLS support for the node's cluster client.
 func (n *Node) EnableTLSClient() {
-	tlsConfig := mustCreateTLSConfig(n.NodeCertPath, n.NodeKeyPath, "")
+	tlsConfig := mustCreateClientTLSConfig(n.NodeCertPath, n.NodeKeyPath, "")
 	clusterDialer := tcp.NewDialer(cluster.MuxClusterHeader, tlsConfig)
 	clusterClient := cluster.NewClient(clusterDialer, 30*time.Second)
 	n.Client = clusterClient
@@ -349,6 +371,52 @@ func (n *Node) Status() (string, error) {
 	return string(body), nil
 }
 
+// Leader returns the leader information for the node.
+func (n *Node) Leader() (string, error) {
+	v, _ := url.Parse("http://" + n.APIAddr + "/leader")
+
+	resp, err := http.Get(v.String())
+	if err != nil {
+		return "", err
+	}
+	defer resp.Body.Close()
+	body, err := io.ReadAll(resp.Body)
+	if err != nil {
+		return "", fmt.Errorf("failed to read leader response: %w", err)
+	}
+	if resp.StatusCode != 200 {
+		return "", fmt.Errorf("leader endpoint returned: %s (%s)", resp.Status,
+			strings.TrimSuffix(string(body), "\n"))
+	}
+	return string(body), nil
+}
+
+// Stepdown triggers leader stepdown on the node.
+func (n *Node) Stepdown(wait bool) error {
+	urlStr := "http://" + n.APIAddr + "/leader"
+	if wait {
+		urlStr += "?wait"
+	}
+
+	req, err := http.NewRequest("POST", urlStr, nil)
+	if err != nil {
+		return err
+	}
+
+	resp, err := http.DefaultClient.Do(req)
+	if err != nil {
+		return err
+	}
+	defer resp.Body.Close()
+
+	if resp.StatusCode != 200 {
+		body, _ := io.ReadAll(resp.Body)
+		return fmt.Errorf("stepdown endpoint returned: %s (%s)", resp.Status,
+			strings.TrimSuffix(string(body), "\n"))
+	}
+	return nil
+}
+
 // IsVoter returns whether the node is a voter or not.
 func (n *Node) IsVoter() (bool, error) {
 	statusJSON, err := n.Status()
@@ -356,17 +424,17 @@ func (n *Node) IsVoter() (bool, error) {
 		return false, err
 	}
 	// Marshal the status into a JSON object
-	var status map[string]interface{}
+	var status map[string]any
 	err = json.Unmarshal([]byte(statusJSON), &status)
 	if err != nil {
 		return false, err
 	}
 
-	strStatus, ok := status["store"].(map[string]interface{})
+	strStatus, ok := status["store"].(map[string]any)
 	if !ok {
 		return false, fmt.Errorf("store status not found")
 	}
-	raftStatus, ok := strStatus["raft"].(map[string]interface{})
+	raftStatus, ok := strStatus["raft"].(map[string]any)
 	if !ok {
 		return false, fmt.Errorf("raft status not found")
 	}
@@ -702,8 +770,8 @@ func mustNewNodeEncrypted(id string, enableSingle, httpEncrypt, nodeEncrypt bool
 	var clstrDialer *tcp.Dialer
 	if nodeEncrypt {
 		mux = mustNewOpenTLSMux(rX509.CertExampleDotComFile(dir), rX509.KeyExampleDotComFile(dir), "")
-		raftDialer = tcp.NewDialer(cluster.MuxRaftHeader, mustCreateTLSConfig(rX509.CertExampleDotComFile(dir), rX509.KeyExampleDotComFile(dir), ""))
-		clstrDialer = tcp.NewDialer(cluster.MuxClusterHeader, mustCreateTLSConfig(rX509.CertExampleDotComFile(dir), rX509.KeyExampleDotComFile(dir), ""))
+		raftDialer = tcp.NewDialer(cluster.MuxRaftHeader, mustCreateClientTLSConfig(rX509.CertExampleDotComFile(dir), rX509.KeyExampleDotComFile(dir), ""))
+		clstrDialer = tcp.NewDialer(cluster.MuxClusterHeader, mustCreateClientTLSConfig(rX509.CertExampleDotComFile(dir), rX509.KeyExampleDotComFile(dir), ""))
 	} else {
 		mux, _ = mustNewOpenMux("")
 		raftDialer = tcp.NewDialer(cluster.MuxRaftHeader, nil)
@@ -725,8 +793,8 @@ func mustNodeEncrypted(id, dir string, enableSingle, httpEncrypt bool, mux *tcp.
 		NodeKeyPath:  nodeKeyPath,
 		HTTPCertPath: httpCertPath,
 		HTTPKeyPath:  httpKeyPath,
-		//TLSConfig:    mustCreateTLSConfig(nodeCertPath, nodeKeyPath, ""),
-		PeersPath: filepath.Join(dir, "raft/peers.json"),
+		PeersPath:    filepath.Join(dir, "raft/peers.json"),
+		Mux:          mux,
 	}
 
 	dbConf := store.NewDBConfig()
@@ -745,18 +813,6 @@ func mustNodeEncrypted(id, dir string, enableSingle, httpEncrypt bool, mux *tcp.
 	node.Store.SnapshotInterval = SnapshotInterval
 	node.Store.ElectionTimeout = ElectionTimeout
 
-	if err := node.Store.Open(); err != nil {
-		panic(fmt.Sprintf("failed to open store: %s", err.Error()))
-	}
-	if enableSingle {
-		if err := node.Store.Bootstrap(store.NewServer(node.Store.ID(), node.Store.Addr(), true)); err != nil {
-			panic(fmt.Sprintf("failed to bootstrap store: %s", err.Error()))
-		}
-	}
-
-	node.RaftAddr = node.Store.Addr()
-	node.ID = node.Store.ID()
-
 	credStr := mustNewMockCredentialStore()
 	clstr := cluster.New(mux.Listen(cluster.MuxClusterHeader), node.Store, node.Store, credStr)
 	if err := clstr.Open(); err != nil {
@@ -781,9 +837,21 @@ func mustNodeEncrypted(id, dir string, enableSingle, httpEncrypt bool, mux *tcp.
 	}
 	node.APIAddr = node.Service.Addr().String()
 
-	// Finally, set API address in Cluster service
+	// Set API address in Cluster service
 	clstr.SetAPIAddr(node.APIAddr)
 
+	if err := node.Store.Open(); err != nil {
+		panic(fmt.Sprintf("failed to open store: %s", err.Error()))
+	}
+	if enableSingle {
+		if err := node.Store.Bootstrap(store.NewServer(node.Store.ID(), node.Store.Addr(), true)); err != nil {
+			panic(fmt.Sprintf("failed to bootstrap store: %s", err.Error()))
+		}
+	}
+
+	node.RaftAddr = node.Store.Addr()
+	node.ID = node.Store.ID()
+
 	return node
 }
 
@@ -845,7 +913,7 @@ func mustNewOpenTLSMux(certFile, keyPath, addr string) *tcp.Mux {
 	}
 
 	var mux *tcp.Mux
-	mux, err = tcp.NewTLSMux(ln, nil, certFile, keyPath, "", true, false)
+	mux, err = tcp.NewTLSMux(ln, nil, certFile, keyPath)
 	if err != nil {
 		panic(fmt.Sprintf("failed to create node-to-node mux: %s", err.Error()))
 	}
@@ -885,7 +953,7 @@ func mustParseDuration(d string) time.Duration {
 	}
 }
 
-func asJSON(v interface{}) string {
+func asJSON(v any) string {
 	enc := encoding.Encoder{}
 	b, err := enc.JSONMarshal(v)
 	if err != nil {
@@ -895,7 +963,7 @@ func asJSON(v interface{}) string {
 }
 
 func isJSON(s string) bool {
-	var js map[string]interface{}
+	var js map[string]any
 	return json.Unmarshal([]byte(s), &js) == nil
 }
 
@@ -906,9 +974,9 @@ func mustWriteFile(path, contents string) {
 	}
 }
 
-// mustCreateTLSConfig returns a TLS config from the given cert, key and optionally
+// mustCreateClientTLSConfig returns a TLS config from the given cert, key and optionally
 // Certificate Authority cert. Config doesn't verify certs.
-func mustCreateTLSConfig(certFile, keyFile, caCertFile string) *tls.Config {
+func mustCreateClientTLSConfig(certFile, keyFile, caCertFile string) *tls.Config {
 	var err error
 	config := &tls.Config{
 		InsecureSkipVerify: true,
@@ -1117,3 +1185,7 @@ func testPoll(t *testing.T, f func() (bool, error), period time.Duration, timeou
 		}
 	}
 }
+
+func sleepForSecond() {
+	time.Sleep(mustParseDuration("1s"))
+}
