@@ -5,6 +5,7 @@ package system
 
 import (
 	"compress/gzip"
+	"encoding/json"
 	"fmt"
 	"io"
 	"os"
@@ -16,8 +17,9 @@ import (
 	"time"
 
 	"github.com/rqlite/rqlite/v8/cluster"
+	"github.com/rqlite/rqlite/v8/http"
 	httpd "github.com/rqlite/rqlite/v8/http"
-	"github.com/rqlite/rqlite/v8/random"
+	"github.com/rqlite/rqlite/v8/internal/random"
 	"github.com/rqlite/rqlite/v8/store"
 	"github.com/rqlite/rqlite/v8/tcp"
 )
@@ -498,25 +500,35 @@ func Test_SingleNodeParameterized(t *testing.T) {
 	defer node.Deprovision()
 
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 		execute  bool
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
 			expected: `{"results":[{}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
+			stmt:     []any{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
 			expected: `{"results":[{"last_insert_id":1,"rows_affected":1}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"SELECT * FROM foo WHERE NAME=?", "fiona"},
+			stmt:     []any{"SELECT * FROM foo WHERE NAME=?", "fiona"},
 			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"],"values":[[1,"fiona",20]]}]}`,
 			execute:  false,
 		},
+		{
+			stmt:     []any{"SELECT * FROM foo WHERE NAME IN (?, ?)", "fiona", "declan"},
+			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"],"values":[[1,"fiona",20]]}]}`,
+			execute:  false,
+		},
+		{
+			stmt:     []any{"SELECT * FROM foo WHERE NAME IN (?, ?)", "ann", "bob"},
+			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"]}]}`,
+			execute:  false,
+		},
 	}
 
 	for i, tt := range tests {
@@ -541,27 +553,27 @@ func Test_SingleNodeBlob_Parameterized(t *testing.T) {
 	defer node.Deprovision()
 
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 		execute  bool
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, data blob) STRICT"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, data blob) STRICT"},
 			expected: `{"results":[{}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(id, data) VALUES(?, ?)", 1, `x'deadbeef'`},
+			stmt:     []any{"INSERT INTO foo(id, data) VALUES(?, ?)", 1, `x'deadbeef'`},
 			expected: `{"results":[{"last_insert_id":1,"rows_affected":1}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(id, data) VALUES(?, ?)", 2, []int{222, 173, 190, 239}},
+			stmt:     []any{"INSERT INTO foo(id, data) VALUES(?, ?)", 2, []int{222, 173, 190, 239}},
 			expected: `{"results":[{"last_insert_id":2,"rows_affected":1}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"SELECT * FROM foo"},
+			stmt:     []any{"SELECT * FROM foo"},
 			expected: `{"results":[{"columns":["id","data"],"types":["integer","blob"],"values":[[1,"3q2+7w=="],[2,"3q2+7w=="]]}]}`,
 			execute:  false,
 		},
@@ -589,19 +601,19 @@ func Test_SingleNodeRequestParameterized(t *testing.T) {
 	defer node.Deprovision()
 
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
 			expected: `{"results":[{}]}`,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
+			stmt:     []any{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
 			expected: `{"results":[{"last_insert_id":1,"rows_affected":1}]}`,
 		},
 		{
-			stmt:     []interface{}{"SELECT * FROM foo WHERE NAME=?", "fiona"},
+			stmt:     []any{"SELECT * FROM foo WHERE NAME=?", "fiona"},
 			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"],"values":[[1,"fiona",20]]}]}`,
 		},
 	}
@@ -622,22 +634,22 @@ func Test_SingleNodeParameterizedNull(t *testing.T) {
 	defer node.Deprovision()
 
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 		execute  bool
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
 			expected: `{"results":[{}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(name, age) VALUES(?, ?)", "declan", nil},
+			stmt:     []any{"INSERT INTO foo(name, age) VALUES(?, ?)", "declan", nil},
 			expected: `{"results":[{"last_insert_id":1,"rows_affected":1}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"SELECT * FROM foo WHERE NAME=?", "declan"},
+			stmt:     []any{"SELECT * FROM foo WHERE NAME=?", "declan"},
 			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"],"values":[[1,"declan",null]]}]}`,
 			execute:  false,
 		},
@@ -665,22 +677,22 @@ func Test_SingleNodeParameterizedNamed(t *testing.T) {
 	defer node.Deprovision()
 
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 		execute  bool
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
 			expected: `{"results":[{}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
+			stmt:     []any{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
 			expected: `{"results":[{"last_insert_id":1,"rows_affected":1}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"SELECT * FROM foo WHERE NAME=:name", map[string]interface{}{"name": "fiona"}},
+			stmt:     []any{"SELECT * FROM foo WHERE NAME=:name", map[string]any{"name": "fiona"}},
 			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"],"values":[[1,"fiona",20]]}]}`,
 			execute:  false,
 		},
@@ -709,13 +721,13 @@ func Test_SingleNodeParameterizedNamedConstraints(t *testing.T) {
 	node := mustNewLeaderNode("leader1")
 	defer node.Deprovision()
 
-	_, err := node.ExecuteParameterized([]interface{}{"CREATE TABLE [TestTable] ([Id] int primary key, [Col1] int not null, [Col2] varchar(500), [Col3] int not null, [Col4] varchar(500))"})
+	_, err := node.ExecuteParameterized([]any{"CREATE TABLE [TestTable] ([Id] int primary key, [Col1] int not null, [Col2] varchar(500), [Col3] int not null, [Col4] varchar(500))"})
 	if err != nil {
 		t.Fatalf("failed to create table: %s", err.Error())
 	}
 
 	for i := 0; i < 100; i++ {
-		r, err := node.ExecuteParameterized([]interface{}{"INSERT into TestTable (Col1, Col2, Col3, Col4) values (:Val1, :Val2, :Val3, :Val4)", map[string]interface{}{"Val1": 1, "Val2": "foo", "Val3": 2, "Val4": nil}})
+		r, err := node.ExecuteParameterized([]any{"INSERT into TestTable (Col1, Col2, Col3, Col4) values (:Val1, :Val2, :Val3, :Val4)", map[string]any{"Val1": 1, "Val2": "foo", "Val3": 2, "Val4": nil}})
 		if err != nil {
 			t.Fatalf("failed to insert record on loop %d: %s", i, err.Error())
 		}
@@ -1109,33 +1121,39 @@ func Test_SingleNodeSQLInjection(t *testing.T) {
 }
 
 // Test_SingleNodeNoSQLInjection demonstrates that using the parameterized API protects
-// against SQL injection attacks.
+// against SQL injection attacks. After the attempted injection the table 'foo' should
+// remain in place.
 func Test_SingleNodeNoSQLInjection(t *testing.T) {
 	node := mustNewLeaderNode("leader1")
 	defer node.Deprovision()
 
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 		execute  bool
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, name text)"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, name text)"},
 			expected: `{"results":[{}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{`SELECT * FROM foo WHERE name="baz"`},
+			stmt:     []any{`SELECT * FROM foo WHERE name="baz"`},
 			expected: `{"results":[{"columns":["id","name"],"types":["integer","text"]}]}`,
 			execute:  false,
 		},
 		{
-			stmt:     []interface{}{`SELECT * FROM foo WHERE name=?`, `"baz";DROP TABLE FOO`},
+			stmt:     []any{`SELECT * FROM foo WHERE name=?`, `"baz";DROP TABLE FOO`},
 			expected: `{"results":[{"columns":["id","name"],"types":["integer","text"]}]}`,
 			execute:  false,
 		},
 		{
-			stmt:     []interface{}{`SELECT * FROM foo`},
+			stmt:     []any{`DELETE FROM foo WHERE name=?`, `"qux";DROP TABLE FOO`},
+			expected: `{"results":[{}]}`,
+			execute:  true,
+		},
+		{
+			stmt:     []any{`SELECT * FROM foo`},
 			expected: `{"results":[{"columns":["id","name"],"types":["integer","text"]}]}`,
 			execute:  false,
 		},
@@ -1187,29 +1205,17 @@ func Test_SingleNodeUpgrades_NoSnapshots(t *testing.T) {
 			t.Fatalf("node never became leader with %s data:", dir)
 		}
 
-		timer := time.NewTimer(5 * time.Second)
-		defer timer.Stop()
-		ticker := time.NewTicker(100 * time.Millisecond)
-		defer ticker.Stop()
-		testSuccess := make(chan struct{})
-
-		for {
-			select {
-			case <-testSuccess:
-				return
-			case <-timer.C:
-				t.Fatalf(`timeout waiting for correct results with %s data`, dir)
-			case <-ticker.C:
-				r, err := node.QueryNoneConsistency(`SELECT COUNT(*) FROM foo`)
-				if err != nil {
-					t.Fatalf("query failed with %s data: %s", dir, err)
-				}
-				expected := `{"results":[{"columns":["COUNT(*)"],"types":["integer"],"values":[[20]]}]}`
-				if r == expected {
-					close(testSuccess)
-				}
+		testPoll(t, func() (bool, error) {
+			r, err := node.QueryNoneConsistency(`SELECT COUNT(*) FROM foo`)
+			if err != nil {
+				return false, fmt.Errorf("query failed with %s data: %w", dir, err)
 			}
-		}
+			expected := `{"results":[{"columns":["COUNT(*)"],"types":["integer"],"values":[[20]]}]}`
+			if r == expected {
+				return true, nil
+			}
+			return false, nil
+		}, 100*time.Millisecond, 5*time.Second)
 	}
 
 	for _, version := range versions {
@@ -1247,29 +1253,17 @@ func Test_SingleNodeUpgrades_Snapshots(t *testing.T) {
 			t.Fatalf("node never became leader with %s data:", dir)
 		}
 
-		timer := time.NewTimer(5 * time.Second)
-		defer timer.Stop()
-		ticker := time.NewTicker(100 * time.Millisecond)
-		defer ticker.Stop()
-		testSuccess := make(chan struct{})
-
-		for {
-			select {
-			case <-testSuccess:
-				return
-			case <-timer.C:
-				t.Fatalf(`timeout waiting for correct results with %s data`, dir)
-			case <-ticker.C:
-				r, err := node.QueryNoneConsistency(`SELECT COUNT(*) FROM foo`)
-				if err != nil {
-					t.Fatalf("query failed with %s data: %s", dir, err)
-				}
-				expected := `{"results":[{"columns":["COUNT(*)"],"types":["integer"],"values":[[20]]}]}`
-				if r == expected {
-					close(testSuccess)
-				}
+		testPoll(t, func() (bool, error) {
+			r, err := node.QueryNoneConsistency(`SELECT COUNT(*) FROM foo`)
+			if err != nil {
+				return false, fmt.Errorf("query failed with %s data: %w", dir, err)
 			}
-		}
+			expected := `{"results":[{"columns":["COUNT(*)"],"types":["integer"],"values":[[20]]}]}`
+			if r == expected {
+				return true, nil
+			}
+			return false, nil
+		}, 100*time.Millisecond, 5*time.Second)
 	}
 
 	for _, version := range versions {
@@ -1279,7 +1273,7 @@ func Test_SingleNodeUpgrades_Snapshots(t *testing.T) {
 	}
 }
 
-func Test_SingleNodeBackup(t *testing.T) {
+func Test_SingleNodeBackup_Binary(t *testing.T) {
 	node := mustNewLeaderNode("leader1")
 	defer node.Deprovision()
 
@@ -1295,7 +1289,7 @@ func Test_SingleNodeBackup(t *testing.T) {
 
 	backup := mustTempFile()
 	defer os.Remove(backup)
-	if err := node.Backup(backup, false); err != nil {
+	if err := node.Backup(backup, false, ""); err != nil {
 		t.Fatalf(`backup failed: %s`, err.Error())
 	}
 
@@ -1315,7 +1309,7 @@ func Test_SingleNodeBackup(t *testing.T) {
 	}
 
 	// Get a compressed backup, test it again.
-	if err := node.Backup(backup, true); err != nil {
+	if err := node.Backup(backup, true, ""); err != nil {
 		t.Fatalf(`backup failed: %s`, err.Error())
 	}
 
@@ -1358,6 +1352,37 @@ func Test_SingleNodeBackup(t *testing.T) {
 	}
 }
 
+func Test_SingleNodeBackup_SQL(t *testing.T) {
+	node := mustNewLeaderNode("leader1")
+	defer node.Deprovision()
+
+	// create a table and insert a row
+	_, err := node.Execute(`CREATE TABLE foo (name text)`)
+	if err != nil {
+		t.Fatalf(`CREATE TABLE failed: %s`, err.Error())
+	}
+
+	backup := mustTempFile()
+	defer os.Remove(backup)
+	if err := node.Backup(backup, false, "sql"); err != nil {
+		t.Fatalf(`backup failed: %s`, err.Error())
+	}
+
+	// Check the backup contents
+	sql, err := os.ReadFile(backup)
+	if err != nil {
+		t.Fatalf(`reading backup file failed: %s`, err.Error())
+	}
+	schema := `PRAGMA foreign_keys=OFF;
+BEGIN TRANSACTION;
+CREATE TABLE foo (name text);
+COMMIT;
+`
+	if exp, got := schema, string(sql); exp != got {
+		t.Fatalf(`contents of backup file are incorrect exp: %s, got %s`, exp, got)
+	}
+}
+
 func Test_SingleNodeNodes(t *testing.T) {
 	node := mustNewLeaderNode("leader1")
 	defer node.Deprovision()
@@ -1479,22 +1504,22 @@ func Test_SingleNodeNoopReopen(t *testing.T) {
 
 	// Ensure node is fully functional after restart.
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 		execute  bool
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
 			expected: `{"results":[{}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
+			stmt:     []any{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
 			expected: `{"results":[{"last_insert_id":1,"rows_affected":1}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"SELECT * FROM foo WHERE NAME=?", "fiona"},
+			stmt:     []any{"SELECT * FROM foo WHERE NAME=?", "fiona"},
 			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"],"values":[[1,"fiona",20]]}]}`,
 			execute:  false,
 		},
@@ -1564,22 +1589,22 @@ func Test_SingleNodeNoopSnapReopen(t *testing.T) {
 
 	// Ensure node is fully functional after restart.
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 		execute  bool
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
 			expected: `{"results":[{}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
+			stmt:     []any{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
 			expected: `{"results":[{"last_insert_id":1,"rows_affected":1}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"SELECT * FROM foo WHERE NAME=?", "fiona"},
+			stmt:     []any{"SELECT * FROM foo WHERE NAME=?", "fiona"},
 			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"],"values":[[1,"fiona",20]]}]}`,
 			execute:  false,
 		},
@@ -1654,22 +1679,22 @@ func Test_SingleNodeNoopSnapLogsReopen(t *testing.T) {
 
 	// Ensure node is fully functional after restart.
 	tests := []struct {
-		stmt     []interface{}
+		stmt     []any
 		expected string
 		execute  bool
 	}{
 		{
-			stmt:     []interface{}{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
+			stmt:     []any{"CREATE TABLE foo (id integer not null primary key, name text, age integer)"},
 			expected: `{"results":[{}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
+			stmt:     []any{"INSERT INTO foo(name, age) VALUES(?, ?)", "fiona", 20},
 			expected: `{"results":[{"last_insert_id":1,"rows_affected":1}]}`,
 			execute:  true,
 		},
 		{
-			stmt:     []interface{}{"SELECT * FROM foo WHERE NAME=?", "fiona"},
+			stmt:     []any{"SELECT * FROM foo WHERE NAME=?", "fiona"},
 			expected: `{"results":[{"columns":["id","name","age"],"types":["integer","text","integer"],"values":[[1,"fiona",20]]}]}`,
 			execute:  false,
 		},
@@ -1702,10 +1727,10 @@ func Test_SingleNodeAutoRestore(t *testing.T) {
 	}
 	defer node.Deprovision()
 
-	mux, _ := mustNewOpenMux("")
-	go mux.Serve()
+	node.Mux, _ = mustNewOpenMux("")
+	go node.Mux.Serve()
 
-	raftLn := mux.Listen(cluster.MuxRaftHeader)
+	raftLn := node.Mux.Listen(cluster.MuxRaftHeader)
 	raftTn := tcp.NewLayer(raftLn, tcp.NewDialer(cluster.MuxRaftHeader, nil))
 	node.Store = store.New(raftTn, &store.Config{
 		DBConf: store.NewDBConfig(),
@@ -1728,7 +1753,7 @@ func Test_SingleNodeAutoRestore(t *testing.T) {
 	node.RaftAddr = node.Store.Addr()
 	node.ID = node.Store.ID()
 
-	clstr := cluster.New(mux.Listen(cluster.MuxClusterHeader), node.Store, node.Store, mustNewMockCredentialStore())
+	clstr := cluster.New(node.Mux.Listen(cluster.MuxClusterHeader), node.Store, node.Store, mustNewMockCredentialStore())
 	if err := clstr.Open(); err != nil {
 		t.Fatalf("failed to open Cluster service: %s", err.Error())
 	}
@@ -1786,3 +1811,64 @@ func Test_SingleNodeBoot_FailNotLeader(t *testing.T) {
 		t.Fatalf("expected error loading data")
 	}
 }
+
+func Test_SingleNodeLoad_OK(t *testing.T) {
+	node := mustNewLeaderNode("leader1")
+	defer node.Deprovision()
+
+	_, err := node.Load(filepath.Join("testdata", "auto-restore.sql"))
+	if err != nil {
+		t.Fatalf("failed to load data: %s", err.Error())
+	}
+
+	r, err := node.Query("SELECT * FROM foo WHERE id=2")
+	if err != nil {
+		t.Fatalf("failed to execute query: %s", err.Error())
+	}
+	if r != `{"results":[{"columns":["id","name"],"types":["integer","text"],"values":[[2,"fiona"]]}]}` {
+		t.Fatalf("test received wrong result got %s", r)
+	}
+}
+
+func Test_SingleNodeLeader_GET(t *testing.T) {
+	node := mustNewLeaderNode("leader1")
+	defer node.Deprovision()
+
+	// Test leader endpoint GET
+	leader, err := node.Leader()
+	if err != nil {
+		t.Fatalf("failed to get leader: %s", err.Error())
+	}
+
+	var server http.Node
+	if err := json.Unmarshal([]byte(leader), &server); err != nil {
+		t.Fatalf("failed to parse leader response: %s", err.Error())
+	}
+
+	// Check that we have addr and api_addr
+	if server.Addr == "" {
+		t.Fatalf("leader addr is empty")
+	}
+	if server.APIAddr == "" {
+		t.Fatalf("leader API dddr is empty")
+	}
+}
+
+func Test_SingleNodeLeader_Stepdown(t *testing.T) {
+	node := mustNewLeaderNode("leader1")
+	defer node.Deprovision()
+
+	// Test stepdown without wait - should succeed if not waiting for result
+	err := node.Stepdown(false)
+	if err != nil {
+		t.Fatalf("failed to trigger stepdown: %s", err.Error())
+	}
+	sleepForSecond() // give time for stepdown attempt to complete
+
+	// Test stepdown with wait - should fail in single node cluster as there's
+	// no peer to transfer to
+	err = node.Stepdown(true)
+	if err == nil {
+		t.Fatalf("expected stepdown with wait to fail in single-node cluster")
+	}
+}
